/*
 * $Id: //depot/prod/orion/sc_main/mintaka/spl/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/AddSmHostCommand.java#1 $
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.SmRegisterHostsRequest;
import com.netapp.smcore.plugin.contracts.SmRegisterHostsResponse;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.snapcenter.cli.command.parser.CustomStringToMapConverter;
import com.netapp.snapcenter.cli.command.parser.NoParameterSplitter;
import com.netapp.snapcenter.cli.contracts.OsType;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;
import com.netapp.spl.common.restclient.helper.PropertiesFileReader;
import com.netapp.spl.common.utils.HostInfoUtil;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.addsmhost.command.description")
public class AddSmHostCommand extends AbstractSMSCommand {

    private static final String PLUGIN_INFO_KEY_PLUGIN_CODE = "pluginCode";
    private static final String PLUGIN_INFO_KEY_ENDPOINT_URI = "endpointURI";

    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(AddSmHostCommand.class);

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_ADDSMHOST_COMMAND_NAME);

    @Parameter(names = { "-HostName" }, descriptionKey = "cli.addsmhost.command.hostName.option.description")
    private String hostName;

    @Parameter(names = { "-ColoHost" }, descriptionKey = "cli.addsmhost.command.coloHost.option.description", hidden = true)
    private boolean coloHost = true;

    @Parameter(names = { "-Port" }, descriptionKey = "cli.addsmhost.command.port.option.description")
    private Integer port;

    @Parameter(names = { "-HostIpAddresses" }, descriptionKey = "cli.addsmhost.command.hostIpAddresses.option.description", hidden = true)
    private List<String> hostIpAddresses = new ArrayList<String>();

    @Parameter(names = { "-OSType" }, descriptionKey = "cli.addsmhost.command.osType.option.description")
    private OsType osType = OsType.LINUX;

    @Parameter(names = { "-DiscoverPlugins" }, descriptionKey = "cli.addsmhost.command.discoverPlugins.option.description")
    private boolean discoverPlugins;

    @Parameter(names = { "-CredentialName" }, descriptionKey = "cli.addsmhost.command.runAsName.option.description")
    private String runAsName;

    @Parameter(names = { "-DoNotAddClusterNodes" }, descriptionKey = "cli.addsmhost.command.doNotAddClusterNodes.option.description")
    private boolean doNotAddClusterNodes;

    /*
     * TODO: this parameter is required for now since add-plugin command is
     * available
     * 
     * @Parameter(names = { "-PluginInfo" }, descriptionKey =
     * "cli.addsmhost.command.pluginInfo.option.description", converter =
     * CustomStringToMapConverter.class, splitter = NoParameterSplitter.class,
     * hidden = true)
     */
    private List<Map<String, String>> pluginInfo;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected void processCommandArguments() {

        if (pluginInfo != null) {
            validatePluginInfo(pluginInfo);
        }

        // If hostname not specified then use the localhost as default.
        if (getHostName() == null) {
            setHostName(HostInfoUtil.getCanonicalHostName());
            String displayUsingLocalhostDefault = MessageBundle
                    .localizeMessage(
                            CLIMsgKey.CLI_ADDSMHOST_USING_LOCALHOST_AS_DEFAULT,
                            getHostName());
            CLIUtils.writeToLogAndConsole(displayUsingLocalhostDefault, logger,
                    LogLevel.INFO);
        }
        /*
         * If port not specified then read and populate it from configuration
         * file.
         */
        if (getPort() == null) {
            setPort(Integer.parseInt(PropertiesFileReader.getInstance()
                    .getProperty(PropertiesFileReader.AGENT_CONFIG_SPL_PORT)));
            String displayUsingPortDefault = MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_ADDSMHOST_USING_PORT_AS_DEFAULT, getPort());
            CLIUtils.writeToLogAndConsole(displayUsingPortDefault, logger,
                    LogLevel.INFO);
        }
    }

    private void validatePluginInfo(List<Map<String, String>> pluginInfos) {
        Set<String> pluginInfoKeys = new HashSet<>();
        pluginInfoKeys.add(PLUGIN_INFO_KEY_PLUGIN_CODE);
        pluginInfoKeys.add(PLUGIN_INFO_KEY_ENDPOINT_URI);

        for (Map<String, String> pluginInfo : pluginInfos) {
            Set<String> keySet = pluginInfo.keySet();
            if (!keySet.containsAll(pluginInfoKeys)) {
                throw new IllegalArgumentException(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_ADDSMHOST_INVALID_PLUGIN_INFO_KEYS));
            }
        }
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmRegisterHostsRequest, SmRegisterHostsResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext
                .setRequest((SmRegisterHostsRequest) cmdRequest);
        smsRestInvocationContext
                .setResponseClass(SmRegisterHostsResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_REGISTER_HOST_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    public void setColoHost(boolean coloHost) {
        this.coloHost = coloHost;
    }

    public boolean getColoHost() {
        return coloHost;
    }

    public OsType getOsType() {
        return osType;
    }

    public void setOsType(OsType osType) {
        this.osType = osType;
    }

    public Integer getPort() {
        return port;
    }

    public void setPort(Integer port) {
        this.port = port;
    }

    public void setHostName(String hostName) {
        this.hostName = hostName;
    }

    public String getHostName() {
        return hostName;
    }

    public void setPluginInfo(List<Map<String, String>> pluginInfo) {
        this.pluginInfo = pluginInfo;
    }

    public List<Map<String, String>> getPluginInfo() {
        return pluginInfo;
    }

    public List<String> getHostIpAddresses() {
        return hostIpAddresses;
    }
    
    public void setHostIpAddresses(List<String> hostIpAddresses) {
        this.hostIpAddresses = hostIpAddresses;
    }

    public void setDiscoverPlugins(boolean discoverPlugins) {
        this.discoverPlugins = discoverPlugins;
    }

    public boolean isDiscoverPlugins() {
        return discoverPlugins;
    }

    public String getRunAsName() {
        return runAsName;
    }

    public void setRunAsName(String runAsName) {
        this.runAsName = runAsName;
    }

    public boolean isDoNotAddClusterNodes() {
        return doNotAddClusterNodes;
    }

    public void setDoNotAddClusterNodes(boolean doNotAddClusterNodes) {
        this.doNotAddClusterNodes = doNotAddClusterNodes;
    }
}
