/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/CatalogSmBackupWithOracleRMANCommand.java#4 $
 * 
 * Copyright (c) 2016 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.SmCatalogBackupRequest;
import com.netapp.smcore.plugin.contracts.SmCatalogBackupResponse;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.smcore.plugin.contracts.SmResponse;
import com.netapp.snapcenter.cli.command.parser.CommaParameterSplitter;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.catalogsmbackupwithoraclerman.command.description")
public class CatalogSmBackupWithOracleRMANCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_CATALOGSMBACKUPWITHORACLERMAN_COMMAND_NAME);

    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(CatalogSmBackupWithOracleRMANCommand.class);

    @Parameter(names = { "-BackupName" }, descriptionKey = "cli.catalogsmbackupwithoraclerman.command.backupName.option.description", splitter = CommaParameterSplitter.class)
    private String backupName;

    @Parameter(names = { "-BackupId" }, descriptionKey = "cli.catalogsmbackupwithoraclerman.command.backupId.option.description", splitter = CommaParameterSplitter.class)
    private String backupId;

    @Parameter(names = { "-WaitForCompletion" }, descriptionKey = "cli.catalogsmbackupwithoraclerman.command.waitForCompletion.option.description")
    private boolean waitForCompletion;

    private PluginCode pluginCode = PluginCode.SCO;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected void processCommandArguments() throws CLIException {
        if (StringUtils.isBlank(getBackupName())
                && StringUtils.isBlank(getBackupId())) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_CATALOGSMBACKUPWITHORACLERMAN_BACKUP_NAME_OR_ID_NOT_SPECIFIED));
        }

        if (StringUtils.isNotBlank(getBackupId())
                && StringUtils.isNotBlank(getBackupName())) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_CATALOGSMBACKUPWITHORACLERMAN_BOTH_BACKUP_NAME_AND_ID_SPECIFIED_ERROR));
        }

    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmCatalogBackupRequest, SmCatalogBackupResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext
                .setRequest((SmCatalogBackupRequest) cmdRequest);
        smsRestInvocationContext
                .setResponseClass(SmCatalogBackupResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_CATALOG_BACKUP_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    @Override
    protected <T> void pollForJobStatus(SmResponse smResponse)
            throws CLIException {

        doPollForJobStatus(smResponse, isWaitForCompletion());
    }

    public String getBackupId() {
        return backupId;
    }

    public void setBackupId(String backupId) {
        this.backupId = backupId;
    }

    public String getBackupName() {
        return backupName;
    }

    public void setBackupName(String backupName) {
        this.backupName = backupName;
    }

    public boolean isWaitForCompletion() {
        return waitForCompletion;
    }

    public void setWaitForCompletion(boolean waitForCompletion) {
        this.waitForCompletion = waitForCompletion;
    }

    public PluginCode getPluginCode() {
        return pluginCode;
    }

    public void setPluginCode(PluginCode pluginCode) {
        this.pluginCode = pluginCode;
    }
}
