/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/CloseSmConnectionCommand.java#3 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;
import com.netapp.snapcenter.cli.utils.TokenStoreUtil;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.closesmconnection.command.description")
public class CloseSmConnectionCommand extends AbstractSMSCommand {

    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(CloseSmConnectionCommand.class);

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_CLOSESMCONNECTION_COMMAND_NAME);

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    /**
     * In Close SmConnection we read the token from security Token Store and if
     * present we set the token to empty string to close the connection to
     * SnapCenter server
     */
    @Override
    public void run() throws CLIException {

        String token = TokenStoreUtil.readToken();
        if (token == null || token.isEmpty()) {
            CLIUtils.writeToLogAndConsole(MessageBundle
                    .localizeMessage(CLIMsgKey.CLI_SERVER_CONNECTION_NOTEXIST),
                    logger, LogLevel.ERROR);
        } else {
            // Overwrite the token with empty string to close connection
            TokenStoreUtil.writeToken(StringUtils.EMPTY);
            CLIUtils.writeToLogAndConsole(MessageBundle
                    .localizeMessage(CLIMsgKey.CLI_SERVER_CONNECTION_CLOSED),
                    logger, LogLevel.INFO);
        }
        CLIUtils.writeToLog(
                MessageBundle.localizeMessage(
                        CLIMsgKey.CLI_COMMAND_EXECUTION_SUCCESS,
                        this.getCommandName()), logger, LogLevel.INFO);
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        return null;
    }

}
