/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/ConfigureSmOracleDatabaseCommand.java#3 $
 *
 * Copyright (c) 2016 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import static com.netapp.snapcenter.cli.helper.SmOracleDatabaseConfigurationHelper.DATABASE_OR_ASM_DEFAULT_PORT;

import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.ArrayOfSmHost;
import com.netapp.smcore.plugin.contracts.SmAuth;
import com.netapp.smcore.plugin.contracts.SmAuthMode;
import com.netapp.smcore.plugin.contracts.SmGetRunAsAccountResponse;
import com.netapp.smcore.plugin.contracts.*;
import com.netapp.smcore.plugin.contracts.SmOracleDb;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.snapcenter.cli.command.parser.CommaParameterSplitter;
import com.netapp.snapcenter.cli.command.validation.SmsCommandArgumentsValidationUtil;
import com.netapp.snapcenter.cli.command.validation.SmsCommandRequestValidationHelper;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.helper.SmOracleDatabaseConfigurationHelper;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;
import com.netapp.snapcenter.cli.utils.ProgressBar;
import com.netapp.snapcenter.cli.utils.ProgressBarUtil;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.configuresmoracledatabase.command.description")
public class ConfigureSmOracleDatabaseCommand extends AbstractSMSCommand {

    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(ConfigureSmOracleDatabaseCommand.class);

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_COMMAND_NAME);

    private static final String NO_AUTHENTICATION = "NONE";
    private static final long NONE_RUN_AS_ID = 0L;

    @Parameter(names = { "-AppObjectId" }, descriptionKey = "cli.configuresmoracledatabase.command.appObjectId.option.description", required = true)
    private String appObjectId;

    @Parameter(names = { "-DatabaseCredentialName" }, descriptionKey = "cli.configuresmoracledatabase.command.databaseRunAsName.option.description")
    private String dbRunAsName;

    @Parameter(names = { "-DatabasePort" }, descriptionKey = "cli.configuresmoracledatabase.command.databasePort.option.description")
    private Integer dbPort;

    @Parameter(names = { "-ASMCredentialName" }, descriptionKey = "cli.configuresmoracledatabase.command.asmRunAsName.option.description")
    private String asmRunAsName;

    @Parameter(names = { "-ASMPort" }, descriptionKey = "cli.configuresmoracledatabase.command.asmPort.option.description")
    private Integer asmPort;

    @Parameter(names = { "-OracleRmanCatalogTnsName" }, descriptionKey = "cli.configuresmoracledatabase.command.oracleRmanCatalogTnsName.option.description")
    private String oracleRmanCatalogTnsName;

    @Parameter(names = { "-OracleRmanCatalogCredentialName" }, descriptionKey = "cli.configuresmoracledatabase.command.oracleRmanCatalogRunAsName.option.description")
    private String oracleRmanCatalogRunAsName;

    @Parameter(names = { "-OracleRACPreferredNodes" }, descriptionKey = "cli.configuresmoracledatabase.command.oracleRACPreferredNodes.option.description", splitter = CommaParameterSplitter.class)
    private List<String> oracleRacPreferredNodes = new ArrayList<String>();

    private Long dbRunAsId;

    private Long asmRunAsId;

    private Long oracleRmanCatalogRunAsId;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    public void run() throws CLIException {
        // initialize the command
        initialize();

        // validate the application object identifier syntax
        if (!SmsCommandArgumentsValidationUtil
                .isValidAppObjectId(getAppObjectId())) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_COMMAND_PARAMETER_APP_OBJECT_ID_NOT_VALID));
        }

        SmOracleDatabaseConfigurationHelper smOracleDatabaseConfigurationHelper = new SmOracleDatabaseConfigurationHelper(
                getSnapCenterServerServiceHelper(), getAppObjectId());

        SmOracleDb smOracleDb = validateAppObjectIdAndGetOracleDatabaseObject(smOracleDatabaseConfigurationHelper);

        /**
         * validate user entered inputs.
         */
        validateUserInput(smOracleDb, smOracleDatabaseConfigurationHelper);

        /**
         * Validate the DbAuth/AsmAuth/RmanCatalog auth entered by the user. This will call
         * SnapCenter which will invoke plug-in to validate the credentials
         * against oracle database.
         */
        validateRunAsCredentials(smOracleDatabaseConfigurationHelper,
                smOracleDb);

        /**
         * This will take care of configuring database and asm runas and ports.
         */
        configureOracleDatabase(smOracleDatabaseConfigurationHelper, smOracleDb);

        /**
         * Sets the Oracle RAC Nodes Preference for Backup Operation.
         */
        setOracleRacNodesPreferrence(smOracleDatabaseConfigurationHelper,
                smOracleDb);

        String executionSucess = MessageBundle.localizeMessage(
                CLIMsgKey.CLI_COMMAND_EXECUTION_SUCCESS, this.getCommandName());
        CLIUtils.writeToLogAndConsole(executionSucess, logger, LogLevel.INFO);

    }

    private SmOracleDb validateAppObjectIdAndGetOracleDatabaseObject(
            SmOracleDatabaseConfigurationHelper smOracleDatabaseConfigurationHelper)
            throws CLIException {
        SmOracleDb smOracleDb = null;
        ProgressBar progressBar = null;
        try {
            progressBar = ProgressBarUtil
                    .createAndStartProgressBar("Validating AppObject details ");
            smOracleDb = smOracleDatabaseConfigurationHelper
                    .getOracleDatabaseDetails();
            if (smOracleDb == null) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_INVALID_APP_OBJECT_ID_DETAILS,
                                        getAppObjectId()));
            }
        } catch (CLIException e) {
            throw e;
        } finally {
            ProgressBarUtil.stopProgressBar(progressBar);
        }
        return smOracleDb;
    }

    private void setOracleRacNodesPreferrence(
            SmOracleDatabaseConfigurationHelper smOracleDatabaseConfigurationHelper,
            SmOracleDb smOracleDb) throws CLIException {
        /*
         * Update preferred hosts.
         */
        if (smOracleDatabaseConfigurationHelper.isOracleRacDatabase(smOracleDb)
                && !getOracleRacPreferredNodes().isEmpty()) {
            ProgressBar progressBar = null;
            try {
                progressBar = ProgressBarUtil
                        .createAndStartProgressBar("Setting RAC nodes preferrence ");
                smOracleDatabaseConfigurationHelper
                        .setPreferredNodes(getOracleRacPreferredNodes());
            } catch (CLIException e) {
                throw e;
            } finally {
                ProgressBarUtil.stopProgressBar(progressBar);
            }
        }
    }

    private void configureOracleDatabase(
            SmOracleDatabaseConfigurationHelper smOracleDatabaseConfigurationHelper,
            SmOracleDb smOracleDb) throws CLIException {
        ProgressBar progressBar = null;
        try {
            progressBar = ProgressBarUtil
                    .createAndStartProgressBar("Configuring Oracle Database ");
            /*
             * Set or remove Authentication based on user input.
             */
            if (getDbRunAsId() != null) {
                if (getDbRunAsId() == NONE_RUN_AS_ID) {
                    smOracleDatabaseConfigurationHelper.removeAuth(
                            NONE_RUN_AS_ID, SmAuthMode.ORACLE_DATABASE);
                } else {
                    smOracleDatabaseConfigurationHelper.addAuth(getDbRunAsId(),
                            SmAuthMode.ORACLE_DATABASE);
                }
            }

            if (getAsmRunAsId() != null) {
                if (getAsmRunAsId() == NONE_RUN_AS_ID) {
                    smOracleDatabaseConfigurationHelper.removeAuth(
                            NONE_RUN_AS_ID, SmAuthMode.ORACLE_ASM);

                } else {
                    smOracleDatabaseConfigurationHelper.addAuth(
                            getAsmRunAsId(), SmAuthMode.ORACLE_ASM);

                }
            }

            if (getOracleRmanCatalogRunAsId() != null) {
                if (getOracleRmanCatalogRunAsId() == NONE_RUN_AS_ID) {
                    smOracleDatabaseConfigurationHelper.removeAuth(
                            NONE_RUN_AS_ID, SmAuthMode.ORACLE_RMAN_CATALOG);

                } else {
                    logger.debug("Configuring RMAN Catalog auth.." + getOracleRmanCatalogRunAsId());
                    smOracleDatabaseConfigurationHelper.addAuth(
                            getOracleRmanCatalogRunAsId(), SmAuthMode.ORACLE_RMAN_CATALOG);

                }
            }

            /*
             * Update port and tnsname if required.
             */
            boolean isPortChangeRequired = false;
            int dbPortToBeUpdated = smOracleDb.getDatabasePort();
            int asmPortToBeUpdated = smOracleDb.getAsmPort();
            String tnsNameToBeUpdated = smOracleDb.getRmanCatalogTnsName();
            boolean isTnsNameUpdateRequired = false;

            if (smOracleDb.getDatabasePort() == 0
                    && StringUtils.isNotBlank(getDbRunAsName())) {
                isPortChangeRequired = true;
                dbPortToBeUpdated = DATABASE_OR_ASM_DEFAULT_PORT;
            }
            if (getDbPort() != null
                    && getDbPort() != smOracleDb.getDatabasePort()) {
                isPortChangeRequired = true;
                dbPortToBeUpdated = getDbPort();
            }
            if (smOracleDb.getAsmPort() == 0
                    && StringUtils.isNotBlank(getAsmRunAsName())) {
                isPortChangeRequired = true;
                asmPortToBeUpdated = DATABASE_OR_ASM_DEFAULT_PORT;
            }
            if (getAsmPort() != null && getAsmPort() != smOracleDb.getAsmPort()) {
                isPortChangeRequired = true;
                asmPortToBeUpdated = getAsmPort();
            }
            if (getOracleRmanCatalogTnsName() != null && StringUtils.isNotBlank(getOracleRmanCatalogTnsName()) && !getOracleRmanCatalogTnsName().equals(smOracleDb.getRmanCatalogTnsName())) {
                isTnsNameUpdateRequired = true;
                tnsNameToBeUpdated = getOracleRmanCatalogTnsName();
            }

            if (isPortChangeRequired || isTnsNameUpdateRequired) {
                smOracleDatabaseConfigurationHelper.configurePortAndRmanCatalogTnsName(
                        dbPortToBeUpdated, asmPortToBeUpdated, tnsNameToBeUpdated);
            }
        } catch (CLIException e) {
            throw e;
        } finally {
            ProgressBarUtil.stopProgressBar(progressBar);
        }
    }

    private void validateRunAsCredentials(
            SmOracleDatabaseConfigurationHelper smOracleDatabaseConfigurationHelper,
            SmOracleDb smOracleDb) throws CLIException {
        int validateOnDbPort = getDbPort() == null ? (smOracleDb
                .getDatabasePort() == 0 ? DATABASE_OR_ASM_DEFAULT_PORT
                : smOracleDb.getDatabasePort()) : getDbPort();
        int validateOnAsmPort = getAsmPort() == null ? (smOracleDb.getAsmPort() == 0 ? DATABASE_OR_ASM_DEFAULT_PORT
                : smOracleDb.getAsmPort())
                : getAsmPort();
        String validateOnTnsName = getOracleRmanCatalogTnsName() == null ? smOracleDb.getRmanCatalogTnsName() : getOracleRmanCatalogTnsName();

        // Validate RMAN TNS name  - if run as id is specified other than NONE and TNS name is not there 
        if (getOracleRmanCatalogRunAsId() != null
                && getOracleRmanCatalogRunAsId() != NONE_RUN_AS_ID) {

            if (StringUtils.isBlank(getOracleRmanCatalogTnsName())
                    && StringUtils.isBlank(smOracleDb.getRmanCatalogTnsName())) {

                throw new CLIException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_INVALID_TNS_NAME,
                                        getOracleRmanCatalogTnsName()));
            }
        }

        if (getOracleRmanCatalogRunAsId() != null
                && getOracleRmanCatalogRunAsId() == NONE_RUN_AS_ID && !StringUtils.isBlank(getOracleRmanCatalogTnsName())) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(
                                    CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_TNS_NAME_WITH_INVALID_RUNAS,
                                    getOracleRmanCatalogTnsName()));
        }

        /**
         * Validation need to be done in two scenarios. 1. When user has
         * specified the run as to configure. 2. When user has to update the
         * port or tns name.
         */
        if ((getDbRunAsId() != null && getDbRunAsId() != NONE_RUN_AS_ID)
                || (getAsmRunAsId() != null && getAsmRunAsId() != NONE_RUN_AS_ID)
                || (getOracleRmanCatalogRunAsId() != null && getOracleRmanCatalogRunAsId() != NONE_RUN_AS_ID)) {

            doValidateDatabaseRmanCatalogAndAsmCredentials(
                    smOracleDatabaseConfigurationHelper, getDbRunAsId(),
                    validateOnDbPort, getAsmRunAsId(), validateOnAsmPort, getOracleRmanCatalogRunAsId(), validateOnTnsName);
        } else if ((getDbPort() != null && getDbPort() != smOracleDb
                .getDatabasePort())
                || (getAsmPort() != null && getAsmPort() != smOracleDb
                        .getAsmPort())
                || (getOracleRmanCatalogTnsName() != null && !getOracleRmanCatalogTnsName().equals(smOracleDb.getRmanCatalogTnsName()) )) {

            long dbRunAsIdForValidation = getRunAsIdFromAuthObject(
                    smOracleDb.getAuth(), SmAuthMode.ORACLE_DATABASE);
            long asmRunAsIdForValidation = getRunAsIdFromAuthObject(
                    smOracleDb.getASMAuth(), SmAuthMode.ORACLE_ASM);
            long rmanCatalogRunAsIdForValidation = getRunAsIdFromAuthObject(
                    smOracleDb.getRmanCatalogAuth(), SmAuthMode.ORACLE_RMAN_CATALOG);

            doValidateDatabaseRmanCatalogAndAsmCredentials(
                    smOracleDatabaseConfigurationHelper,
                    dbRunAsIdForValidation, validateOnDbPort,
                    asmRunAsIdForValidation, validateOnAsmPort,
                    rmanCatalogRunAsIdForValidation, validateOnTnsName
                    );
        }
    }

    private long getRunAsIdFromAuthObject(SmAuth auth, SmAuthMode smAuthMode) {
        long runAsId = NONE_RUN_AS_ID;
        if (auth != null && auth.getAuthMode() != null
                && auth.getAuthMode().equals(smAuthMode)) {
            runAsId = auth.getId();
        }
        return runAsId;
    }

    private void doValidateDatabaseRmanCatalogAndAsmCredentials(
            SmOracleDatabaseConfigurationHelper smOracleDatabaseConfigurationHelper,
            Long dbRunAsIdForValidation, int validateOnDbPort,
            Long asmRunAsIdForValidation, int validateOnAsmPort, Long rmanCatalogRunAsId, String tnsName) throws CLIException {
        ProgressBar progressBar = null;
        try {
            progressBar = ProgressBarUtil
                    .createAndStartProgressBar("Validating Run As Credentials ");

            ArrayOfSmObjectOperationResult arrayOfSmObjectOperationResult = smOracleDatabaseConfigurationHelper
                    .validateAuth(dbRunAsIdForValidation, validateOnDbPort,
                            asmRunAsIdForValidation, validateOnAsmPort, rmanCatalogRunAsId, tnsName);
            for (SmObjectOperationResult opResult : arrayOfSmObjectOperationResult
                    .getSmObjectOperationResult()) {
                if (!getSnapCenterServerServiceHelper()
                        .isSnapCenterRestOperationResponseResultSuccess(
                                opResult.getResult())) {
                    throw new CLIException(opResult.getResult().getMessage());
                }
            }
        } catch (CLIException e) {
            throw e;
        } finally {
            ProgressBarUtil.stopProgressBar(progressBar);
        }
    }

    private void validateUserInput(
            SmOracleDb smOracleDb,
            SmOracleDatabaseConfigurationHelper smOracleDatabaseConfigurationHelper)
            throws CLIException {

        ProgressBar progressBar = null;
        try {
            progressBar = ProgressBarUtil
                    .createAndStartProgressBar("Validating user input ");

            if (getAsmPort() == null && getDbPort() == null
                    && getDbRunAsName() == null && getAsmRunAsName() == null
                    && getOracleRacPreferredNodes().isEmpty()
                    && StringUtils.isBlank(getOracleRmanCatalogRunAsName())
                    && StringUtils.isBlank(getOracleRmanCatalogTnsName())) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_NOTHING_WAS_SPECIFIED_TO_CONFIGURE));
            }

            SmsCommandRequestValidationHelper smsCommandRequestValidationHelper = new SmsCommandRequestValidationHelper(
                    getSnapCenterServerServiceHelper());

            /**
             * Validate oracle database run as account existence and set the id
             * if exists.
             */
            checkForValidDbRunAsNameAndSetDbRunAsId(
                    smsCommandRequestValidationHelper, smOracleDb);

            /**
             * Validate oracle ASM run as account existence and set the id if
             * exists.
             */
            checkForValidAsmRunAsNameAndSetAsmRunAsId(
                    smsCommandRequestValidationHelper, smOracleDb);

            /**
             * Validate RMAN catalog database run as account existence and set the id
             * if exists.
             */
            checkForValidRmanCatalogRunAsNameAndSetRmanCatalogRunAsId(
                    smsCommandRequestValidationHelper, smOracleDb);

            /**
             * Validate Database and ASM listener port numbers
             */
            validateDatabaseAndAsmPortsAndRmanTnsName(smOracleDb);

            /**
             * Validate RAC database preferred nodes.
             */
            validateOracleRacDatabasePreferredNodes(
                    smOracleDatabaseConfigurationHelper, smOracleDb);

        } catch (CLIException e) {
            throw e;
        } finally {
            ProgressBarUtil.stopProgressBar(progressBar);
        }

    }

    private void validateOracleRacDatabasePreferredNodes(
            SmOracleDatabaseConfigurationHelper smOracleDatabaseConfigurationHelper,
            SmOracleDb smOracleDb) throws CLIException {
        if (!getOracleRacPreferredNodes().isEmpty()) {
            if (StringUtils.isBlank(smOracleDb.getRacName())) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_NOT_A_RAC_DATABASE_TO_SET_PREFERRED_NODES,
                                        getAppObjectId()));
            }
            Set<String> uniqueRacNodes = new LinkedHashSet<>();
            for (String host : getOracleRacPreferredNodes()) {
                if (StringUtils.isBlank(host)) {
                    throw new CLIException(
                            MessageBundle
                                    .localizeMessage(CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_NOT_A_RAC_DATABASE_TO_SET_PREFERRED_NODES));
                }
                uniqueRacNodes.add(host.toLowerCase());
            }

            ArrayOfSmHost arrayOfSmHost = smOracleDatabaseConfigurationHelper
                    .getOracleClusterDatabaseHostsList();
            List<String> racNodes = new ArrayList<String>();
            for (SmHost smHost : arrayOfSmHost.getSmHost()) {
                racNodes.add(smHost.getHostName().toLowerCase());
            }

            for (String node : uniqueRacNodes) {
                if (!racNodes.contains(node)) {
                    throw new CLIException(
                            MessageBundle
                                    .localizeMessage(
                                            CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_INVALID_RAC_DATABASE_NODE,
                                            node, getAppObjectId()));
                }
            }
            setOracleRacPreferredNodes(new ArrayList<>(uniqueRacNodes));
        }

    }

    private void validateDatabaseAndAsmPortsAndRmanTnsName(SmOracleDb smOracleDb)
            throws CLIException {

        if (getDbPort() != null) {
            if (!CLIUtils.isValidPortNumber(getDbPort())) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_INVALID_DB_PORT,
                                        getDbPort()));
            }

            /**
             * If port has to be changed or specified then either there should
             * be existing run as associated or user has specified run as. Check
             * for this condition and if doesn't match the error out.
             */
            boolean isDbAuthAlreadyAttached = isAuthenticationWithSpecifiedAuthModeConfigured(
                    smOracleDb.getAuth(), SmAuthMode.ORACLE_DATABASE);
            boolean isDbAuthSpecifiedByUser = StringUtils
                    .isNotBlank(getDbRunAsName())
                    && !getDbRunAsName().equalsIgnoreCase(NO_AUTHENTICATION);
            if (!isDbAuthAlreadyAttached && !isDbAuthSpecifiedByUser) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_DB_PORT_CANNOT_BE_UPDATED));
            }
        }

        if (getAsmPort() != null) {
            if (!CLIUtils.isValidPortNumber(getAsmPort())) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_INVALID_ASM_PORT,
                                        getAsmPort()));
            }
            /**
             * If port has to be changed or specified then either there should
             * be existing run as associated or user has specified run as. Check
             * for this condition and if doesn't match the error out.
             */
            boolean isAsmAuthAlreadyAttached = isAuthenticationWithSpecifiedAuthModeConfigured(
                    smOracleDb.getASMAuth(), SmAuthMode.ORACLE_ASM);
            boolean isAsmAuthSpecifiedByUser = StringUtils
                    .isNotBlank(getAsmRunAsName())
                    && !getAsmRunAsName().equalsIgnoreCase(NO_AUTHENTICATION);
            if (!isAsmAuthAlreadyAttached && !isAsmAuthSpecifiedByUser) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_ASM_PORT_CANNOT_BE_UPDATED));
            }

        }

        // validate RMAN TNS name
        if (getOracleRmanCatalogTnsName() != null) {
            if (!CLIUtils.isValidRmanTnsName(getOracleRmanCatalogTnsName())) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_INVALID_TNS_NAME,
                                        getOracleRmanCatalogTnsName()));
            }

            /**
             * If tnsName has to be changed or specified then either there should
             * be existing run as associated or user has specified run as. Check
             * for this condition and if doesn't match the error out.
             */
            boolean isRmanAuthAlreadyAttached = isAuthenticationWithSpecifiedAuthModeConfigured(
                    smOracleDb.getRmanCatalogAuth(), SmAuthMode.ORACLE_RMAN_CATALOG);
            boolean isRmanAuthSpecifiedByUser = StringUtils
                    .isNotBlank(getOracleRmanCatalogRunAsName())
                    && !getOracleRmanCatalogRunAsName().equalsIgnoreCase(NO_AUTHENTICATION);

            if (!isRmanAuthAlreadyAttached && !isRmanAuthSpecifiedByUser) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_TNS_NAME_CANNOT_BE_UPDATED));
            }

        }
    }

    private boolean isAuthenticationWithSpecifiedAuthModeConfigured(
            SmAuth auth, SmAuthMode smAuthMode) {
        if (auth != null && auth.getAuthMode() != null
                && auth.getAuthMode().equals(smAuthMode)
                && StringUtils.isNotBlank(auth.getRunAsName())) {
            return true;
        }
        return false;
    }

    private void checkForValidAsmRunAsNameAndSetAsmRunAsId(
            SmsCommandRequestValidationHelper smsCommandRequestValidationHelper,
            SmOracleDb smOracleDb) throws CLIException {
        if (getAsmRunAsName() != null) {
            if (StringUtils.isBlank(getAsmRunAsName())) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_ASM_RUNAS_EMPTY_ERROR));
            } else {
                SmAuth asmAuth = smOracleDb.getASMAuth();
                if (getAsmRunAsName().equalsIgnoreCase(NO_AUTHENTICATION)) {
                    if (asmAuth == null
                            || !asmAuth.getAuthMode().equals(
                                    SmAuthMode.ORACLE_ASM)) {
                        throw new CLIException(
                                MessageBundle
                                        .localizeMessage(CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_NO_ASM_AUTH_ATTAHCED_TO_REMOVE));
                    }
                    setAsmRunAsId(NONE_RUN_AS_ID);
                } else {
                    if (asmAuth != null
                            && asmAuth.getAuthMode().equals(
                                    SmAuthMode.ORACLE_ASM)
                            && StringUtils.isNotBlank(asmAuth.getRunAsName())
                            && asmAuth.getRunAsName().equalsIgnoreCase(
                                    getAsmRunAsName())) {
                        throw new CLIException(
                                MessageBundle
                                        .localizeMessage(
                                                CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_ASM_AUTH_ALREADY_ATTAHCED,
                                                getAsmRunAsName()));
                    } else {
                        SmGetRunAsAccountResponse resp = smsCommandRequestValidationHelper
                                .getRunAsAccountInformation(getAsmRunAsName());
                        if (resp.getAuths() == null
                                || resp.getAuths().getSmAuth().isEmpty()) {
                            throw new CLIException(
                                    MessageBundle
                                            .localizeMessage(
                                                    CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_RUNAS_ACCOUNT_DOESNT_EXISTS,
                                                    getAsmRunAsName()));
                        }
                        SmAuth smAuth = resp.getAuths().getSmAuth().iterator()
                                .next();
                        if (!smAuth.getAuthMode().equals(SmAuthMode.ORACLE_ASM)) {
                            throw new CLIException(
                                    MessageBundle
                                            .localizeMessage(
                                                    CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_INVALID_ORACLE_ASM_RUNAS_ACCOUNT,
                                                    getAsmRunAsName()));
                        } else {
                            setAsmRunAsId(smAuth.getId());
                        }
                    }
                }
            }
        }
    }

    private void checkForValidDbRunAsNameAndSetDbRunAsId(
            SmsCommandRequestValidationHelper smsCommandRequestValidationHelper,
            SmOracleDb smOracleDb) throws CLIException {
        if (getDbRunAsName() != null) {
            if (StringUtils.isBlank(getDbRunAsName())) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_DATABASE_RUNAS_EMPTY_ERROR));
            } else {
                SmAuth dbAuth = smOracleDb.getAuth();
                if (getDbRunAsName().equalsIgnoreCase(NO_AUTHENTICATION)) {
                    if (dbAuth == null
                            || !dbAuth.getAuthMode().equals(
                                    SmAuthMode.ORACLE_DATABASE)) {
                        throw new CLIException(
                                MessageBundle
                                        .localizeMessage(CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_NO_DB_AUTH_ATTAHCED_TO_REMOVE));
                    }
                    setDbRunAsId(NONE_RUN_AS_ID);
                } else {
                    if (dbAuth != null
                            && dbAuth.getAuthMode().equals(
                                    SmAuthMode.ORACLE_DATABASE)
                            && StringUtils.isNotBlank(dbAuth.getRunAsName())
                            && dbAuth.getRunAsName().equalsIgnoreCase(
                                    getDbRunAsName())) {
                        throw new CLIException(
                                MessageBundle
                                        .localizeMessage(
                                                CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_DB_AUTH_ALREADY_ATTAHCED,
                                                getDbRunAsName()));
                    } else {
                        SmGetRunAsAccountResponse resp = smsCommandRequestValidationHelper
                                .getRunAsAccountInformation(getDbRunAsName());
                        if (resp.getAuths() == null
                                || resp.getAuths().getSmAuth().isEmpty()) {
                            throw new CLIException(
                                    MessageBundle
                                            .localizeMessage(
                                                    CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_RUNAS_ACCOUNT_DOESNT_EXISTS,
                                                    getDbRunAsName()));
                        }
                        SmAuth smAuth = resp.getAuths().getSmAuth().iterator()
                                .next();
                        if (!smAuth.getAuthMode().equals(
                                SmAuthMode.ORACLE_DATABASE)) {
                            throw new CLIException(
                                    MessageBundle
                                            .localizeMessage(
                                                    CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_INVALID_ORACLE_DATABASE_RUNAS_ACCOUNT,
                                                    getDbRunAsName()));
                        } else {
                            setDbRunAsId(smAuth.getId());
                        }
                    }
                }
            }
        }
    }

    private void checkForValidRmanCatalogRunAsNameAndSetRmanCatalogRunAsId(
            SmsCommandRequestValidationHelper smsCommandRequestValidationHelper,
            SmOracleDb smOracleDb) throws CLIException {

        if (getOracleRmanCatalogRunAsName() == null) {
            return;
        }

        if (StringUtils.isBlank(getOracleRmanCatalogRunAsName())) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_RMAN_CATALOG_RUNAS_EMPTY_ERROR));
        } else {
            SmAuth rmanCatalogAuth = smOracleDb.getRmanCatalogAuth();
            if (getOracleRmanCatalogRunAsName().equalsIgnoreCase(NO_AUTHENTICATION)) {
                if (rmanCatalogAuth == null
                        || !rmanCatalogAuth.getAuthMode().equals(
                                SmAuthMode.ORACLE_RMAN_CATALOG)) {
                    throw new CLIException(
                            MessageBundle
                                    .localizeMessage(CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_NO_RMAN_CATALOG_AUTH_ATTAHCED_TO_REMOVE));
                }
                setOracleRmanCatalogRunAsId(NONE_RUN_AS_ID);
            } else {
                if (rmanCatalogAuth != null
                        && rmanCatalogAuth.getAuthMode().equals(
                                SmAuthMode.ORACLE_RMAN_CATALOG)
                        && StringUtils.isNotBlank(rmanCatalogAuth.getRunAsName())
                        && rmanCatalogAuth.getRunAsName().equalsIgnoreCase(
                                getOracleRmanCatalogRunAsName())) {
                    throw new CLIException(
                            MessageBundle
                                    .localizeMessage(
                                            CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_RMAN_CATALOG_AUTH_ALREADY_ATTAHCED,
                                            getOracleRmanCatalogRunAsName()));
                } else {
                    SmGetRunAsAccountResponse resp = smsCommandRequestValidationHelper
                            .getRunAsAccountInformation(getOracleRmanCatalogRunAsName());
                    if (resp.getAuths() == null
                            || resp.getAuths().getSmAuth().isEmpty()) {
                        throw new CLIException(
                                MessageBundle
                                        .localizeMessage(
                                                CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_RUNAS_ACCOUNT_DOESNT_EXISTS,
                                                getOracleRmanCatalogRunAsName()));
                    }
                    SmAuth smAuth = resp.getAuths().getSmAuth().iterator()
                            .next();
                    if (!smAuth.getAuthMode().equals(
                            SmAuthMode.ORACLE_RMAN_CATALOG)) {
                        throw new CLIException(
                                MessageBundle
                                        .localizeMessage(
                                                CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_INVALID_ORACLE_RMAN_CATALOG_RUNAS_ACCOUNT,
                                                getOracleRmanCatalogRunAsName()));
                    } else {
                        setOracleRmanCatalogRunAsId(smAuth.getId());
                    }
                }
            }
        }
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        return null;
    }

    public Long getDbRunAsId() {
        return dbRunAsId;
    }

    public void setDbRunAsId(Long dbRunAsId) {
        this.dbRunAsId = dbRunAsId;
    }

    public Integer getDbPort() {
        return dbPort;
    }

    public void setDbPort(Integer dbPort) {
        this.dbPort = dbPort;
    }

    public Long getAsmRunAsId() {
        return asmRunAsId;
    }

    public void setAsmRunAsId(Long asmRunAsId) {
        this.asmRunAsId = asmRunAsId;
    }

    public Integer getAsmPort() {
        return asmPort;
    }

    public void setAsmPort(Integer asmPort) {
        this.asmPort = asmPort;
    }

    public String getAppObjectId() {
        return appObjectId;
    }

    public void setAppObjectId(String appObjectId) {
        this.appObjectId = appObjectId;
    }

    public void setOracleRacPreferredNodes(List<String> oracleRacPreferredNodes) {
        this.oracleRacPreferredNodes = oracleRacPreferredNodes;
    }

    public List<String> getOracleRacPreferredNodes() {
        return oracleRacPreferredNodes;
    }

    public String getDbRunAsName() {
        return dbRunAsName;
    }

    public void setDbRunAsName(String dbRunAsName) {
        this.dbRunAsName = dbRunAsName;
    }

    public String getAsmRunAsName() {
        return asmRunAsName;
    }

    public void setAsmRunAsName(String asmRunAsName) {
        this.asmRunAsName = asmRunAsName;
    }

    public String getOracleRmanCatalogRunAsName() {
        return oracleRmanCatalogRunAsName;
    }

    public void setOracleRmanCatalogRunAsName(String oracleRmanCatalogRunAsName) {
        this.oracleRmanCatalogRunAsName = oracleRmanCatalogRunAsName;
    }

    public String getOracleRmanCatalogTnsName() {
        return oracleRmanCatalogTnsName;
    }

    public void setOracleRmanCatalogTnsName(String oracleRmanCatalogTnsName) {
        this.oracleRmanCatalogTnsName = oracleRmanCatalogTnsName;
    }

    public Long getOracleRmanCatalogRunAsId() {
        return oracleRmanCatalogRunAsId;
    }

    public void setOracleRmanCatalogRunAsId(Long oracleRmanCatalogRunAsId) {
        this.oracleRmanCatalogRunAsId = oracleRmanCatalogRunAsId;
    }

   
}