/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/CopySmPolicyCommand.java#1 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.plugin.contracts.SmCopyPolicyRequest;
import com.netapp.smcore.plugin.contracts.SmCopyPolicyResponse;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;
import com.netapp.snapcenter.cli.command.validation.PolicyCommandParametersValidationUtil;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.copysmpolicy.command.description")
public class CopySmPolicyCommand extends AbstractSMSCommand {

	
    private static final String COMMANDNAME = MessageBundle
        .localizeMessage(CLIMsgKey.CLI_COPYSMPOLICY_COMMAND_NAME);

    @Parameter(names = { "-PolicyName" }, descriptionKey = "cli.copysmpolicy.command.policyName.option.description", required = true)
    private String policyName;
    @Parameter(names = { "-NewPolicyName" }, descriptionKey = "cli.copysmpolicy.command.newPolicyName.option.description", required = true)
    private String newPolicyName;

    private boolean allowMultipleSchedules = true;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected void processCommandArguments() throws CLIException {
        if (StringUtils.isBlank(policyName)) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_COPYSMPOLICY_POLICY_NAME_EMPTY));
        }
        if (StringUtils.isBlank(newPolicyName)) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_COPYSMPOLICY_NEW_POLICY_NAME_EMPTY));
        }
        
        PolicyCommandParametersValidationUtil.isValidPolicyName(newPolicyName);
        
    }
    
    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext <SmCopyPolicyRequest, SmCopyPolicyResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext.setRequest((SmCopyPolicyRequest) cmdRequest);
        smsRestInvocationContext.setResponseClass(SmCopyPolicyResponse.class);
        smsRestInvocationContext.setServiceUrlPath(SMSRestConfiguration.SMS_COPY_POLICY_SERVICE_PATH);
        return smsRestInvocationContext;
    }
    
    
    
    public String getPolicyName() {
        return policyName;
    }
    public void setPolicyName(String policyName) {
        this.policyName = policyName;
    }
    public String getNewPolicyName() {
        return newPolicyName;
    }
    public void setNewPolicyName(String newPolicyName) {
        this.newPolicyName = newPolicyName;
    }

    public boolean isAllowMultipleSchedules() {
        return allowMultipleSchedules;
    }

    public void setAllowMultipleSchedules(boolean allowMultipleSchedules) {
        this.allowMultipleSchedules = allowMultipleSchedules;
    }
}
