/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/CreateSmOracleCloneSpecificationCommand.java#1 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import static com.netapp.snapcenter.cli.command.validation.OracleCloneSpecificationValidationUtil.SID_PATTERN;
import static com.netapp.snapcenter.cli.command.validation.SmsBackupValidationHelper.ORACLE_DATA_BACKUP_TYPE;
import static com.netapp.snapcenter.cli.utils.CLIUtils.DATE_FORMAT_WITH_MILLISECONDS;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.SmBackup;
import com.netapp.smcore.plugin.contracts.SmGetBackupDetailsRequest;
import com.netapp.smcore.plugin.contracts.SmGetBackupDetailsResponse;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.snapcenter.cli.clone.contracts.OracleCloneSpecification;
import com.netapp.snapcenter.cli.command.validation.GetRecentBackupInformationRequest;
import com.netapp.snapcenter.cli.command.validation.GetRecentBackupInformationResponse;
import com.netapp.snapcenter.cli.command.validation.SmsBackupValidationHelper;
import com.netapp.snapcenter.cli.command.validation.SmsCommandArgumentsValidationUtil;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.contracts.SecondaryStorageType;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.formatter.FormatterType;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;
import com.netapp.snapcenter.cli.utils.ProgressBar;
import com.netapp.snapcenter.cli.utils.ProgressBarUtil;
import com.netapp.snapcenter.cli.utils.SmoCloneCliUtil;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;


@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.createsmoracleclonespecification.command.description")
public class CreateSmOracleCloneSpecificationCommand extends AbstractSMSCommand {

    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(CreateSmOracleCloneSpecificationCommand.class);

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_CREATESMORACLECLONESPECIFICATION_COMMAND_NAME);

    @Parameter(names = { "-BackupName" }, descriptionKey = "cli.createsmoracleclonespecification.command.backupName.option.description")
    private String backupName;

    @Parameter(names = { "-CloneLastBackup" }, descriptionKey = "cli.createsmoracleclonespecification.command.cloneLastBackup.option.description")
    private Integer cloneLastBackup;

    @Parameter(names = { "-AppObjectId" }, descriptionKey = "cli.createsmoracleclonespecification.command.appObjectId.option.description", required = true)
    private String appObjectId;

    @Parameter(names = { "-CloneDatabaseSID" }, descriptionKey = "cli.createsmoracleclonespecification.command.cloneDatabaseSid.option.description", required = true)
    private String cloneDatabaseSid;

    @Parameter(names = { "-IncludeSecondaryDetails" }, descriptionKey = "cli.createsmoracleclonespecification.command.includeSecondaryDetails.option.description")
    private boolean includeSecondaryDetails;

    @Parameter(names = { "-SecondaryStorageType" }, descriptionKey = "cli.createsmoracleclonespecification.command.secondaryStorageType.option.description")
    private String secondaryStorageType;
    
	private PluginCode appPluginCode = PluginCode.SCO;

    private String backupId;

    private String dbName;

    private String hostOrClusterName;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmGetBackupDetailsRequest, SmGetBackupDetailsResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext
                .setRequest((SmGetBackupDetailsRequest) cmdRequest);
        smsRestInvocationContext
                .setResponseClass(SmGetBackupDetailsResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_GET_BACKUP_DETAILS_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    @Override
    protected void processCommandArguments() throws CLIException {
        SmsBackupValidationHelper smsBackupValidationHelper = new SmsBackupValidationHelper(
                getSnapCenterServerServiceHelper());

        if (StringUtils.isBlank(getBackupName()) && cloneLastBackup == null) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_CREATESMORACLECLONESPECIFICATION_BACKUP_INFORMATION_NOT_SPECIFIED));
        }

        if (StringUtils.isNotBlank(getBackupName()) && cloneLastBackup != null) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_CREATESMORACLECLONESPECIFICATION_MORE_THAN_ONE_BACKUP_INFORMATION_SPECIFIED));
        }

        if (cloneLastBackup != null && cloneLastBackup < 0) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_CREATESMORACLECLONESPECIFICATION_CLONE_LAST_BACKUP_VALUE_CANNOT_BE_NEGATIVE));
        }

        if (!SmsCommandArgumentsValidationUtil.isValidAppObjectId(appObjectId)) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_COMMAND_PARAMETER_APP_OBJECT_ID_NOT_VALID));
        } else {
            String[] appObjectSplit = StringUtils.split(appObjectId, "\\");
            setHostOrClusterName(appObjectSplit[0]);
            setDbName(appObjectSplit[1]);
        }

        if (StringUtils.isBlank(getCloneDatabaseSid())) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_CREATESMORACLECLONESPECIFICATION_CLONE_DATABASE_SID_CANNOT_BE_EMPTY));
        } else if (!SID_PATTERN.matcher(getCloneDatabaseSid()).matches()){
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_CREATESMORACLECLONESPECIFICATION_CLONE_DATABASE_SID_INVALID));            
        } else if (getCloneDatabaseSid().length() > 8) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_CREATESMORACLECLONESPECIFICATION_CLONE_DATABASE_SID_IS_MORE_THAN_EIGHT_CHARACTERS));
        }

        if (getDbName().equals(getCloneDatabaseSid())) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_CREATESMORACLECLONESPECIFICATION_CLONE_DATABASE_SID_CANNOT_BE_SAME_AS_SOURCE_DB));
        }
        
        if (!StringUtils.isBlank(getSecondaryStorageType())) {
        	if (!isIncludeSecondaryDetails()) {
	            throw new CLIException(
	                    MessageBundle
	                            .localizeMessage(CLIMsgKey.CLI_CREATESMORACLECLONESPECIFICATION_INCLUDESECONDARYDETAILS_NOT_PASSED));
        	}
        	String storageType = getSecondaryStorageType();
			if (!(storageType.equalsIgnoreCase(SecondaryStorageType.MIRROR
					.toString()) || storageType
					.equalsIgnoreCase(SecondaryStorageType.VAULT.toString()))) {
	            throw new CLIException(
	                    MessageBundle
	                            .localizeMessage(CLIMsgKey.CLI_CREATESMORACLECLONESPECIFICATION_INVALID_SECONDARY_STORAGE_TYPE));
			}
        }

        if (cloneLastBackup != null) {
            GetRecentBackupInformationRequest request = new GetRecentBackupInformationRequest();
            request.setAppObjectId(getAppObjectId());
            request.setSecondary(isIncludeSecondaryDetails());
            request.setRecentBackupNumber(cloneLastBackup);
            request.setPluginCode(PluginCode.SCO);
            request.setBackupType(ORACLE_DATA_BACKUP_TYPE);

            SmBackup lastBackup = getLastOracleDataBackup(
                    smsBackupValidationHelper, request);
            CLIUtils.writeToLogAndConsole(
                    MessageBundle
                            .localizeMessage(
                                    CLIMsgKey.CLI_CREATESMORACLECLONESPECIFICATION_CLONE_LAST_BACKUP_OPTION_IS_CHOSEN,
                                    cloneLastBackup,
                                    lastBackup.getBackupName()), logger,
                    LogLevel.INFO);
            setBackupName(lastBackup.getBackupName());
            setBackupId(lastBackup.getBackupId());
        } else {

            ProgressBar progressBar = ProgressBarUtil
                    .createAndStartProgressBar("Validating backup details ");

            try {
                setBackupId(smsBackupValidationHelper
                        .fetchBackupIdIfItIsValidOracleDataBackup(backupName,
                                appObjectId, isIncludeSecondaryDetails()));
            } catch (CLIException e) {
                throw e;
            } finally {
                ProgressBarUtil.stopProgressBar(progressBar);
            }

            if (StringUtils.isBlank(getBackupId())) {
                if (isIncludeSecondaryDetails()) {
                    throw new CLIException(
                            MessageBundle
                                    .localizeMessage(
                                            CLIMsgKey.CLI_ORACLE_CLONESPEC_CREATION_NOT_VALID_SECONDARY_BACKUP_NAME_OR_RESOURCE_NOT_IN_BACKUP,
                                            getBackupName(), appObjectId));
                } else {
                    throw new CLIException(
                            MessageBundle
                                    .localizeMessage(
                                            CLIMsgKey.CLI_ORACLE_CLONESPEC_CREATION_NOT_VALID_BACKUP_NAME_OR_RESOURCE_NOT_IN_BACKUP,
                                            getBackupName(), appObjectId));
                }
            }
        }

    }

    @Override
    protected <T> void printCommandResponseObject(T smResponse)
            throws CLIException {

        SmGetBackupDetailsResponse smGetBackupDetailsResponse = (SmGetBackupDetailsResponse) smResponse;
        if (!isCommandExecutionSuccessfull(smGetBackupDetailsResponse
                .getResult())) {
            return;
        }

        // Override formatter type to XML always
        formatterType = FormatterType.XML;
        OracleCloneSpecification cloneSpecification = SmoCloneCliUtil
                .createCloneSpec(this, smGetBackupDetailsResponse);

        // If clonespec directory doesn't exists then create one
        File cloneSpecDir = new File(SmoCloneCliUtil.CLONE_SPEC_DIRECTORY);
        if (!cloneSpecDir.exists() && !cloneSpecDir.isDirectory()) {
            try {
                // TODO: Need to set perm of folder to 755
                cloneSpecDir.mkdirs();
            } catch (Exception e) {
                throw new CLIException(e.getLocalizedMessage(), e);
            }
        }

        File cloneSpecFile = new File(cloneSpecDir, String.format(
                SmoCloneCliUtil.ORACLE_CLONE_SPEC_FILE_FORMAT, getDbName(),
                getCloneDatabaseSid(),
                new SimpleDateFormat(DATE_FORMAT_WITH_MILLISECONDS).format(new Date())));
        try {
            cloneSpecFile.createNewFile();
            JAXBContext jaxbContext = JAXBContext
                    .newInstance(OracleCloneSpecification.class);
            Marshaller marshaller = jaxbContext.createMarshaller();
            marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);
            marshaller.marshal(cloneSpecification, cloneSpecFile);

            CLIUtils.writeToLogAndConsole(
                    MessageBundle
                            .localizeMessage(
                                    CLIMsgKey.CLI_CREATESMORACLECLONESPECIFICATION_ORACLE_CLONESPEC_FILE_CREATION_SUCCESS,
                                    cloneSpecFile.getCanonicalPath()), logger,
                    LogLevel.INFO);

        } catch (Exception e) {
            cloneSpecFile.delete();
            throw new CLIException(e.getLocalizedMessage(), e);
        }
    }

    private SmBackup getLastOracleDataBackup(
            SmsBackupValidationHelper smsBackupValidationHelper,
            GetRecentBackupInformationRequest req) throws CLIException {
        SmBackup lastBackup = null;
        ProgressBar progressBar = ProgressBarUtil
                .createAndStartProgressBar("Fetching last backup ");
        try {
            GetRecentBackupInformationResponse resp = smsBackupValidationHelper
                    .getRecentOracleDataBackupInformation(req);
            if (resp.getNumberOfBackups() == 0) {
                String errorMessage = null;
                if (req.isSecondary()) {
                    errorMessage = MessageBundle
                            .localizeMessage(
                                    CLIMsgKey.CLI_CREATESMORACLECLONESPECIFICATION_CLONE_LAST_BACKUP_NO_ORACLE_SECONDARY_DATA_BACKUP_EXISTS_FOR_RESOURCE,
                                    req.getAppObjectId());
                } else {
                    errorMessage = MessageBundle
                            .localizeMessage(
                                    CLIMsgKey.CLI_CREATESMORACLECLONESPECIFICATION_CLONE_LAST_BACKUP_NO_ORACLE_DATA_BACKUP_EXISTS_FOR_RESOURCE,
                                    req.getAppObjectId());
                }
                throw new CLIException(errorMessage);
            } else if (resp.getRecentBackup() == null) {
                String errorMessage = null;
                if (req.isSecondary()) {
                    errorMessage = MessageBundle
                            .localizeMessage(
                                    CLIMsgKey.CLI_CREATESMORACLECLONESPECIFICATION_CLONE_LAST_BACKUP_NUMBER_OF_LAST_BACKUP_IS_MORE_THAN_EXISTSING_SECONDARY_BACKUPS,
                                    req.getRecentBackupNumber(),
                                    req.getAppObjectId(),
                                    resp.getNumberOfBackups());
                } else {
                    errorMessage = MessageBundle
                            .localizeMessage(
                                    CLIMsgKey.CLI_CREATESMORACLECLONESPECIFICATION_CLONE_LAST_BACKUP_NUMBER_OF_LAST_BACKUP_IS_MORE_THAN_EXISTSING_BACKUPS,
                                    req.getRecentBackupNumber(),
                                    req.getAppObjectId(),
                                    resp.getNumberOfBackups());
                }
                throw new CLIException(errorMessage);
            } else {
                lastBackup = resp.getRecentBackup();
            }
        } catch (CLIException e) {
            throw e;
        } finally {
            ProgressBarUtil.stopProgressBar(progressBar);
        }
        return lastBackup;
    }

    public String getBackupName() {
        return backupName;
    }

    public void setBackupName(String backupName) {
        this.backupName = backupName;
    }

    public String getBackupId() {
        return backupId;
    }

    public void setBackupId(String backupId) {
        this.backupId = backupId;
    }

    public String getAppObjectId() {
        return appObjectId;
    }

    public void setAppObjectId(String appObjectId) {
        this.appObjectId = appObjectId;
    }

    public void setDbName(String dbName) {
        this.dbName = dbName;
    }

    public String getDbName() {
        return dbName;
    }

    public void setHostOrClusterName(String hostOrClusterName) {
        this.hostOrClusterName = hostOrClusterName;
    }

    public String getHostOrClusterName() {
        return hostOrClusterName;
    }

    public void setAppPluginCode(PluginCode appPluginCode) {
        this.appPluginCode = appPluginCode;
    }

    public PluginCode getAppPluginCode() {
        return appPluginCode;
    }

    public void setCloneDatabaseSid(String cloneDatabaseSid) {
        this.cloneDatabaseSid = cloneDatabaseSid;
    }

    public String getCloneDatabaseSid() {
        return cloneDatabaseSid;
    }

    public void setIncludeSecondaryDetails(boolean includeSecondaryDetails) {
        this.includeSecondaryDetails = includeSecondaryDetails;
    }

    public boolean isIncludeSecondaryDetails() {
        return includeSecondaryDetails;
    }

    public void setCloneLastBackup(Integer cloneLastBackup) {
        this.cloneLastBackup = cloneLastBackup;
    }

    public Integer getCloneLastBackup() {
        return cloneLastBackup;
    }
    
    public String getSecondaryStorageType() {
		return secondaryStorageType;
	}

	public void setSecondaryStorageType(String secondaryStorageType) {
		this.secondaryStorageType = secondaryStorageType;
	}
}