/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/GetSmBackupReportCommand.java#4 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import static com.netapp.snapcenter.cli.command.validation.SmsBackupValidationHelper.ORACLE;

import java.util.Date;

import javax.xml.datatype.DatatypeFactory;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.plugin.contracts.ArrayOfSmBackupReport;
import com.netapp.smcore.plugin.contracts.SmBackupReport;
import com.netapp.smcore.plugin.contracts.SmCreateBackupReportRequest;
import com.netapp.smcore.plugin.contracts.SmCreateBackupReportResponse;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.snapcenter.cli.command.parser.CustomStringToDateConverter;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.formatter.Formatter;
import com.netapp.snapcenter.cli.formatter.FormatterFactory;
import com.netapp.snapcenter.cli.formatter.FormatterType;
import com.netapp.snapcenter.cli.formatter.TextTable;
import com.netapp.snapcenter.cli.formatter.TextTableInterface;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.getsmbackupreport.command.description")
public class GetSmBackupReportCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMBACKUPREPORT_COMMAND_NAME);

    
    @Parameter(names = { "-BackupId" }, descriptionKey = "cli.getsmbackupreport.command.backupId.option.description")
    private Long backupId;

    @Parameter(names = { "-JobId" }, descriptionKey = "cli.getsmbackupreport.command.jobId.option.description")
    private Long jobId;

    @Parameter(names = { "-FromDateTime" }, descriptionKey = "cli.getsmbackupreport.command.fromDateTime.option.description", converter = CustomStringToDateConverter.class)
    private Date fromDate;

    @Parameter(names = { "-ToDateTime" }, descriptionKey = "cli.getsmbackupreport.command.toDateTime.option.description", converter = CustomStringToDateConverter.class)
    private Date toDate;

    @Parameter(names = { "-BackupName" }, descriptionKey = "cli.getsmbackupreport.command.backupName.option.description")
    protected String backupName;

    @Parameter(names = { "-Policy" }, descriptionKey = "cli.getsmbackupreport.command.policy.option.description")
    protected String policyName;

    @Parameter(names = { "-Resource" }, descriptionKey = "cli.getsmbackupreport.command.resource.option.description")
    protected String resourceName;

    @Parameter(names = { "-ResourceGroup" }, descriptionKey = "cli.getsmbackupreport.command.dataset.option.description")
    protected String datasetName;

    @Parameter(names = { "-Status" }, descriptionKey = "cli.getsmbackupreport.command.status.option.description")
    protected String status;

    @Parameter(names = { "-PluginCode" }, descriptionKey = "cli.getsmbackupreport.command.pluginCode.option.description")
    private PluginCode pluginCode = PluginCode.SCO;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected void processCommandArguments() throws CLIException {

        switch (pluginCode) {
            case SCO:
                break;
            default:
                throw new CLIException(MessageBundle.localizeMessage(
                        CLIMsgKey.CLI_COMMAND_NOT_SUPPORTED_FOR_PLUGIN,
                        pluginCode.name()));
        }
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmCreateBackupReportRequest, SmCreateBackupReportResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext
                .setRequest((SmCreateBackupReportRequest) cmdRequest);
        smsRestInvocationContext
                .setResponseClass(SmCreateBackupReportResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_CREATE_BACKUPREPORT_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    @Override
    protected <T> void printCommandResponseObject(T smResponse)
            throws CLIException {
        SmCreateBackupReportResponse smCreateBackupReportResponse = (SmCreateBackupReportResponse) smResponse;
        ArrayOfSmBackupReport arrayOfSmBackupReport = smCreateBackupReportResponse
                .getReport();

        if (arrayOfSmBackupReport == null
                || CLIUtils.isNullOrEmptyCollection(arrayOfSmBackupReport
                        .getSmBackupReport())) {
            return;
        }

        String headerBackupId = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMBACKUPREPORT_TABLE_BACKUP_ID_HEADER);
        String headerJobId = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMBACKUPREPORT_TABLE_JOB_ID_HEADER);
        String headerBackupName = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMBACKUPREPORT_TABLE_BACKUP_NAME_HEADER);
        String headerBackupStartTime = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMBACKUPREPORT_TABLE_BACKUP_STARTTIME_HEADER);
        String headerBackupEndTime = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMBACKUPREPORT_TABLE_BACKUP_ENDTIME_HEADER);
        String headerBackupDuration = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMBACKUPREPORT_TABLE_BACKUP_DURATION_HEADER);
        String headerBackupDatasetName = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMBACKUPREPORT_TABLE_BACKUP_DATASETNAME_HEADER);
        String headerBackupPolicyName = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMBACKUPREPORT_TABLE_BACKUP_POLICYNAME_HEADER);
        String headerBackupStatus = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMBACKUPREPORT_TABLE_BACKUP_STATUS_HEADER);

        String[] backupReportTableHeaders = { headerBackupId, headerJobId,
                headerBackupName, headerBackupStartTime, headerBackupEndTime,
                headerBackupDuration, headerBackupDatasetName,
                headerBackupPolicyName, headerBackupStatus };

        TextTableInterface backupReportTable = new TextTable(
                backupReportTableHeaders);

        for (SmBackupReport smBackupReport : arrayOfSmBackupReport
                .getSmBackupReport()) {

            if (pluginCode.equals(PluginCode.SCO)
                    && StringUtils.isEmpty(smBackupReport.getBackupType())
                    || !smBackupReport.getBackupType().toLowerCase().contains(ORACLE.toLowerCase())) {
                continue;
            }

            String columnBackupId = (smBackupReport.getSmBackupId() == null) ? ""
                    : smBackupReport.getSmBackupId().toString();
            String columnJobId = (smBackupReport.getSmJobId() == null) ? ""
                    : smBackupReport.getSmJobId().toString();
            String columnBackupName = CLIUtils.getStringValue(smBackupReport
                    .getBackupName());
            String columnBackupStartTime = smBackupReport.getStartDateTime() == null ? ""
                    : smBackupReport.getStartDateTime().toString();
            String columnBackupEndTime = smBackupReport.getEndDateTime() == null ? ""
                    : smBackupReport.getEndDateTime().toString();
            String columnBackupDuration = CLIUtils.getDuration(
                    smBackupReport.getStartDateTime(),
                    smBackupReport.getEndDateTime());
            String columnBackupDatasetName = CLIUtils
                    .getStringValue(smBackupReport.getProtectionGroupName());
            String columnBackupPolicyName = CLIUtils
                    .getStringValue(smBackupReport.getPolicyName());
            String columnBackupStatus = CLIUtils.getStringValue(smBackupReport
                    .getStatus());

            backupReportTable.addRow(columnBackupId, columnJobId,
                    columnBackupName, columnBackupStartTime,
                    columnBackupEndTime, columnBackupDuration,
                    columnBackupDatasetName, columnBackupPolicyName,
                    columnBackupStatus);
        }
        if (getFormatterType() == FormatterType.TEXT_TABLE) {
            backupReportTable.displayFormattedTable(headerBackupId,
                    headerJobId, headerBackupName, headerBackupDuration,
                    headerBackupDatasetName, headerBackupPolicyName,
                    headerBackupStatus);
        } else {
            Formatter formatter = FormatterFactory
                    .getFormatter(getFormatterType());
            formatter.writeFormattedObject(arrayOfSmBackupReport);
        }
    }

    public Long getBackupId() {
        return backupId;
    }

    public void setBackupId(Long backupId) {
        this.backupId = backupId;
    }

    public Long getJobId() {
        return jobId;
    }

    public void setJobId(Long jobId) {
        this.jobId = jobId;
    }

    public Date getFromDate() {
        return fromDate;
    }

    public void setFromDate(Date fromDate) {
        this.fromDate = fromDate;
    }

    public Date getToDate() {
        return toDate;
    }

    public void setToDate(Date toDate) {
        this.toDate = toDate;
    }

    public String getBackupName() {
        return backupName;
    }

    public void setBackupName(String backupName) {
        this.backupName = backupName;
    }

    public String getPolicyName() {
        return policyName;
    }

    public void setPolicyName(String policyName) {
        this.policyName = policyName;
    }

    public String getResourceName() {
        return resourceName;
    }

    public void setResourceName(String resourceName) {
        this.resourceName = resourceName;
    }

    public String getDatasetName() {
        return datasetName;
    }

    public void setDatasetName(String datasetName) {
        this.datasetName = datasetName;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public void setPluginCode(PluginCode pluginCode) {
        this.pluginCode = pluginCode;
    }

    public PluginCode getPluginCode() {
        return pluginCode;
    }
}
