/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/GetSmLogsCommand.java#3 $
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import static com.netapp.snapcenter.cli.utils.CLIUtils.DATE_FORMAT_WITH_MILLISECONDS;
import static com.netapp.spl.common.restclient.config.SMSRestConfiguration.SMS_GET_ALL_LOGS_SERVICE_PATH;
import static com.netapp.spl.common.restclient.config.SMSRestConfiguration.SMS_GET_JOB_LOGS_SERVICE_PATH;
import static com.netapp.spl.common.restclient.config.SMSRestConfiguration.SMS_GET_PLUGIN_LOGS_SERVICE_PATH;
import static com.netapp.spl.common.restclient.config.SMSRestConfiguration.SMS_GET_SERVER_LOGS_SERVICE_PATH;
import static com.netapp.spl.common.restclient.context.GenericRestInvocationContext.APPLICATION_XML;
import static com.netapp.spl.common.restclient.context.GenericRestInvocationContext.APPLICATION_ZIP;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.ArrayOfSmPluginInfo;
import com.netapp.smcore.plugin.contracts.SmGetJobLogsRequest;
import com.netapp.smcore.plugin.contracts.SmGetPluginLogsRequest;
import com.netapp.smcore.plugin.contracts.SmHost;
import com.netapp.smcore.plugin.contracts.SmJob;
import com.netapp.smcore.plugin.contracts.SmPluginCode;
import com.netapp.smcore.plugin.contracts.SmPluginInfo;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;
import com.netapp.snapcenter.cli.utils.ProgressBar;
import com.netapp.snapcenter.cli.utils.ProgressBarUtil;
import com.netapp.spl.common.restclient.config.HttpMethod;
import com.netapp.spl.common.restclient.context.GenericRestInvocationContext;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;
import com.netapp.spl.common.utils.HostInfoUtil;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.getsmlogs.command.description")
public class GetSmLogsCommand extends AbstractSMSCommand {

    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(GetSmLogsCommand.class);

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMLOGS_COMMAND_NAME);

    private static final String DEFAULT_LOG_DUMP_FILE_PATH_FORMAT = "/tmp/%s_%s.zip";

    private static final String PROGRESS_BAR_MESSAGE_FETCHING_SERVER_LOGS = "Fetching server logs ";
    private static final String PROGRESS_BAR_MESSAGE_FETCHING_ALL_LOGS = "Fetching all logs ";
    private static final String PROGRESS_BAR_MESSAGE_FETCHING_JOB_LOGS = "Fetching job logs ";
    private static final String PROGRESS_BAR_MESSAGE_FETCHING_PLUGIN_LOGS = "Fetching plugin logs ";

    private static final String DEFAULT_SERVER_LOGS_FILENAME_PREFIX = "snapcenter_server_logs";
    private static final String DEFAULT_ALL_LOGS_FILENAME_PREFIX = "snapcenter_all_logs";
    private static final String DEFAULT_JOB_LOGS_FILENAME_PREFIX_FORMAT = "snapcenter_job_%s_logs";
    private static final String DEFAULT_PLUGIN_LOGS_FILENAME_PREFIX_FORMAT = "snapcenter_%s_plugin_%s_host_logs";

    @Parameter(names = { "-Path" }, descriptionKey = "cli.getsmlogs.command.path.option.description")
    private String path;

    @Parameter(names = { "-ServerLogs" }, descriptionKey = "cli.getsmlogs.command.serverLogs.option.description")
    private boolean serverLogs;

    @Parameter(names = { "-AllLogs" }, descriptionKey = "cli.getsmlogs.command.allLogs.option.description")
    private boolean allLogs;

    @Parameter(names = { "-JobId" }, descriptionKey = "cli.getsmlogs.command.jobId.option.description")
    private Long jobId;

    @Parameter(names = { "-HostName" }, descriptionKey = "cli.getsmlogs.command.hostName.option.description")
    private String hostName;

    @Parameter(names = { "-PluginCode" }, descriptionKey = "cli.getsmlogs.command.pluginCode.option.description")
    private PluginCode pluginCode;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    public void run() throws CLIException {
        initialize();

        if (!isServerLogs() && !isAllLogs() && getJobId() == null
                && getPluginCode() == null) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_GETSMLOGS_NOTHING_WAS_SPECIFIED_TO_FETCH_LOGS));
        }

        // Validate user entered path
        if (getPath() != null) {
            if (StringUtils.isBlank(getPath())) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_GETSMLOGS_PATH_CANNOT_BE_EMPTY));
            }
            File verifyLogDumpFile = new File(getPath());
            if (verifyLogDumpFile.exists()) {
                String logDumpCanonicalFilePath = CLIUtils
                        .getCanonicalFilePath(verifyLogDumpFile);
                logDumpCanonicalFilePath = (logDumpCanonicalFilePath == null) ? verifyLogDumpFile
                        .getPath() : logDumpCanonicalFilePath;
                throw new CLIException(MessageBundle.localizeMessage(
                        CLIMsgKey.CLI_GETSMLOGS_PATH_ALREADY_EXISTS,
                        logDumpCanonicalFilePath));
            }
            File verifyLogDumpPathDirectory = verifyLogDumpFile.getParentFile();
            if (verifyLogDumpPathDirectory != null
                    && !verifyLogDumpPathDirectory.exists()
                    && !verifyLogDumpPathDirectory.isDirectory()) {
                try {
                    verifyLogDumpPathDirectory.mkdirs();
                } catch (Exception e) {
                    throw new CLIException(e.getLocalizedMessage(), e);
                }
            }
        }

        String progressBarMessage = null;
        String defaultLogFileNamePrefix = null;

        if (isServerLogs()) {
            progressBarMessage = PROGRESS_BAR_MESSAGE_FETCHING_SERVER_LOGS;
            defaultLogFileNamePrefix = DEFAULT_SERVER_LOGS_FILENAME_PREFIX;
            CLIUtils.writeToLogAndConsole(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_GETSMLOGS_CHOSEN_TO_FETCH_SERVER_LOGS),
                    logger, LogLevel.INFO);
        } else if (isAllLogs()) {
            progressBarMessage = PROGRESS_BAR_MESSAGE_FETCHING_ALL_LOGS;
            defaultLogFileNamePrefix = DEFAULT_ALL_LOGS_FILENAME_PREFIX;
            CLIUtils.writeToLogAndConsole(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_GETSMLOGS_CHOSEN_TO_FETCH_ALL_LOGS),
                    logger, LogLevel.INFO);
        } else if (getJobId() != null) {
            progressBarMessage = PROGRESS_BAR_MESSAGE_FETCHING_JOB_LOGS;
            defaultLogFileNamePrefix = String.format(
                    DEFAULT_JOB_LOGS_FILENAME_PREFIX_FORMAT, getJobId());
            CLIUtils.writeToLogAndConsole(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_GETSMLOGS_CHOSEN_TO_FETCH_JOB_LOGS,
                    getJobId()), logger, LogLevel.INFO);
        } else if (getPluginCode() != null) {
            progressBarMessage = PROGRESS_BAR_MESSAGE_FETCHING_PLUGIN_LOGS;
            if (getHostName() == null) {
                setHostName(HostInfoUtil.getCanonicalHostName());
                String displayUsingLocalhostDefaultForFetchingPluginLogs = MessageBundle
                        .localizeMessage(
                                CLIMsgKey.CLI_GETSMLOGS_USING_LOCALHOST_AS_DEFAULT_FOR_FETCHING_PLUGIN_LOGS,
                                getHostName(), getPluginCode().name());
                CLIUtils.writeToLogAndConsole(
                        displayUsingLocalhostDefaultForFetchingPluginLogs,
                        logger, LogLevel.INFO);
            } else {
                String displayFetchPluginLogsForHost = MessageBundle
                        .localizeMessage(
                                CLIMsgKey.CLI_GETSMLOGS_USING_HOST_FOR_FETCHING_PLUGIN_LOGS,
                                getPluginCode().name(), getHostName());
                CLIUtils.writeToLogAndConsole(displayFetchPluginLogsForHost,
                        logger, LogLevel.INFO);
            }
            defaultLogFileNamePrefix = String.format(
                    DEFAULT_PLUGIN_LOGS_FILENAME_PREFIX_FORMAT,
                    getPluginCode(), StringUtils.substring(getHostName(), 0, getHostName().indexOf(".")));
        }

        ProgressBar progressBar = null;
        InputStream responseInputStream = null;
        File logDumpFile = null;
        try {
            progressBar = ProgressBarUtil
                    .createAndStartProgressBar(progressBarMessage);
            GenericRestInvocationContext<?, ?> genericRestInvocationContext = getLogsSpecificRestInvocationContext();
            responseInputStream = (InputStream) getSnapCenterServerServiceHelper()
                    .makeRestCallToSnapCenterServerWithGenericRestInvocationContext(
                            genericRestInvocationContext);

            if (getPath() == null) {
                setPath(String.format(DEFAULT_LOG_DUMP_FILE_PATH_FORMAT,
                        defaultLogFileNamePrefix, new SimpleDateFormat(
                                DATE_FORMAT_WITH_MILLISECONDS)
                                .format(new Date())));
            }
            logDumpFile = new File(getPath());
            try (OutputStream outputStream = new FileOutputStream(logDumpFile)) {

                int read = 0;
                byte[] bytes = new byte[1024];

                while ((read = responseInputStream.read(bytes)) != -1) {
                    outputStream.write(bytes, 0, read);
                }
            }

        } catch (CLIException e) {
            throw e;
        } catch (Exception e) {
            throw new CLIException(e);
        } finally {
            ProgressBarUtil.stopProgressBar(progressBar);
            if (responseInputStream != null) {
                try {
                    responseInputStream.close();
                } catch (IOException e) {
                    CLIUtils.writeToLog(e, logger, LogLevel.DEBUG);
                }
            }
        }

        String logFileDumpCanonicalPath = CLIUtils
                .getCanonicalFilePath(logDumpFile);
        logFileDumpCanonicalPath = (logFileDumpCanonicalPath == null) ? getPath()
                : logFileDumpCanonicalPath;
        String logFileDumpSuccess = MessageBundle.localizeMessage(
                CLIMsgKey.CLI_GETSMLOGS_LOGS_FETCH_SUCCESS,
                logFileDumpCanonicalPath);
        CLIUtils.writeToLogAndConsole(logFileDumpSuccess, logger, LogLevel.INFO);

        String executionSucess = MessageBundle.localizeMessage(
                CLIMsgKey.CLI_COMMAND_EXECUTION_SUCCESS, this.getCommandName());
        CLIUtils.writeToLogAndConsole(executionSucess, logger, LogLevel.INFO);

    }

    private GenericRestInvocationContext<Object, InputStream> getLogsSpecificRestInvocationContext() {
        GenericRestInvocationContext<Object, InputStream> restInvocationContext = new GenericRestInvocationContext<>();
        restInvocationContext.setAccepts(APPLICATION_ZIP);
        restInvocationContext.setHttpMethod(HttpMethod.POST);
        restInvocationContext.setResponseClass(InputStream.class);

        if (isServerLogs()) {
            restInvocationContext.setContentType(null);
            restInvocationContext.setRequest(null);
            restInvocationContext
                    .setServiceUrlPath(SMS_GET_SERVER_LOGS_SERVICE_PATH);
        } else if (isAllLogs()) {
            restInvocationContext.setContentType(null);
            restInvocationContext.setRequest(null);
            restInvocationContext
                    .setServiceUrlPath(SMS_GET_ALL_LOGS_SERVICE_PATH);
        } else if (getJobId() != null) {
            SmGetJobLogsRequest smGetJobLogsRequest = new SmGetJobLogsRequest();
            SmJob smJob = new SmJob();
            smJob.setId(getJobId());
            smGetJobLogsRequest.setJob(smJob);

            restInvocationContext.setContentType(APPLICATION_XML);
            restInvocationContext.setRequest(smGetJobLogsRequest);
            restInvocationContext
                    .setServiceUrlPath(SMS_GET_JOB_LOGS_SERVICE_PATH);
        } else if (getPluginCode() != null) {
            SmGetPluginLogsRequest smGetPluginLogsRequest = new SmGetPluginLogsRequest();

            SmHost smHost = new SmHost();
            ArrayOfSmPluginInfo arrayOfSmPluginInfo = new ArrayOfSmPluginInfo();

            SmPluginInfo smPluginInfo = new SmPluginInfo();
            smPluginInfo.setHostName(getHostName());
            smPluginInfo.setPluginCode(SmPluginCode.fromValue(getPluginCode()
                    .name()));

            arrayOfSmPluginInfo.getSmPluginInfo().add(smPluginInfo);
            smHost.setHostPluginInfos(arrayOfSmPluginInfo);

            smGetPluginLogsRequest.setHostEx(smHost);

            restInvocationContext.setContentType(APPLICATION_XML);
            restInvocationContext.setRequest(smGetPluginLogsRequest);
            restInvocationContext
                    .setServiceUrlPath(SMS_GET_PLUGIN_LOGS_SERVICE_PATH);
        }

        return restInvocationContext;
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        return null;
    }

    public String getPath() {
        return path;
    }

    public void setPath(String path) {
        this.path = path;
    }

    public boolean isServerLogs() {
        return serverLogs;
    }

    public void setServerLogs(boolean serverLogs) {
        this.serverLogs = serverLogs;
    }

    public boolean isAllLogs() {
        return allLogs;
    }

    public void setAllLogs(boolean allLogs) {
        this.allLogs = allLogs;
    }

    public Long getJobId() {
        return jobId;
    }

    public void setJobId(Long jobId) {
        this.jobId = jobId;
    }

    public String getHostName() {
        return hostName;
    }

    public void setHostName(String hostName) {
        this.hostName = hostName;
    }

    public PluginCode getPluginCode() {
        return pluginCode;
    }

    public void setPluginCode(PluginCode pluginCode) {
        this.pluginCode = pluginCode;
    }

}