/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/GetSmOracleDatabaseConfigurationCommand.java#3 $
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import static com.netapp.snapcenter.cli.helper.SmOracleDatabaseConfigurationHelper.DATABASE_OR_ASM_DEFAULT_PORT;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.ArrayOfSmHost;
import com.netapp.smcore.plugin.contracts.SmAuth;
import com.netapp.smcore.plugin.contracts.SmAuthMode;
import com.netapp.smcore.plugin.contracts.SmHost;
import com.netapp.smcore.plugin.contracts.SmOracleDb;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.snapcenter.cli.command.validation.SmsCommandArgumentsValidationUtil;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.formatter.TextTable;
import com.netapp.snapcenter.cli.formatter.TextTableInterface;
import com.netapp.snapcenter.cli.helper.SmOracleDatabaseConfigurationHelper;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.ProgressBar;
import com.netapp.snapcenter.cli.utils.ProgressBarUtil;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.getsmoracledatabaseconfiguration.command.description")
public class GetSmOracleDatabaseConfigurationCommand extends AbstractSMSCommand {

    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(GetSmOracleDatabaseConfigurationCommand.class);

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMORACLEDATABASECONFIGURATION_COMMAND_NAME);

    @Parameter(names = { "-AppObjectId" }, descriptionKey = "cli.getsmoracledatabaseconfiguration.command.appObjectId.option.description", required = true)
    private String appObjectId;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    public void run() throws CLIException {
        initialize();

        if (!SmsCommandArgumentsValidationUtil
                .isValidAppObjectId(getAppObjectId())) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_COMMAND_PARAMETER_APP_OBJECT_ID_NOT_VALID));
        }

        ProgressBar progressBar = null;
        SmOracleDb smOracleDb = null;
        ArrayOfSmHost racNodes = null;
        SmOracleDatabaseConfigurationHelper smOracleDatabaseConfigurationHelper = new SmOracleDatabaseConfigurationHelper(
                getSnapCenterServerServiceHelper(), getAppObjectId());

        try {
            progressBar = ProgressBarUtil
                    .createAndStartProgressBar("Contacting SnapCenter Server ");
            smOracleDb = smOracleDatabaseConfigurationHelper
                    .getOracleDatabaseDetails();
            if (smOracleDb == null) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_GETSMORACLEDATABASECONFIGURATION_INVALID_APP_OBJECT_ID_DETAILS,
                                        getAppObjectId()));
            }
            if (smOracleDatabaseConfigurationHelper
                    .isOracleRacDatabase(smOracleDb)) {
                racNodes = smOracleDatabaseConfigurationHelper
                        .getOracleClusterDatabaseHostsList();
            }
        } catch (CLIException e) {
            throw e;
        } finally {
            ProgressBarUtil.stopProgressBar(progressBar);
        }

        showDbConfiguration(smOracleDb);
        CLIUtils.writeToConsole(StringUtils.EMPTY);
        if (smOracleDatabaseConfigurationHelper.isOracleRacDatabase(smOracleDb)) {
            showRacNodes(racNodes);
        }

        String executionSucess = MessageBundle.localizeMessage(
                CLIMsgKey.CLI_COMMAND_EXECUTION_SUCCESS, this.getCommandName());
        CLIUtils.writeToLogAndConsole(executionSucess, logger, LogLevel.INFO);

    }

    private void showDbConfiguration(SmOracleDb smOracleDb) {
        CLIUtils.writeToConsole("Credentials for Oracle Database : "
                + smOracleDb.getDbSid());

        String dbRunAsForDispaly = getRunAsForDisplay(smOracleDb.getAuth());
        CLIUtils.writeToConsole("Database RunAs: " + dbRunAsForDispaly);
        int dbPortForDisplay = smOracleDb.getDatabasePort() == 0 ? DATABASE_OR_ASM_DEFAULT_PORT
                : smOracleDb.getDatabasePort();
        CLIUtils.writeToConsole("Database Port: " + dbPortForDisplay);

        String asmRunAs = getRunAsForDisplay(smOracleDb.getASMAuth());
        CLIUtils.writeToConsole("ASM Instance RunAs: " + asmRunAs);
        int asmPortForDisplay = smOracleDb.getAsmPort() == 0 ? DATABASE_OR_ASM_DEFAULT_PORT
                : smOracleDb.getAsmPort();
        CLIUtils.writeToConsole("ASM Port: " + asmPortForDisplay);

        String rmanCatalogRunAs = getRunAsForDisplay(smOracleDb.getRmanCatalogAuth());
        CLIUtils.writeToConsole("Oracle RMAN catalog RunAs: " + rmanCatalogRunAs);
        CLIUtils.writeToConsole("Oracle RMAN catalog TNS name: " + CLIUtils.getStringValue(smOracleDb.getRmanCatalogTnsName()));

    }

    private String getRunAsForDisplay(SmAuth auth) {
        if (auth == null || auth.getAuthMode() == null
                || auth.getAuthMode().equals(SmAuthMode.NONE)) {
            return SmAuthMode.NONE.name();
        } else {
            return auth.getRunAsName();
        }
    }

    private void showRacNodes(ArrayOfSmHost arrayOfSmHost) {
        String availableRacNodesHeader = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMORACLEDATABASECONFIGURATION_RACNODESTABLE_AVAILABLERACNODES_HEADER);
        String preferredNodesHeader = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMORACLEDATABASECONFIGURATION_RACNODESTABLE_PREFERREDRACNODES_HEADER);

        String[] racNodesTableHeaders = { availableRacNodesHeader,
                preferredNodesHeader };
        TextTableInterface racNodesTableFormatter = new TextTable(
                racNodesTableHeaders);

        List<String> availableRacNodesList = new LinkedList<>();
        List<String> preferredRacNodesList = new LinkedList<>();

        Map<Long, String> prefToNodeMap = new HashMap<Long, String>();

        for (SmHost smHost : arrayOfSmHost.getSmHost()) {
            if (smHost.getPreference() == 0) {
                availableRacNodesList.add(smHost.getHostName());
            } else {
                prefToNodeMap.put(smHost.getPreference(), smHost.getHostName());
            }
        }

        for (long i = 1; i <= prefToNodeMap.size(); i++) {
            preferredRacNodesList.add(prefToNodeMap.get(i));
        }

        int availCount = availableRacNodesList.size();
        int prefCount = preferredRacNodesList.size();

        Iterator<String> availIter = availableRacNodesList.iterator();
        Iterator<String> prefIter = preferredRacNodesList.iterator();

        while (availCount > 0 && prefCount > 0) {
            racNodesTableFormatter.addRow(availIter.next(), prefIter.next());
            availCount--;
            prefCount--;
        }
        while (availCount > 0) {
            racNodesTableFormatter.addRow(availIter.next(), StringUtils.EMPTY);
            availCount--;
        }
        while (prefCount > 0) {
            racNodesTableFormatter.addRow(StringUtils.EMPTY, prefIter.next());
            prefCount--;
        }
        CLIUtils.writeToConsole("Oracle RAC nodes preference for Backup operation");
        racNodesTableFormatter.displayFormattedTable();
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        return null;
    }

    public String getAppObjectId() {
        return appObjectId;
    }

    public void setAppObjectId(String appObjectId) {
        this.appObjectId = appObjectId;
    }

}