/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/GetSmPolicyCommand.java#4 $
 * 
 * Copyright (c) 2016 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.plugin.contracts.SmGetPolicyRequest;
import com.netapp.smcore.plugin.contracts.SmGetPolicyResponse;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.snapcenter.cli.command.output.formatter.CommandOutputFormatter;
import com.netapp.snapcenter.cli.command.output.formatter.GetSmPolicyCommandOutputFormatter;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.getsmpolicy.command.description")
public class GetSmPolicyCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMPOLICY_COMMAND_NAME);

    @Parameter(names = { "-PolicyName" }, descriptionKey = "cli.getsmpolicy.command.policyName.option.description")
    private String policyName;

    @Parameter(names = { "-PluginCode" }, descriptionKey = "cli.getsmpolicy.command.pluginCode.option.description")
    private PluginCode pluginCode = PluginCode.SCO;

    @Parameter(names = { "-ResourceGroupName" }, descriptionKey = "cli.getsmpolicy.command.resourcegroup.option.description")
    private String datasetName;

    // TODO: NOt yet available in powershell so hiding it for now.
    // @Parameter(names = { "-IsGetAttachedPoliciesOnly" }, descriptionKey = "cli.getsmpolicy.command.IsGetAttachedPoliciesOnly.option.description", arity = 1, hidden = true)
    private boolean getAttachedPoliciesOnly = true;

    private boolean allowMultipleSchedules = true;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmGetPolicyRequest, SmGetPolicyResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext.setRequest((SmGetPolicyRequest) cmdRequest);
        smsRestInvocationContext.setResponseClass(SmGetPolicyResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_GET_POLICY_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    @Override
    protected CommandOutputFormatter getCommandSpecificOutputFormatter() {
        return new GetSmPolicyCommandOutputFormatter(pluginCode);

    }

    public String getPolicyName() {
        return policyName;
    }

    public void setPolicyName(String policyName) {
        this.policyName = policyName;
    }

    public String getDatasetName() {
        return datasetName;
    }

    public void setDatasetName(String datasetName) {
        this.datasetName = datasetName;
    }

    public void setGetAttachedPoliciesOnly(boolean getAttachedPoliciesOnly) {
        this.getAttachedPoliciesOnly = getAttachedPoliciesOnly;
    }

    public boolean isGetAttachedPoliciesOnly() {
        return getAttachedPoliciesOnly;
    }

    public boolean isAllowMultipleSchedules() {
        return allowMultipleSchedules;
    }

    public void setAllowMultipleSchedules(boolean allowMultipleSchedules) {
        this.allowMultipleSchedules = allowMultipleSchedules;
    }
}
