/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/ModifySmResourceGroupCommand.java#4 $
 *
 * Copyright (c) 2016 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.beust.jcommander.ParametersDelegate;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.ArrayOfSmPolicy;
import com.netapp.smcore.plugin.contracts.SmBackupPolicy;
import com.netapp.smcore.plugin.contracts.SmBackupPolicyVerificationSettings;
import com.netapp.smcore.plugin.contracts.SmGetSMTPServerResponse;
import com.netapp.smcore.plugin.contracts.SmModifyProtectionGroupRequest;
import com.netapp.smcore.plugin.contracts.SmModifyProtectionGroupResponse;
import com.netapp.smcore.plugin.contracts.SmPolicy;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.smcore.plugin.contracts.SmRetention;
import com.netapp.smcore.plugin.contracts.SmSchedule;
import com.netapp.smcore.plugin.contracts.SmSchedulerType;
import com.netapp.smcore.plugin.contracts.SmvalidatePolicyResponse;
import com.netapp.snapcenter.cli.command.parameters.BackupResourceGroupParameters;
import com.netapp.snapcenter.cli.command.parameters.ResourceGroupScheduleParameters;
import com.netapp.snapcenter.cli.command.parameters.ResourceGroupVerificationParameters;
import com.netapp.snapcenter.cli.command.parameters.SMOBackupDataSetParameters;
import com.netapp.snapcenter.cli.command.parameters.VerificationResourceGroupParameters;
import com.netapp.snapcenter.cli.command.parser.CommaParameterSplitter;
import com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter;
import com.netapp.snapcenter.cli.command.parser.CustomStringToSchedulesMapConverter;
import com.netapp.snapcenter.cli.command.parser.NoParameterSplitter;
import com.netapp.snapcenter.cli.command.validation.DataSetCommandParametersValidationUtil;
import com.netapp.snapcenter.cli.command.validation.ResourceGroupScheduleParametersValidationUtil;
import com.netapp.snapcenter.cli.command.validation.ResourceGroupVerificationParametersValidationUtil;
import com.netapp.snapcenter.cli.command.validation.SmsCommandArgumentsValidationUtil;
import com.netapp.snapcenter.cli.command.validation.SmsCommandRequestValidationHelper;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.ProgressBar;
import com.netapp.snapcenter.cli.utils.ProgressBarUtil;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.modifysmresourcegroup.command.description")
public class ModifySmResourceGroupCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_MODIFYSMRESOURCEGROUP_COMMAND_NAME);
    
    private static ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(ModifySmResourceGroupCommand.class);

    public static final String SKIP_RESOURCE_MODIFICATION_CONTEXT_VARIABLE = "SkipResourceModification";

    @Parameter(names = { "-ResourceGroupName" }, descriptionKey = "cli.addsmresourcegroup.command.dataSetName.option.description", required = true)
    private String resourceGroupName;

    @Parameter(names = { "-PluginCode" }, descriptionKey = "cli.modifysmresourcegroup.command.pluginCode.option.description")
    private PluginCode pluginCode = PluginCode.SCO;

    @Parameter(names = { "-Description" }, descriptionKey = "cli.modifysmresourcegroup.command.description.option.description")
    private String description;



    @Parameter(names = { "-Tags" }, descriptionKey = "cli.addsmresourcegroup.command.tag.option.description")
    private String tag;

    @Parameter(names = { "-Resources" }, descriptionKey = "cli.addsmresourcegroup.command.resources.option.description", converter = CustomStringToHostResourcesMapConverter.class, splitter = NoParameterSplitter.class)
    private List<Map<String, String>> resources = new ArrayList<Map<String, String>>();

    @Parameter(names = { "-RemoveCustomSnapShot" }, descriptionKey = "cli.modifysmresourcegroup.command.removecustomsnapshot.option.description")
    private boolean removeCustomSnapShot;

    @Parameter(names = { "-Schedules" }, descriptionKey = "cli.addsmresourcegroup.command.schedules.option.description", converter = CustomStringToSchedulesMapConverter.class, splitter = NoParameterSplitter.class)
    private List<Map<String, String>> schedules = new ArrayList<Map<String, String>>();

    @ParametersDelegate
    private BackupResourceGroupParameters backupResourceGroupParameters = new BackupResourceGroupParameters();

    @ParametersDelegate
    private SMOBackupDataSetParameters smoBackupDataSetParameters = new SMOBackupDataSetParameters();
    
    @ParametersDelegate
    private VerificationResourceGroupParameters verificationResourceGroupParameters = new VerificationResourceGroupParameters();

    private boolean skipPolicyModification;

    private boolean skipResourceModification;
    
    private List<ResourceGroupScheduleParameters> scheduleParametersList = new ArrayList<ResourceGroupScheduleParameters>();
    
    private List<ResourceGroupVerificationParameters> verificationscheduleParametersList;

    private List<SmPolicy> resourceGroupPolicyList;
    
    private Map<String, List<SmSchedulerType>> policyVsScheduleTypeMap = new HashMap<String, List<SmSchedulerType>>();
    
    private boolean verificationEnabledForPolicy;
    
    private List<SmBackupPolicyVerificationSettings> smBackupPolicyVerificationSettings = new ArrayList<SmBackupPolicyVerificationSettings>();

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected void processCommandArguments() throws CLIException {

        SmsCommandRequestValidationHelper smsCommandRequestValidationHelper = new SmsCommandRequestValidationHelper(
                getSnapCenterServerServiceHelper());

        switch (pluginCode) {
            case SCO:
                break;
            default:
                throw new CLIException(MessageBundle.localizeMessage(
                        CLIMsgKey.CLI_COMMAND_NOT_SUPPORTED_FOR_PLUGIN,
                        pluginCode.name()));
        }

        DataSetCommandParametersValidationUtil.validateDatasetName(resourceGroupName);

        if(getTag() != null){
            setTag(DataSetCommandParametersValidationUtil
                    .validateTag(getTag()));
        }

        SmGetSMTPServerResponse smGetSMTPServerResponse = smsCommandRequestValidationHelper
				.getSmSMTPServerDetailsFromServer();

		SmsCommandArgumentsValidationUtil.validateAndSetNotificationParameters(
				backupResourceGroupParameters.getNotificationParameters(),
				smGetSMTPServerResponse,COMMANDNAME);

        DataSetCommandParametersValidationUtil.validateSchedulerRunAsName(backupResourceGroupParameters);


        //Check if duplicate schedules are provided for the same policy and error out if true.
        //Also, check if the policy for which the schedule is provided is given in the -Policies parameter
        DataSetCommandParametersValidationUtil
                .validateIfDuplicatePolicySchedulesProvided(backupResourceGroupParameters.getPolicies(),schedules);



        if (CLIUtils.isNotNullAndNotEmptyCollection(resources)) {
            DataSetCommandParametersValidationUtil.validateHostResources(
                    resources, pluginCode);
        }

        DataSetCommandParametersValidationUtil.validateIfPolicyIsAssociatedWithSchedules(backupResourceGroupParameters.getPolicies(), schedules);

        DataSetCommandParametersValidationUtil
                .processAndValidateResourceGroupParameters(backupResourceGroupParameters);

        if (removeCustomSnapShot) {
            backupResourceGroupParameters.setCustomSnapshot(false);
        }

        if (backupResourceGroupParameters.getPolicies() != null
                && backupResourceGroupParameters.getPolicies().isEmpty()) {
            skipPolicyModification = true;
        }

        if (resources.isEmpty()) {
            skipResourceModification = true;
        }

        if (backupResourceGroupParameters.getPolicies().size() > 0) {
            ProgressBar progressBar = ProgressBarUtil.createAndStartProgressBar("Validating policies ");

            try {
                SmvalidatePolicyResponse smvalidatePolicyResponse = smsCommandRequestValidationHelper
                        .checkIfPoliciesAreValid(backupResourceGroupParameters.getPolicies());

                DataSetCommandParametersValidationUtil.validateBackupPolicies(smvalidatePolicyResponse, pluginCode);

                ArrayOfSmPolicy arrayOfSmPolicy = smvalidatePolicyResponse.getAvailablePolicies();

                for (SmPolicy smPolicy : arrayOfSmPolicy.getSmPolicy()) {
                    if (smPolicy instanceof SmBackupPolicy) {
                        SmBackupPolicy smBackupPolicy = (SmBackupPolicy) smPolicy;

                        List<SmSchedulerType> schedulerTypeList = new ArrayList<SmSchedulerType>();
                        for (SmSchedule s : smBackupPolicy.getSchedules()
                                .getSmSchedule()) {
                            if (s.getSchedulerType() != SmSchedulerType.NONE) {
                                schedulerTypeList.add(s.getSchedulerType());
                            }
                        }

                        policyVsScheduleTypeMap.put(smBackupPolicy.getName(),
                                schedulerTypeList);

                        // Validate verification schedules
                        if (!verificationEnabledForPolicy) {
                            for (SmRetention smRetention : smBackupPolicy
                                    .getRetentions().getSmRetention()) {
                                if (smRetention.isVerificationEnabled()) {
                                    verificationEnabledForPolicy = true;
                                }

                            }
                        }
                    }
                }

                setScheduleParametersList(ResourceGroupScheduleParametersValidationUtil
                        .processScheduleParameters(schedules,
                                policyVsScheduleTypeMap));

                // It is first level of check, as in verification schedules are
                // not provided but verification is enabled for backup schedule
                // types or other way around. Validation for each policy is done
                // at later stage.
                if (verificationEnabledForPolicy) {
                    if (CLIUtils.isEmpty(verificationResourceGroupParameters
                            .getVerificationSchedules())) {
                        throw new CLIException(
                                MessageBundle
                                        .localizeMessage(CLIMsgKey.CLI_VERIFICATION_SCHEDULE_TYPES_NOT_PROVIDED_FOR_VERIFICATION_ENABLED_BACKUP_SCHEDULE_TYPES));
                    }
                } else {
                    if (!CLIUtils.isEmpty(verificationResourceGroupParameters
                            .getVerificationSchedules())) {
                        throw new CLIException(
                                MessageBundle
                                        .localizeMessage(CLIMsgKey.CLI_VERIFICATION_SCHEDULE_TYPES_PROVIDED_FOR_VERIFICATION_DISABLED_BACKUP_SCHEDULE_TYPES));
                    }
                }


                // validated if policy has schedule
                setResourceGroupPolicyList(DataSetCommandParametersValidationUtil.validateAndBuildPolicyAndSchedules(
                        backupResourceGroupParameters.getPolicies(), scheduleParametersList, policyVsScheduleTypeMap));
                
                // validated if verification is enabled
                if (verificationEnabledForPolicy) {
                    setVerificationscheduleParametersList(ResourceGroupVerificationParametersValidationUtil
                            .processVerificationScheduleParameters(
                                    verificationResourceGroupParameters
                                            .getVerificationSchedules(),
                                    arrayOfSmPolicy,
                                    backupResourceGroupParameters.getPolicies()));
                    DataSetCommandParametersValidationUtil
                            .buildSmBackupPolicyVerificationSettings(
                                    smBackupPolicyVerificationSettings,
                                    getVerificationscheduleParametersList());
                }

            } catch (CLIException e) {
                throw e;
            } finally {
                ProgressBarUtil.stopProgressBar(progressBar);

            }

        }

    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmModifyProtectionGroupRequest, SmModifyProtectionGroupResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext
                .setRequest((SmModifyProtectionGroupRequest) cmdRequest);
        smsRestInvocationContext
                .setResponseClass(SmModifyProtectionGroupResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_MODIFY_PROTECTION_GROUP_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    public PluginCode getPluginCode() {
        return pluginCode;
    }

    public void setPluginCode(PluginCode pluginCode) {
        this.pluginCode = pluginCode;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public List<Map<String, String>> getResources() {
        return resources;
    }

    public void setResources(List<Map<String, String>> resources) {
        this.resources = resources;
    }

    public BackupResourceGroupParameters getBackupResourceGroupParameters() {
        return backupResourceGroupParameters;
    }

    public void setBackupResourceGroupParameters(
            BackupResourceGroupParameters backupDataSetParameters) {
        this.backupResourceGroupParameters = backupDataSetParameters;
    }

    public boolean isSkipPolicyModification() {
        return skipPolicyModification;
    }

    public void setSkipPolicyModification(boolean skipPolicyModification) {
        this.skipPolicyModification = skipPolicyModification;
    }

    public boolean isSkipResourceModification() {
        return skipResourceModification;
    }

    public void setSkipResourceModification(boolean skipResourceModification) {
        this.skipResourceModification = skipResourceModification;
    }

    public void setSmoBackupDataSetParameters(
            SMOBackupDataSetParameters smoBackupDataSetParameters) {
        this.smoBackupDataSetParameters = smoBackupDataSetParameters;
    }

    public SMOBackupDataSetParameters getSmoBackupDataSetParameters() {
        return smoBackupDataSetParameters;
    }
    
    public VerificationResourceGroupParameters getVerificationResourceGroupParameters() {
        return verificationResourceGroupParameters;
    }

    public void setVerificationResourceGroupParameters(
            VerificationResourceGroupParameters verificationResourceGroupParameters) {
        this.verificationResourceGroupParameters = verificationResourceGroupParameters;
    }

    public List<ResourceGroupVerificationParameters> getVerificationscheduleParametersList() {
        return verificationscheduleParametersList;
    }

    public void setVerificationscheduleParametersList(
            List<ResourceGroupVerificationParameters> verificationscheduleParametersList) {
        this.verificationscheduleParametersList = verificationscheduleParametersList;
    }
    

    public String getResourceGroupName() {
        return resourceGroupName;
    }

    public void setResourceGroupName(String resourceGroupName) {
        this.resourceGroupName = resourceGroupName;
    }
    
    public List<SmPolicy> getResourceGroupPolicyList() {
        return resourceGroupPolicyList;
    }

    public void setResourceGroupPolicyList(List<SmPolicy> resourceGroupPolicyList) {
        this.resourceGroupPolicyList = resourceGroupPolicyList;
    }
    
    public List<ResourceGroupScheduleParameters> getScheduleParametersList() {
        return scheduleParametersList;
    }

    public void setScheduleParametersList(List<ResourceGroupScheduleParameters> scheduleList) {
        this.scheduleParametersList = scheduleList;
    }

    public List<Map<String, String>> getSchedules() {
        return schedules;
    }

    public void setSchedules(List<Map<String, String>> schedules) {
        this.schedules = schedules;
    }
    
    public boolean isVerificationEnabledForPolicy() {
        return verificationEnabledForPolicy;
    }

    public void setVerificationEnabledForPolicy(boolean verificationEnabledForPolicy) {
        this.verificationEnabledForPolicy = verificationEnabledForPolicy;
    }

    public List<SmBackupPolicyVerificationSettings> getSmBackupPolicyVerificationSettings() {
        return smBackupPolicyVerificationSettings;
    }

    public void setSmBackupPolicyVerificationSettings(
            List<SmBackupPolicyVerificationSettings> smBackupPolicyVerificationSettings) {
        this.smBackupPolicyVerificationSettings = smBackupPolicyVerificationSettings;
    }
    public String getTag () {
        return tag;
    }

    public void setTag (String tag) {
        this.tag = tag;
    }

}