/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/OpenSmConnection.java#3 $
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.ArrayOfLong;
import com.netapp.smcore.plugin.contracts.ArrayOfSmKeyValue;
import com.netapp.smcore.plugin.contracts.ArrayOfString;
import com.netapp.smcore.plugin.contracts.SMResult;
import com.netapp.smcore.plugin.contracts.SmContext;
import com.netapp.smcore.plugin.contracts.SmGetRolesByUserRequest;
import com.netapp.smcore.plugin.contracts.SmGetRolesByUserResponse;
import com.netapp.smcore.plugin.contracts.SmKeyValue;
import com.netapp.smcore.plugin.contracts.SmKeyValueCollection;
import com.netapp.smcore.plugin.contracts.SmLoginRequest;
import com.netapp.smcore.plugin.contracts.SmLoginResponse;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.smcore.plugin.contracts.SmResponse;
import com.netapp.smcore.plugin.contracts.SmUser;
import com.netapp.snapcenter.cli.command.validation.SmsCommandRequestValidationHelper;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.ProgressBar;
import com.netapp.snapcenter.cli.utils.ProgressBarUtil;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;
import com.netapp.snapcenter.cli.utils.TokenStoreUtil;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.opensmconnection.command.description")
public class OpenSmConnectionCommand extends AbstractSMSCommand {
    /*
     * TODO: Need to decide on to provide user to configure which server they
     * have to communicate to instead of we reading from a file.
     */
    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(OpenSmConnectionCommand.class);

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_OPENSMCONNECTION_COMMAND_NAME);

    @Parameter(names = { "-RoleName" }, descriptionKey = "cli.opensmconnection.command.rolename.option.description")
    private String roleName;

    @Parameter(names = { "-UserName" }, descriptionKey = "cli.opensmconnection.command.username.option.description")
    private String userName;

    /*
     * TODO: Should decide on having password option exposed
     */
    @Parameter(names = { "-Password" }, descriptionKey = "cli.opensmconnection.command.password.option.description", hidden = true)
    private String password;

    // Token Expiry defaults to 24 hours if not specified
    @Parameter(names = { "-TokenNeverExpires" }, descriptionKey = "cli.opensmconnection.command.tokenNeverExpires.option.description")
    private boolean tokenNeverExpires;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected void processCommandArguments() throws CLIException {

        if (getUserName() != null && StringUtils.isBlank(getUserName())) {
            CLIUtils.writeToLogAndConsole(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_OPENSMCONNECTION_USERNAME_EMPTY_ERROR,
                    getUserName()), logger, LogLevel.ERROR);
        }

        if (getRoleName() != null && StringUtils.isBlank(getRoleName())) {
            CLIUtils.writeToLogAndConsole(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_OPENSMCONNECTION_ROLENAME_EMPTY_ERROR,
                    getRoleName()), logger, LogLevel.ERROR);
        }

        CLIUtils.writeToLogAndConsole(
                MessageBundle
                        .localizeMessage(
                                CLIMsgKey.CLI_OPENSMCONNECTION_SNAPCENTER_SERVER_CONNECTION_INFO,
                                getSnapCenterServerServiceHelper()
                                        .getSnapCenterBaseURL()), logger,
                LogLevel.INFO);
        /**
         * Expiry time in not supported by Server yet and by default a token is
         * valid for 24hours.
         *
         * Long millis = new Date().getTime() + expiryTime * 60000;
         * GregorianCalendar gregorianCalendar = new GregorianCalendar();
         * gregorianCalendar.setTimeInMillis(millis); XMLGregorianCalendar
         * value; try { value =
         * DatatypeFactory.newInstance().newXMLGregorianCalendar(
         * gregorianCalendar); } catch (DatatypeConfigurationException e1) {
         * throw new CLIException(e1.getLocalizedMessage()); }
         *
         * smLoginRequest.getUserOperationContext().getUser().setTokenTime(value
         * ) ;
         */

        readUserNameAndPassword();
        if (StringUtils.isBlank(this.roleName)) {
         ProgressBar progressBar = ProgressBarUtil
        .createAndStartProgressBar(MessageBundle
                .localizeMessage(CLIMsgKey.CLI_PROGRESSBAR_VALIDATING_ROLENAME));
        validateRoleName();
        ProgressBarUtil.stopProgressBar(progressBar);
        }

    }

    @Override
    protected void processCommandResponse(SmResponse smResponse)
            throws CLIException {
        if (isCommandExecutionSuccessfull(smResponse.getResult())) {
            if (smResponse instanceof SmLoginResponse) {
                String tokenFromLoginResponse = ((SmLoginResponse) smResponse)
                        .getUser().getToken();
                TokenStoreUtil.writeToken(tokenFromLoginResponse);
            }
        }
        super.processCommandResponse(smResponse);
    }

    @Override
    protected void displayCommandResult(SmResponse smResponse)
            throws CLIException {

        SMResult smResult = smResponse.getResult();
        if (isCommandExecutionSuccessfull(smResult)) {
            CLIUtils.writeToLogAndConsole(MessageBundle
                    .localizeMessage(CLIMsgKey.CLI_SERVER_CONNECTION_OPENED),
                    logger, LogLevel.INFO);
            CLIUtils.writeToLog(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_COMMAND_EXECUTION_SUCCESS,
                    this.getCommandName()), logger, LogLevel.INFO);
        } else {
            String executionFailed = String.format("%s %s", smResult
                    .getMessage(), MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_SERVER_ERROR_CODE, smResult.getErrorCode()));
            CLIUtils.writeToLogAndConsole(executionFailed, logger,
                    LogLevel.ERROR);
        }
    }

    private void readUserNameAndPassword() {
        if (getUserName() == null) {
            String user = CLIUtils
                    .readLineFromConsole(
                            "%s: ",
                            MessageBundle
                                    .localizeMessage(CLIMsgKey.CLI_OPENSMCONNECTION_PROMPT_USER_SERVER_USERNAME));
            setUserName(user);
        }
        if (getPassword() == null) {
            String pass = CLIUtils
                    .readPasswordFromConsole(
                            "%s: ",
                            MessageBundle
                                    .localizeMessage(CLIMsgKey.CLI_OPENSMCONNECTION_PROMPT_USER_SERVER_PASSWORD));
            setPassword(pass);
        }
    }

    private void validateRoleName() throws CLIException {

        SmsCommandRequestValidationHelper smsCommandRequestValidationHelper = new SmsCommandRequestValidationHelper(
                super.getSnapCenterServerServiceHelper());
        SmGetRolesByUserResponse smGetRolesByUserResponse = smsCommandRequestValidationHelper
                .getRolesByUser(userName,getPassword());
        CLIUtils.writeToLog(smGetRolesByUserResponse.getResult().toString(),
                logger, LogLevel.INFO);

        if (smGetRolesByUserResponse.getRoles().getSmRole().size() > 1) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_OPENSMCONNECTION_COMMAND_ROLENAME_OPTION_REQUIRED));
        }

        if (StringUtils.isNotBlank(smGetRolesByUserResponse.getRoles()
                .getSmRole().get(0).getName())) {
            roleName = smGetRolesByUserResponse.getRoles().getSmRole().get(0)
                    .getName();
        }
        
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmLoginRequest, SmLoginResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext.setRequest((SmLoginRequest) cmdRequest);
        smsRestInvocationContext.setResponseClass(SmLoginResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_RBAC_LOGIN_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    public void setUserName(String userName) {
        this.userName = userName;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public String getUserName() {
        return userName;
    }

    public String getPassword() {
        return password;
    }

    public void setTokenNeverExpires(boolean tokenNeverExpires) {
        this.tokenNeverExpires = tokenNeverExpires;
    }

    public boolean isTokenNeverExpires() {
        return tokenNeverExpires;
    }

    public String getRoleName() {
        return roleName;
    }

    public void setRoleName(String roleName) {
        this.roleName = roleName;
    }
    
}
