/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/RemoveSmBackupCommand.java#4 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import java.util.ArrayList;
import java.util.List;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.SmDeleteBackupsRequest;
import com.netapp.smcore.plugin.contracts.SmDeleteBackupsResponse;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.smcore.plugin.contracts.SmResponse;
import com.netapp.snapcenter.cli.command.parser.CommaParameterSplitter;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.removesmbackup.command.description")
public class RemoveSmBackupCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_REMOVESMBACKUP_COMMAND_NAME);

    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(RemoveSmBackupCommand.class);

    @Parameter(names = { "-BackupNames" }, descriptionKey = "cli.removesmbackup.command.backupNames.option.description", splitter = CommaParameterSplitter.class)
    private List<String> backupNames = new ArrayList<String>();

    @Parameter(names = { "-BackupIds" }, descriptionKey = "cli.removesmbackup.command.backupIds.option.description", splitter = CommaParameterSplitter.class)
    private List<String> backupIds = new ArrayList<String>();

    @Parameter(names = { "-DeleteSecondaryMetadata" }, descriptionKey = "cli.removesmbackup.command.deleteSecondaryMetadata.option.description")
    private boolean deleteSecondaryMetadata;

    @Parameter(names = { "-CleanupSecondaryBackups" }, descriptionKey = "cli.removesmbackup.command.cleanupSecondaryBackups.option.description")
    private boolean cleanupSecondaryBackups;

    @Parameter(names = { "-CleanupBackups" }, descriptionKey = "cli.removesmbackup.command.cleanupBackups.option.description")
    private boolean cleanupBackups;

    @Parameter(names = { "-WaitForCompletion" }, descriptionKey = "cli.removesmbackup.command.waitForCompletion.option.description")
    private boolean waitForCompletion;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected void processCommandArguments() throws CLIException {
        if (!isCleanupSecondaryBackups() && !isCleanupBackups()) {
            if (CLIUtils.isNullOrEmptyCollection(backupNames)
                    && CLIUtils.isNullOrEmptyCollection(backupIds)) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_REMOVESMBACKUP_BACKUP_NAMES_OR_ID_NOT_SPECIFIED));
            }
            if (CLIUtils.isNotNullAndNotEmptyCollection(backupNames)
                    && CLIUtils.isNotNullAndNotEmptyCollection(backupIds)) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_REMOVESMBACKUP_BOTH_BACKUP_NAMES_AND_ID_SPECIFIED_ERROR));
            }
            if (isDeleteSecondaryMetadata()) {
                CLIUtils.writeToLogAndConsole(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_REMOVESMBACKUP_DELETE_SECONDARY_METADATA_IS_SELECTED),
                        logger, LogLevel.INFO);
            }

            CLIUtils.promptUserToContinue(MessageBundle
                    .localizeMessage(CLIMsgKey.CLI_REMOVESMBACKUP_PROMPT_USER_REMOVE_BACKUP));

        } else {
            if (CLIUtils.isNotNullAndNotEmptyCollection(backupNames)
                    || CLIUtils.isNotNullAndNotEmptyCollection(backupIds)) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_REMOVESMBACKUP_CLEANUP_BACKUPS_AND_DELETE_BACKUPS_CANNOT_HAPPEN_SAME_TIME));
            }

            if (isCleanupBackups()) {
                CLIUtils.writeToLogAndConsole(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_REMOVESMBACKUP_CLEANUP_BACKUPS_IS_SELECTED),
                        logger, LogLevel.INFO);
            }

            if (isCleanupSecondaryBackups()) {
                CLIUtils.writeToLogAndConsole(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_REMOVESMBACKUP_CLEANUP_SECONDARY_BACKUPS_IS_SELECTED),
                        logger, LogLevel.INFO);
            }

            CLIUtils.promptUserToContinue(MessageBundle
                    .localizeMessage(CLIMsgKey.CLI_REMOVESMBACKUP_PROMPT_USER_REMOVE_BACKUPS));

        }
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmDeleteBackupsRequest, SmDeleteBackupsResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext
                .setRequest((SmDeleteBackupsRequest) cmdRequest);
        smsRestInvocationContext
                .setResponseClass(SmDeleteBackupsResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_DELETE_BACKUP_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    @Override
    protected <T> void pollForJobStatus(SmResponse smResponse) throws CLIException {
        doPollForJobStatus(smResponse, isWaitForCompletion());
    }

    public List<String> getBackupNames() {
        return backupNames;
    }

    public void setBackupNames(List<String> backupNames) {
        this.backupNames = backupNames;
    }

    public List<String> getBackupIds() {
        return backupIds;
    }

    public void setBackupIds(List<String> backupIds) {
        this.backupIds = backupIds;
    }

    public void setCleanupSecondaryBackups(boolean cleanupSecondaryBackups) {
        this.cleanupSecondaryBackups = cleanupSecondaryBackups;
    }

    public boolean isCleanupSecondaryBackups() {
        return cleanupSecondaryBackups;
    }

    public void setDeleteSecondaryMetadata(boolean deleteSecondaryMetadata) {
        this.deleteSecondaryMetadata = deleteSecondaryMetadata;
    }

    public boolean isDeleteSecondaryMetadata() {
        return deleteSecondaryMetadata;
    }

    public void setWaitForCompletion(boolean waitForCompletion) {
        this.waitForCompletion = waitForCompletion;
    }

    public boolean isWaitForCompletion() {
        return waitForCompletion;
    }

    public void setCleanupBackups(boolean cleanupBackups) {
        this.cleanupBackups = cleanupBackups;
    }

    public boolean isCleanupBackups() {
        return cleanupBackups;
    }
}
