
/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/SetSmConfigSettingsCommand#1 $
 *
 * Copyright (c) 2016 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.snapcenter.cli.command;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.ConfigSettingsType;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.smcore.plugin.contracts.SmResponse;
import com.netapp.smcore.plugin.contracts.SmSetConfigSettingsRequest;
import com.netapp.snapcenter.cli.command.parser.CustomStringToMapConverter;
import com.netapp.snapcenter.cli.command.parser.CustomStringToMapConverterWithNullValueAllowed;
import com.netapp.snapcenter.cli.command.parser.NoParameterSplitter;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;
import com.netapp.spl.common.utils.HostInfoUtil;
import org.apache.commons.lang.StringUtils;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;


@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.setsmconfigsettings.command.description")
public class SetSmConfigSettingsCommand extends AbstractSMSCommand {

    private static final String CONFIGSETTINGS_KEYSTRING = "KEY";
    private static final String CONFIGSETTINGS_VALUESTRING  = "VALUE";

    public static final String COMMANDNAME = MessageBundle.localizeMessage(
            CLIMsgKey.CLI_SETSMCONFIGSETTINGS_COMMAND_NAME);


    @Parameter(names = { "-PluginCode" } , descriptionKey = "cli.setsmconfigsettings.command.pluginCode.option.description")
    private PluginCode pluginCode;


    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(SetSmConfigSettingsCommand.class);

    @Parameter(names = { "-HostName" }, descriptionKey = "cli.setsmconfigsettings.command.hostname.option.description")
    private String hostname;


    @Parameter(names = { "-ConfigSettings" }, descriptionKey = "cli.setsmconfigsettings.command.configsettings.option.description",converter = CustomStringToMapConverterWithNullValueAllowed.class, splitter = NoParameterSplitter.class, required = true)
    private List<Map<String,String>> configSettingsList = new ArrayList<>();

    @Parameter(names = { "-ConfigSettingsType" } ,descriptionKey = "cli.configsettings.command.configsettingstype.option.decription",required = true)
    ConfigSettingsType configSettingsType;


    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }


    @Override
    protected  void processCommandArguments() throws CLIException {

        if(configSettingsType==ConfigSettingsType.PLUGIN && pluginCode==null) {
            throw new CLIException(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_CONFIGSETTINGS_COMMAND_CONFIGSETTINGSTYPESETTOPLUGIN_PLUGINCODENOTSET_ERROR));
        }
        if(configSettingsType == ConfigSettingsType.PLUGIN) {
            //if Config settings is not specified, then the default type is plugin
            switch(pluginCode) {
                case SCO:
                    break;
                case SCU:
                    break;
                default:
                    throw new CLIException(MessageBundle.localizeMessage(
                            CLIMsgKey.CLI_COMMAND_NOT_SUPPORTED_FOR_PLUGIN,
                            pluginCode.name()));
            }
        }

        //Validation to check if same key is provided twice
        Set<String> configurationParameters = new TreeSet<>(String.CASE_INSENSITIVE_ORDER);
        for(Map<String,String> configs : configSettingsList) {
            String keyName = configs.get("KEY");
            if(configurationParameters.contains(keyName)){
                throw new CLIException(MessageBundle.localizeMessage(
                        CLIMsgKey.CLI_CONFIGSETTINGS_DUPLICATE_CONFIGS));
            }
            else{
                configurationParameters.add(keyName);
            }
        }

        Set<String> keyString = new HashSet<>();
        keyString.add(CONFIGSETTINGS_KEYSTRING);
        keyString.add(CONFIGSETTINGS_VALUESTRING);
        for(Map<String, String> config : configSettingsList) {
            Set<String> keySet = config.keySet();
            if (!keySet.containsAll(keyString)) {
                throw new IllegalArgumentException(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_SETSMCONFIGSETTINGS_INVALID_KEY));
            }

            if(StringUtils.isBlank(config.get(CONFIGSETTINGS_KEYSTRING))) {
                throw new CLIException(MessageBundle.localizeMessage(CLIMsgKey.CLI_PARAMETER_CANNOT_BE_BLANK));
            }
        }

        if(getHostname() != null && StringUtils.isBlank(getHostname())){
            throw new CLIException(MessageBundle.localizeMessage(CLIMsgKey.CLI_CONFIGSETTINGS_COMMAND_HOSTNAME_BLANK_ERROR));
        }

        if ((configSettingsType == ConfigSettingsType.PLUGIN) && getHostname() == null) {
            setHostname(HostInfoUtil.getCanonicalHostName());
            String displayUsingLocalhostDefault = MessageBundle
                    .localizeMessage(
                            CLIMsgKey.CLI_SETCONFIGSETTINGS_USING_LOCALHOST_AS_DEFAULT,
                            getHostname());
            CLIUtils.writeToLogAndConsole(displayUsingLocalhostDefault, logger,
                    CLIUtils.LogLevel.INFO);
        }


    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(T cmdRequest) {


        SMSRestInvocationContext<SmSetConfigSettingsRequest, SmResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext.setRequest((SmSetConfigSettingsRequest) cmdRequest);
        smsRestInvocationContext.setResponseClass(SmResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_SET_CONFIGSETTINGS_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    public ConfigSettingsType getConfigSettingsType() {
        return configSettingsType;
    }

    public void setConfigSettingsType(ConfigSettingsType configSettingsType) {
        this.configSettingsType = configSettingsType;
    }

    public String getHostname() {
        return hostname;
    }

    public void setHostname(String hostname) {
        this.hostname = hostname;
    }

    public PluginCode getPluginCode() {
        return pluginCode;
    }

    public void setPluginCode(PluginCode pluginCode) {
        this.pluginCode = pluginCode;
    }

    public List<Map<String, String>> getConfigSettingsList() {
        return configSettingsList;
    }


    public void setConfigSettingsList(List<Map<String, String>> configSettingsList) {
        this.configSettingsList = configSettingsList;
    }
}
