/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/SetSmSMTPServerCommand.java#4 $
 *
 * Copyright (c) 2017 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.snapcenter.cli.command;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.SmGetSMTPServerRequest;
import com.netapp.smcore.plugin.contracts.SmGetSMTPServerResponse;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.snapcenter.cli.command.validation.SmsCommandArgumentsValidationUtil;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.spl.common.restclient.config.HttpMethod;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.setsmsmtpserver.command.description")
public class SetSmSMTPServerCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_SETSMSMTPSERVER_COMMAND_NAME);
    private static ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(SetSmSMTPServerCommand.class);

    @Parameter(names = { "-EmailTo" }, descriptionKey = "cli.notificationParameters.emailTo.option.description", required = true)
    private String emailTo;

    @Parameter(names = { "-EmailFrom" }, descriptionKey = "cli.notificationParameters.emailFrom.option.description", required = true)
    private String emailFrom;

    @Parameter(names = { "-EmailSubject" }, descriptionKey = "cli.notificationParameters.emailSubject.option.description")
    private String emailSubject = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_SETSMSMTPSERVER_COMMAND_DEFAULT_SUBJECT);

    @Parameter(names = { "-SMTPServer" }, descriptionKey = "cli.notificationParameters.emailSMTPServer.option.description", required = true)
    private String smtpServer;

    @Parameter(names = { "-IsSendEmail" }, descriptionKey = "cli.setSmSmtpServer.command.issendmail.option.description")
    private boolean isSendEmail;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmGetSMTPServerRequest, SmGetSMTPServerResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext
                .setRequest((SmGetSMTPServerRequest) cmdRequest);
        smsRestInvocationContext
                .setResponseClass(SmGetSMTPServerResponse.class);
        smsRestInvocationContext.setHttpMethod(HttpMethod.PUT);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_SET_SMTP_SERVER_PATH);
        return smsRestInvocationContext;
    }

    @Override
    protected void processCommandArguments() throws CLIException {

        SmsCommandArgumentsValidationUtil.validateEmailToandEmailFrom(
                emailFrom, emailTo);

    }

    public String getEmailTo() {
        return emailTo;
    }

    public void setEmailTo(String emailTo) {
        this.emailTo = emailTo;
    }

    public String getEmailFrom() {
        return emailFrom;
    }

    public void setEmailFrom(String emailFrom) {
        this.emailFrom = emailFrom;
    }

    public String getEmailSubject() {
        return emailSubject;
    }

    public void setEmailSubject(String emailSubject) {
        this.emailSubject = emailSubject;
    }

    public String getSmtpServer() {
        return smtpServer;
    }

    public void setSmtpServer(String smtpServer) {
        this.smtpServer = smtpServer;
    }

    public boolean getIsSendEmail() {
        return isSendEmail;
    }

    public void setIsSendEmail(boolean isSendEmail) {
        this.isSendEmail = isSendEmail;
    }

}
