/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/UncatalogSmBackupWithOracleRMANCommand.java#4 $
 * 
 * Copyright (c) 2016 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import java.util.ArrayList;
import java.util.List;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.smcore.plugin.contracts.SmResponse;
import com.netapp.smcore.plugin.contracts.SmUncatalogBackupRequest;
import com.netapp.smcore.plugin.contracts.SmUncatalogBackupResponse;
import com.netapp.snapcenter.cli.command.parser.CommaParameterSplitter;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.uncatalogsmbackupwithoraclerman.command.description")
public class UncatalogSmBackupWithOracleRMANCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_UNCATALOGSMBACKUPWITHORACLERMAN_COMMAND_NAME);

    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(UncatalogSmBackupWithOracleRMANCommand.class);

    @Parameter(names = { "-BackupNames" }, descriptionKey = "cli.uncatalogsmbackupwithoraclerman.command.backupNames.option.description", splitter = CommaParameterSplitter.class)
    private List<String> backupNames = new ArrayList<String>();

    @Parameter(names = { "-BackupIds" }, descriptionKey = "cli.uncatalogsmbackupwithoraclerman.command.backupIds.option.description", splitter = CommaParameterSplitter.class)
    private List<String> backupIds = new ArrayList<String>();

    @Parameter(names = { "-WaitForCompletion" }, descriptionKey = "cli.uncatalogsmbackupwithoraclerman.command.waitForCompletion.option.description")
    private boolean waitForCompletion;

    private PluginCode pluginCode = PluginCode.SCO;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected void processCommandArguments() throws CLIException {
            if (CLIUtils.isNullOrEmptyCollection(backupNames)
                    && CLIUtils.isNullOrEmptyCollection(backupIds)) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_UNCATALOGSMBACKUPWITHORACLERMAN_BACKUP_NAMES_OR_ID_NOT_SPECIFIED));
            }

            if (CLIUtils.isNotNullAndNotEmptyCollection(backupNames)
                    && CLIUtils.isNotNullAndNotEmptyCollection(backupIds)) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_UNCATALOGSMBACKUPWITHORACLERMAN_BOTH_BACKUP_NAMES_AND_ID_SPECIFIED_ERROR));
            }

            CLIUtils.promptUserToContinue(MessageBundle
                    .localizeMessage(CLIMsgKey.CLI_UNCATALOGSMBACKUPWITHORACLERMAN_PROMPT_USER_UNCATALOG_BACKUPS));

    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmUncatalogBackupRequest, SmUncatalogBackupResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext
                .setRequest((SmUncatalogBackupRequest) cmdRequest);
        smsRestInvocationContext
                .setResponseClass(SmUncatalogBackupResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_UNCATALOG_BACKUP_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    @Override
    protected <T> void pollForJobStatus(SmResponse smResponse)
            throws CLIException {

        doPollForJobStatus(smResponse, isWaitForCompletion());
    }

    public List<String> getBackupNames() {
        return backupNames;
    }

    public void setBackupNames(List<String> backupNames) {
        this.backupNames = backupNames;
    }

    public List<String> getBackupIds() {
        return backupIds;
    }

    public void setBackupIds(List<String> backupIds) {
        this.backupIds = backupIds;
    }

    public boolean isWaitForCompletion() {
        return waitForCompletion;
    }

    public void setWaitForCompletion(boolean waitForCompletion) {
        this.waitForCompletion = waitForCompletion;
    }

    public PluginCode getPluginCode() {
        return pluginCode;
    }

    public void setPluginCode(PluginCode pluginCode) {
        this.pluginCode = pluginCode;
    }

}
