/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/mapper/CustomSMSRestoreObjectMapperHelper.java#1 $
 * 
 * Copyright (c) 2016 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command.mapper;

import java.util.ArrayList;
import java.util.List;

import javax.xml.datatype.XMLGregorianCalendar;

import org.apache.commons.lang3.StringUtils;

import com.netapp.smcore.plugin.contracts.*;
import com.netapp.snapcenter.cli.command.RestoreSmBackupCommand;
import com.netapp.snapcenter.cli.command.parameters.SDURestoreParameters;
import com.netapp.snapcenter.cli.command.parameters.SMORestoreParameters;
import com.netapp.snapcenter.cli.utils.CLIUtils;

/**
 * 
 * This class is used to custom map the restore smbackup command to its request
 * object. This custom mapper is required since the configuration classes and
 * component info classes need to be populated are plugin specific.
 * 
 * TODO: Need to see if this custom mapping can be removed with the help of
 * Orika Framework. Need to investigate a little more on orika capabilities, and
 * check if : 1. We can use 2 mappings where result of first can be an input to
 * the 2nd. We need to see if the result can be specified in the default mapping
 * declaration of the 2nd mapper, or a simple custom mapper of few lines for
 * setting the result is needed.2. The mapper for configuration class specific
 * to SCO/some other plugin can be registered based on the generic pluginCode
 * parameter. So, we need to see how we can achieve a clean generic/specific
 * separation in the logic which builds the mappers.
 *
 */
public class CustomSMSRestoreObjectMapperHelper {
    

    private CustomSMSRestoreObjectMapperHelper() {

    }


    public static void populateRecoveryRequest(
            RestoreSmBackupCommand restoreSmBackupCommand,
            SmRecoveryRequest smRecoveryRequest) {

        switch (restoreSmBackupCommand.getPluginCode()) {
            case SCO:
                setSmOracleRestoreConfiguration(
                        restoreSmBackupCommand.getSmoRestoreParameters(),
                        restoreSmBackupCommand.getSduRestoreParameters(),
                        smRecoveryRequest);
                setSmOracleRestoreComponentInfos(
                        restoreSmBackupCommand.getSmoRestoreParameters(),
                        smRecoveryRequest);
                // Set RAC Host in case of user specified it.
                if (StringUtils.isNotBlank(restoreSmBackupCommand
                        .getSmoRestoreParameters().getRestoreOnClusterHost())) {
                    SmHost smHost = new SmHost();
                    smHost.setHostName(restoreSmBackupCommand
                            .getSmoRestoreParameters().getRestoreOnClusterHost());
                    /*
                     * TODO: server should perform null check. for now we are only
                     * initializing it.
                     */
                    smHost.setHostPluginInfos(new ArrayOfSmPluginInfo());
                    smRecoveryRequest.setHostEx(smHost);
                }

                break;
            default:
                break;
        }

        // Map Pre-Script and Post-Script Configuration
        SmRestoreConfiguration smRestoreConfiguration = smRecoveryRequest
                .getConfiguration();
        smRestoreConfiguration.setPreScriptPath(restoreSmBackupCommand
                .getPrePostScriptParameters().getPreScriptPath());
        smRestoreConfiguration.setPreScriptArguments(restoreSmBackupCommand
                .getPrePostScriptParameters().getPreScriptArguments());
        smRestoreConfiguration.setPostScriptPath(restoreSmBackupCommand
                .getPrePostScriptParameters().getPostScriptPath());
        smRestoreConfiguration.setPostScriptArguments(restoreSmBackupCommand
                .getPrePostScriptParameters().getPostScriptArguments());
        smRestoreConfiguration.setScriptTimeOut(restoreSmBackupCommand
                .getPrePostScriptParameters().getScriptTimeout());

        // Map Notification parameters
        smRecoveryRequest.setEnableEmail(restoreSmBackupCommand.getNotificationParameters().isEnableEmail());
        smRecoveryRequest.setEmailFrom(restoreSmBackupCommand.getNotificationParameters().getEmailFrom());
        smRecoveryRequest.setEmailTo(restoreSmBackupCommand.getNotificationParameters().getEmailTo());
        smRecoveryRequest.setEmailSubject(restoreSmBackupCommand.getNotificationParameters().getEmailSubject());

    }

    /**
     * This method will set the Oracle Specific Restore Configuration selected
     * by user to corresponding request object.
     * 
     * @param smoRestoreConfigurationParameters
     * @param smRecoveryRequest
     */
    private static void setSmOracleRestoreConfiguration(
            SMORestoreParameters smoRestoreConfigurationParameters,
            SDURestoreParameters sduRestoreParameters,
            SmRecoveryRequest smRecoveryRequest) {
        SmOracleRestoreConfiguration smOracleRestoreConfiguration = new SmOracleRestoreConfiguration();

        smOracleRestoreConfiguration
                .setForceInPlaceRestore(sduRestoreParameters
                        .isForceInplaceRestore());

        smOracleRestoreConfiguration
                .setChangeDatabaseState(smoRestoreConfigurationParameters
                        .isOracleChangeState());
        smOracleRestoreConfiguration
                .setSkipRecovery(smoRestoreConfigurationParameters
                        .isOracleSkipRecovery());
        smOracleRestoreConfiguration
                .setUntilScn(smoRestoreConfigurationParameters
                        .getOracleUntilScn());

        if (smoRestoreConfigurationParameters.getOracleUntilTime() != null) {

            XMLGregorianCalendar untilTime = CLIUtils
                    .getXMLGregorianCalendarFromDate(smoRestoreConfigurationParameters
                            .getOracleUntilTime());
            smOracleRestoreConfiguration.setUntilTime(untilTime);

        }

        smOracleRestoreConfiguration
                .setControlFileRestore(smoRestoreConfigurationParameters
                        .isOracleControlFileRestore());
        smOracleRestoreConfiguration
                .setRestoreRedologFile(smoRestoreConfigurationParameters
                        .isOracleRedologFileRestore());
        smOracleRestoreConfiguration
                .setFullRestore(smoRestoreConfigurationParameters
                        .isOracleFullRestore());

        if (CLIUtils
                .isNotNullAndNotEmptyCollection(smoRestoreConfigurationParameters
                        .getAlternateArchiveLogPaths())) {
            ArrayOfSmPath logPaths = new ArrayOfSmPath();
            for (String logPath : smoRestoreConfigurationParameters
                    .getAlternateArchiveLogPaths()) {
                SmPath alternateArchiveLogsPath = new SmPath();
                alternateArchiveLogsPath.setPath(logPath);
                logPaths.getSmPath().add(alternateArchiveLogsPath);
            }
            smOracleRestoreConfiguration.setLogPaths(logPaths);
        }

        smOracleRestoreConfiguration
                .setOpenDatabaseAfterRecovery(smoRestoreConfigurationParameters
                        .isOracleOpenDatabaseAfterRecovery()
                        || smoRestoreConfigurationParameters
                                .isOracleOpenPluggableDatabaseAfterRecovery()
                        || smoRestoreConfigurationParameters
                                .isOracleOpenTablespaceAfterRecovery());

        // Powershell initializes implicitly.
        ArrayOfSmHost arrayOfSmHost = new ArrayOfSmHost();
        smRecoveryRequest.setClusterHosts(arrayOfSmHost);

        smRecoveryRequest.setConfiguration(smOracleRestoreConfiguration);
    }

    /**
     * This method will set selected oracle components to be restored into
     * request object.
     * 
     * @param smoRestoreConfigurationParameters
     * @param smRecoveryRequest
     */
    private static void setSmOracleRestoreComponentInfos(
            SMORestoreParameters smoRestoreConfigurationParameters,
            SmRecoveryRequest smRecoveryRequest) {

        ArrayOfSmBackupComponentInfo arrayOfSmBackupComponentInfo = new ArrayOfSmBackupComponentInfo();

        /*
         * Three cases: 1. Tablespaces 2. Pdbs 3. Pdb Tablespaces
         * 
         * if there is one PDB then check if it had tablespaces also and form
         * request else populate only pdbs. it no pdb then populate only
         * tablespaces.
         */
        List<String> oraclePdbs = smoRestoreConfigurationParameters
                .getOraclePluggableDatabases();
        List<String> oracleTablespaces = smoRestoreConfigurationParameters
                .getOracleTablespaces();

        if (CLIUtils.isNotNullAndNotEmptyCollection(oraclePdbs)) {

            if (oraclePdbs.size() == 1
                    && CLIUtils
                            .isNotNullAndNotEmptyCollection(oracleTablespaces)) {
                populateOraclePdbTablespaces(oraclePdbs, oracleTablespaces,
                        arrayOfSmBackupComponentInfo);
            } else {
                populateOraclePdbs(oraclePdbs, arrayOfSmBackupComponentInfo);
            }
        } else if (CLIUtils.isNotNullAndNotEmptyCollection(oracleTablespaces)) {
            populateOracleTablespaces(oracleTablespaces,
                    arrayOfSmBackupComponentInfo);
        }

        smRecoveryRequest
                .setRestoreComponentInfos(arrayOfSmBackupComponentInfo);
    }

    private static void populateOracleTablespaces(
            List<String> oracleTablespaces,
            ArrayOfSmBackupComponentInfo arrayOfSmBackupComponentInfo) {
        List<SmOracleRestoreChosenTablespace> smOracleRestoreChosenTablespaceList = createListOfSmOracleRestoreChosenTablespaceFromStringList(oracleTablespaces);
        for (SmOracleRestoreChosenTablespace smOracleRestoreChosenTablespace : smOracleRestoreChosenTablespaceList) {

            SmBackupComponentInfo smBackupComponentInfo = new SmBackupComponentInfo();
            smBackupComponentInfo
                    .setComponentObject(smOracleRestoreChosenTablespace);
            arrayOfSmBackupComponentInfo.getSmBackupComponentInfo().add(
                    smBackupComponentInfo);
        }
    }

    private static void populateOraclePdbs(List<String> oraclePdbs,
            ArrayOfSmBackupComponentInfo arrayOfSmBackupComponentInfo) {
        List<SmOracleRestoreChosenPluggableDatabase> smOracleRestoreChosenPluggableDatabasesList = createListOfSmOracleRestoreChosenPluggableDatabaseFromStringList(oraclePdbs);
        for (SmOracleRestoreChosenPluggableDatabase smOracleRestoreChosenPluggableDatabase : smOracleRestoreChosenPluggableDatabasesList) {
            SmBackupComponentInfo smBackupComponentInfo = new SmBackupComponentInfo();
            smBackupComponentInfo
                    .setComponentObject(smOracleRestoreChosenPluggableDatabase);
            arrayOfSmBackupComponentInfo.getSmBackupComponentInfo().add(
                    smBackupComponentInfo);
        }
    }

    private static void populateOraclePdbTablespaces(List<String> oraclePdbs,
            List<String> oracleTablespaces,
            ArrayOfSmBackupComponentInfo arrayOfSmBackupComponentInfo) {
        SmOracleRestoreChosenPluggableDatabase smOracleRestoreChosenPluggableDatabase = new SmOracleRestoreChosenPluggableDatabase();
        smOracleRestoreChosenPluggableDatabase.setName(oraclePdbs.get(0));

        List<SmOracleRestoreChosenTablespace> SmOracleRestoreChosenPdbTablespaces = createListOfSmOracleRestoreChosenTablespaceFromStringList(oracleTablespaces);
        ArrayOfSmOracleRestoreChosenTablespace arrayOfSmOracleRestoreChosenTablespace = new ArrayOfSmOracleRestoreChosenTablespace();
        arrayOfSmOracleRestoreChosenTablespace
                .getSmOracleRestoreChosenTablespace().addAll(
                        SmOracleRestoreChosenPdbTablespaces);
        smOracleRestoreChosenPluggableDatabase
                .setSmOracleRestoreChosenTablespaces(arrayOfSmOracleRestoreChosenTablespace);

        SmBackupComponentInfo smBackupComponentInfo = new SmBackupComponentInfo();
        smBackupComponentInfo
                .setComponentObject(smOracleRestoreChosenPluggableDatabase);
        arrayOfSmBackupComponentInfo.getSmBackupComponentInfo().add(
                smBackupComponentInfo);
    }

    private static List<SmOracleRestoreChosenPluggableDatabase> createListOfSmOracleRestoreChosenPluggableDatabaseFromStringList(
            List<String> oraclePdbs) {
        List<SmOracleRestoreChosenPluggableDatabase> smOracleRestoreChosenPluggableDatabaseList = new ArrayList<>();
        for (String pdb : oraclePdbs) {
            SmOracleRestoreChosenPluggableDatabase smOracleRestoreChosenPluggableDatabase = new SmOracleRestoreChosenPluggableDatabase();
            smOracleRestoreChosenPluggableDatabase.setName(pdb);
            smOracleRestoreChosenPluggableDatabaseList
                    .add(smOracleRestoreChosenPluggableDatabase);
        }
        return smOracleRestoreChosenPluggableDatabaseList;
    }

    private static List<SmOracleRestoreChosenTablespace> createListOfSmOracleRestoreChosenTablespaceFromStringList(
            List<String> oracleTablespaces) {
        List<SmOracleRestoreChosenTablespace> smOracleRestoreChosenTablespaceList = new ArrayList<>();
        for (String tablespace : oracleTablespaces) {
            SmOracleRestoreChosenTablespace smOracleRestoreChosenTablespace = new SmOracleRestoreChosenTablespace();
            smOracleRestoreChosenTablespace.setName(tablespace);
            smOracleRestoreChosenTablespaceList
                    .add(smOracleRestoreChosenTablespace);
        }
        return smOracleRestoreChosenTablespaceList;
    }

}
