/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/output/formatter/invokesmresourceclonesplit/InvokeSmResourceCloneSplitCommandEstimateOptionOutputFormatter.java#1 $
 *
 * Copyright (c) 2017 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.snapcenter.cli.command.output.formatter.invokesmresourceclonesplit;

import org.apache.commons.lang.StringUtils;

import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.SmCloneResourceSplitEstimateResponse;
import com.netapp.smcore.plugin.contracts.SmCloneResourceSplitResponse;
import com.netapp.smcore.plugin.contracts.VolumeAggregateSpaceMapping;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.formatter.Formatter;
import com.netapp.snapcenter.cli.formatter.FormatterFactory;
import com.netapp.snapcenter.cli.formatter.FormatterType;
import com.netapp.snapcenter.cli.formatter.TextTable;
import com.netapp.snapcenter.cli.formatter.TextTableInterface;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;

/**
 * 
 * This class Prints the output for estimate option of Invoke-SmResourceSplit in
 * Text, Table, XML and JSON format
 *
 */

public class OptionEstimateOutputFormatter implements OutputFormatter {

    private static ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(OptionEstimateOutputFormatter.class);

    @Override
    public <T> void formatAndPrintCommandResponseObject(T smResponse,
            FormatterType formatterType) throws CLIException {
        SmCloneResourceSplitEstimateResponse smCloneResourceSplitEstimateResponse = (SmCloneResourceSplitEstimateResponse) smResponse;

        if (StringUtils.isNotBlank(smCloneResourceSplitEstimateResponse
                .getResult().getMessage())) {
            String errorMsg = MessageBundle
                    .localizeMessage(
                            CLIMsgKey.CLI_COMMAND_INVOKESMCLONERESOURCESPLIT_RESPONSE_ERROR,
                            smCloneResourceSplitEstimateResponse.getResult()
                                    .getMessage());
            throw new CLIException(errorMsg);
        }

        if (!smCloneResourceSplitEstimateResponse.isCanSplit()) {
            String errorMsg = MessageBundle
                    .localizeMessage(
                            CLIMsgKey.CLI_COMMAND_INVOKESMCLONERESOURCESPLIT_CAN_SPLIT_ERROR,
                            smCloneResourceSplitEstimateResponse.getResult()
                                    .getMessage());
            throw new CLIException(errorMsg);
        }

        switch (formatterType) {
            case TEXT:
                printOutputInTextFormat(smCloneResourceSplitEstimateResponse);
                break;
            case JSON:
            case XML:
                logger.debug("output formatter "
                        + smCloneResourceSplitEstimateResponse);
                Formatter formatter = FormatterFactory.getFormatter(formatterType);
                formatter
                        .writeFormattedObject(smCloneResourceSplitEstimateResponse);
                break;
            case TEXT_TABLE:
            default:
                printOutputInTableFormat(smCloneResourceSplitEstimateResponse);
        }
    }

    /**
     * Prints the output in table format.
     * 
     * @param smCloneResourceSplitEstimateResponse
     */
    @Override
    public void printOutputInTableFormat(
            SmCloneResourceSplitResponse smCloneResourceSplitResponse)
            throws CLIException {

        SmCloneResourceSplitEstimateResponse smCloneResourceSplitEstimateResponse = (SmCloneResourceSplitEstimateResponse) smCloneResourceSplitResponse;

        printOutputJobSummaryInTableFormat(smCloneResourceSplitEstimateResponse);
        printOutputOfEstimateInTableFormat(smCloneResourceSplitEstimateResponse);

    }

    private void printOutputJobSummaryInTableFormat(
            SmCloneResourceSplitEstimateResponse smCloneResourceSplitEstimateResponse) {

        String headerJobId = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_COMMON_TABLE_JOB_ID_HEADER);
        String headerCanSplit = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_ESTIMATE_TABLE_JOB_SUMMARY_CAN_SPLIT_HEADER);
        String headerAppObjectId = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_COMMON_TABLE_APP_OBJECT_ID_HEADER);
        String headerCloneName = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_COMMON_TABLE_CLONE_NAME_HEADER);

        String[] resourcesTableHeaders = { headerJobId, headerCanSplit,
                headerAppObjectId, headerCloneName };

        TextTableInterface resourcesTable = new TextTable(resourcesTableHeaders);

        String columnJobId = Long.toString(smCloneResourceSplitEstimateResponse
                .getJobId());
        String columnCanSplit = Boolean
                .toString(smCloneResourceSplitEstimateResponse.isCanSplit());

        String columnAppObjectId = smCloneResourceSplitEstimateResponse
                .getAppObjectId();
        String columnCloneName = smCloneResourceSplitEstimateResponse
                .getCloneName();
        resourcesTable.addRow(columnJobId, columnCanSplit, columnAppObjectId,
                columnCloneName);

        CLIUtils.writeToConsole(MessageBundle
                .localizeMessage(CLIMsgKey.CLI_COMMAND_INVOKESMCLONERESOURCESPLIT_ESTIMATE_JOB_SUMMARY_DETAILS_HEADER));
        resourcesTable.displayFormattedTable();
    }

    private void printOutputOfEstimateInTableFormat(
            SmCloneResourceSplitEstimateResponse smCloneResourceSplitEstimateResponse)
            throws CLIException {

        if (smCloneResourceSplitEstimateResponse.getListVolumeAggSpaceMapping()
                .getVolumeAggregateSpaceMapping() == null) {
            String errorMsg = MessageBundle
                    .localizeMessage(CLIMsgKey.CLI_COMMAND_INVOKESMCLONERESOURCESPLIT_RESPONSE_VOLUME_AGGREGATE_SPACE_MAPPING_ERROR);
            throw new CLIException(errorMsg);
        }

        String headerAggregateName = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_ESTIMATE_TABLE_AGGREGATE_NAME_HEADER);
        String headerAvailableSpaceInAggregate = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_ESTIMATE_TABLE_AVAILABLE_SPACE_IN_AGGREGATE_HEADER);
        String headerRequiredSpace = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_ESTIMATE_TABLE_REQUIRED_SPACE_HEADER);
        String headerVolumeName = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_ESTIMATE_TABLE_VOLUME_NAME_HEADER);
        String headerIsVolumeAlreadySplit = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_ESTIMATE_TABLE_IS_VOLUME_ALREADY_SPLIT_HEADER);
        String headerStorageSystemName = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_ESTIMATE_TABLE_STORAGE_SYSTEM_NAME_HEADER);

        String[] resourcesTableHeaders = { headerVolumeName,
                headerAggregateName, headerAvailableSpaceInAggregate,
                headerRequiredSpace, headerIsVolumeAlreadySplit,
                headerStorageSystemName };

        TextTableInterface resourcesTable = new TextTable(resourcesTableHeaders);
        for (VolumeAggregateSpaceMapping volumeAggregateSpaceMapping : smCloneResourceSplitEstimateResponse
                .getListVolumeAggSpaceMapping()
                .getVolumeAggregateSpaceMapping()) {

            String columnAggregateName = volumeAggregateSpaceMapping
                    .getAggregateName();
            String columnAvailableSpaceInAggregate = volumeAggregateSpaceMapping
                    .getAvailableSpaceInAggregate().toString()
                    + " "
                    + volumeAggregateSpaceMapping.getAggregateSizeUnit();
            String columnRequiredSpace = null;
            String columnIsVolumeAlreadySplit = null;
            if (volumeAggregateSpaceMapping.getRequiredSpace() == -1) {
                columnRequiredSpace = "0";
                columnIsVolumeAlreadySplit = "True";
            } else {
                columnRequiredSpace = Long.toString(volumeAggregateSpaceMapping
                        .getRequiredSpace())
                        + " "
                        + volumeAggregateSpaceMapping.getVolumeSizeUnit();
                columnIsVolumeAlreadySplit = "False";
            }

            String columnVolumeName = volumeAggregateSpaceMapping
                    .getVolumeName();

            String columnStorageSystemName = volumeAggregateSpaceMapping
                    .getStorageSystemName();

            resourcesTable.addRow(columnVolumeName, columnAggregateName,
                    columnAvailableSpaceInAggregate, columnRequiredSpace,
                    columnIsVolumeAlreadySplit, columnStorageSystemName);

        }

        CLIUtils.writeToConsole(MessageBundle
                .localizeMessage(CLIMsgKey.CLI_COMMAND_INVOKESMCLONERESOURCESPLIT_ESTIMATE_DETAILS_HEADER));
        resourcesTable.displayFormattedTable();

    }

    /**
     * 
     * Prints the output in text format.
     * 
     * @param smCloneResourceSplitEstimateResponse
     */
    @Override
    public void printOutputInTextFormat(
            SmCloneResourceSplitResponse smCloneResourceSplitResponse)
            throws CLIException {

        SmCloneResourceSplitEstimateResponse smCloneResourceSplitEstimateResponse = (SmCloneResourceSplitEstimateResponse) smCloneResourceSplitResponse;
        printOutputJobSummeryInTextFormat(smCloneResourceSplitEstimateResponse);
        printOutputOfEstimateInTextFormat(smCloneResourceSplitEstimateResponse);

    }

    private void printOutputJobSummeryInTextFormat(
            SmCloneResourceSplitEstimateResponse smCloneResourceSplitEstimateResponse) {

        StringBuilder commandOutput = new StringBuilder();

        String jobId = MessageBundle
                .localizeMessage(
                        CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_COMMON_TEXTVIEW_JOB_ID,
                        Long.toString(smCloneResourceSplitEstimateResponse
                                .getJobId()));
        commandOutput.append(jobId);
        commandOutput.append(CLIUtils.NEW_LINE);

        String canSplit = MessageBundle
                .localizeMessage(
                        CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_ESTIMATE_JOB_SUMMARY_TEXTVIEW_CAN_SPLIT,
                        Boolean.toString(smCloneResourceSplitEstimateResponse
                                .isCanSplit()));
        commandOutput.append(canSplit);
        commandOutput.append(CLIUtils.NEW_LINE);

        String appObjectId = MessageBundle
                .localizeMessage(
                        CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_COMMON_TEXTVIEW_APP_OBJECT_ID,
                        smCloneResourceSplitEstimateResponse.getAppObjectId());
        commandOutput.append(appObjectId);
        commandOutput.append(CLIUtils.NEW_LINE);

        String cloneName = MessageBundle
                .localizeMessage(
                        CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_COMMON_TEXTVIEW_CLONE_NAME,
                        smCloneResourceSplitEstimateResponse.getCloneName());
        commandOutput.append(cloneName);
        commandOutput.append(CLIUtils.NEW_LINE);
        commandOutput.append(CLIUtils.NEW_LINE);

        CLIUtils.writeToConsole(MessageBundle
                .localizeMessage(CLIMsgKey.CLI_COMMAND_INVOKESMCLONERESOURCESPLIT_ESTIMATE_JOB_SUMMARY_DETAILS_HEADER));
        CLIUtils.writeToConsole("=================================");
        CLIUtils.writeToConsole(commandOutput.toString());
    }

    private void printOutputOfEstimateInTextFormat(
            SmCloneResourceSplitEstimateResponse smCloneResourceSplitEstimateResponse)
            throws CLIException {

        if (smCloneResourceSplitEstimateResponse.getListVolumeAggSpaceMapping()
                .getVolumeAggregateSpaceMapping() == null) {
            String errorMsg = MessageBundle
                    .localizeMessage(CLIMsgKey.CLI_COMMAND_INVOKESMCLONERESOURCESPLIT_RESPONSE_VOLUME_AGGREGATE_SPACE_MAPPING_ERROR);
            throw new CLIException(errorMsg);
        }

        StringBuilder commandOutput = new StringBuilder();

        for (VolumeAggregateSpaceMapping volumeAggregateSpaceMapping : smCloneResourceSplitEstimateResponse
                .getListVolumeAggSpaceMapping()
                .getVolumeAggregateSpaceMapping()) {

            String volumeName = MessageBundle
                    .localizeMessage(
                            CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_ESTIMATE_TEXTVIEW_VOLUME_NAME,
                            volumeAggregateSpaceMapping.getVolumeName());
            commandOutput.append(volumeName);
            commandOutput.append(CLIUtils.NEW_LINE);

            String aggregateName = MessageBundle
                    .localizeMessage(
                            CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_ESTIMATE_TEXTVIEW_AGGREGATE_NAME,
                            volumeAggregateSpaceMapping.getAggregateName());
            commandOutput.append(aggregateName);
            commandOutput.append(CLIUtils.NEW_LINE);

            String availableSpaceInAggregate = MessageBundle
                    .localizeMessage(
                            CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_ESTIMATE_TEXTVIEW_AVAILABLE_SPACE_IN_AGGREGATE,
                            volumeAggregateSpaceMapping
                                    .getAvailableSpaceInAggregate().toString()
                                    + " "
                                    + volumeAggregateSpaceMapping
                                            .getAggregateSizeUnit());
            commandOutput.append(availableSpaceInAggregate);
            commandOutput.append(CLIUtils.NEW_LINE);

            String alreadySplit = null;

            if (volumeAggregateSpaceMapping.getRequiredSpace() == -1) {
                String noSpaceError = MessageBundle
                        .localizeMessage(
                                CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_ESTIMATE_TEXTVIEW_REQUIRED_SPACE,
                                "0");
                commandOutput.append(noSpaceError);
                commandOutput.append(CLIUtils.NEW_LINE);

                alreadySplit = MessageBundle
                        .localizeMessage(
                                CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_ESTIMATE_TEXTVIEW_IS_VOLUME_ALREADY_SPLIT,
                                "True");
                commandOutput.append(alreadySplit);
                commandOutput.append(CLIUtils.NEW_LINE);
            } else {
                String requiredSpace = MessageBundle
                        .localizeMessage(
                                CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_ESTIMATE_TEXTVIEW_REQUIRED_SPACE,
                                Long.toString(volumeAggregateSpaceMapping
                                        .getRequiredSpace())
                                        + " "
                                        + volumeAggregateSpaceMapping
                                                .getVolumeSizeUnit());
                commandOutput.append(requiredSpace);
                commandOutput.append(CLIUtils.NEW_LINE);

                alreadySplit = MessageBundle
                        .localizeMessage(
                                CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_ESTIMATE_TEXTVIEW_IS_VOLUME_ALREADY_SPLIT,
                                "False");

            }

            commandOutput.append(alreadySplit);
            commandOutput.append(CLIUtils.NEW_LINE);

            String storageSystemName = MessageBundle
                    .localizeMessage(
                            CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_ESTIMATE_TEXTVIEW_STORAGE_SYSTEM_NAME,
                            volumeAggregateSpaceMapping.getStorageSystemName());
            commandOutput.append(storageSystemName);
            commandOutput.append(CLIUtils.NEW_LINE);
            commandOutput.append(CLIUtils.NEW_LINE);
        }
        CLIUtils.writeToConsole(MessageBundle
                .localizeMessage(CLIMsgKey.CLI_COMMAND_INVOKESMCLONERESOURCESPLIT_ESTIMATE_DETAILS_HEADER));
        CLIUtils.writeToConsole("===========================================================");
        CLIUtils.writeToConsole(commandOutput.toString());

    }
}
