/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/storage/preferences/GetPreferredStorageExportIpsCommand.java#1 $
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command.storage.preferences;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.snapcenter.cli.command.CommandInterface;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.formatter.TextTable;
import com.netapp.snapcenter.cli.formatter.TextTableInterface;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.getpreferredstorageexportips.command.description")
public class GetPreferredStorageExportIpsCommand extends AbstractStoragePreferencesCommand
        implements CommandInterface {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETPREFERREDSTORAGEEXPORTIPS_COMMAND_NAME);

    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(GetPreferredStorageExportIpsCommand.class);

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    public void run() throws CLIException {

        initializeStoragePreferencesProperties();

        String headerPreferredStorageExportIps = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETPREFERREDSTORAGEEXPORTIPS_TABLE_PREFERRED_STORAGE_EXPORT_IPS_HEADER);

        String[] preferredStorageExportIpsTableHeaders = { headerPreferredStorageExportIps };
        TextTableInterface preferredStorageExportIpsTable = new TextTable(
                preferredStorageExportIpsTableHeaders);

        if (getStoragePreferencesProperty(PREFERRED_IPADDRESSES_FOR_STORAGE_EXPORT_KEY) != null) {
            for (String ip : getStoragePreferencesProperty(
                    PREFERRED_IPADDRESSES_FOR_STORAGE_EXPORT_KEY).split(",")) {
                preferredStorageExportIpsTable.addRow(StringUtils.trim(ip));
            }
        }

        preferredStorageExportIpsTable.displayFormattedTable();

        String executionSucess = MessageBundle.localizeMessage(
                CLIMsgKey.CLI_COMMAND_EXECUTION_SUCCESS, this.getCommandName());
        CLIUtils.writeToLogAndConsole(executionSucess, logger, LogLevel.INFO);
    }

}