/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/validation/DataSetCommandParametersValidationUtil.java#1 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.snapcenter.cli.command.validation;

import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.ArrayOfSmPolicy;
import com.netapp.smcore.plugin.contracts.ArrayOfSmSchedule;
import com.netapp.smcore.plugin.contracts.SmAppPolicy;
import com.netapp.smcore.plugin.contracts.SmBackupPolicyVerificationSettings;
import com.netapp.smcore.plugin.contracts.SmBackupVerificationType;
import com.netapp.smcore.plugin.contracts.SmOraclePolicy;
import com.netapp.smcore.plugin.contracts.SmPluginCode;
import com.netapp.smcore.plugin.contracts.SmPolicy;
import com.netapp.smcore.plugin.contracts.SmPolicyType;
import com.netapp.smcore.plugin.contracts.SmSchedule;
import com.netapp.smcore.plugin.contracts.SmSchedulerType;
import com.netapp.smcore.plugin.contracts.SmString;
import com.netapp.smcore.plugin.contracts.SmvalidatePolicyResponse;
import com.netapp.snapcenter.cli.command.mapper.CustomSMSPolicyObjectMapperHelper;
import com.netapp.snapcenter.cli.command.parameters.BackupResourceGroupParameters;
import com.netapp.snapcenter.cli.command.parameters.ResourceGroupScheduleParameters;
import com.netapp.snapcenter.cli.command.parameters.ResourceGroupVerificationParameters;
import com.netapp.snapcenter.cli.command.parameters.SMOBackupDataSetParameters;
import com.netapp.snapcenter.cli.command.parser.CustomStringToSchedulesMapConverter;
import com.netapp.snapcenter.cli.contracts.BackupVerificationType;
import com.netapp.snapcenter.cli.contracts.EmailNotificationPreference;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.contracts.ScheduleType;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;

import javax.xml.datatype.XMLGregorianCalendar;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import static com.netapp.snapcenter.cli.command.parameters.BackupResourceGroupParameters.CUSTOM_SNAPSHOT_CUSTOMTEXT_FORMAT;
import static com.netapp.snapcenter.cli.command.parameters.BackupResourceGroupParameters.CUSTOM_SNAPSHOT_DATASET_FORMAT;
import static com.netapp.snapcenter.cli.command.parameters.BackupResourceGroupParameters.CUSTOM_SNAPSHOT_HOSTNAME_FORMAT;
import static com.netapp.snapcenter.cli.command.parameters.BackupResourceGroupParameters.CUSTOM_SNAPSHOT_POLICY_FORMAT;
import static com.netapp.snapcenter.cli.command.parameters.BackupResourceGroupParameters.CUSTOM_SNAPSHOT_SCHEDULETYPE_FORMAT;
import static com.netapp.snapcenter.cli.command.parameters.BackupResourceGroupParameters.CUSTOM_TEXT_MAX_LENGTH;
import static com.netapp.snapcenter.cli.command.parameters.BackupResourceGroupParameters.PARAMETER_CUSTOM_SNAPSHOT;
import static com.netapp.snapcenter.cli.command.parameters.BackupResourceGroupParameters.PARAMETER_CUSTOM_TEXT;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.ORACLE_DATABASE;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.RESOURCE_HOST;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.RESOURCE_NAMES;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.RESOURCE_NAMES_DELIMETER;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.RESOURCE_TYPE;

/**
 * This class contains dataset command related arguments validation.
 *
 */
public class DataSetCommandParametersValidationUtil {

    private DataSetCommandParametersValidationUtil() {

    }

    private static ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(DataSetCommandParametersValidationUtil.class);

    private static final String OPTION_RESOURCEGROUP = "-ResourceGroupName";
    private static final int DATASET_NAME_MAX_LENGTH = 250;

    public static void validateDatasetName(String datasetName)
            throws CLIException {
        if (StringUtils.isBlank(datasetName)) {
            throw new CLIException(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_PARAMETER_CANNOT_BE_BLANK,
                    OPTION_RESOURCEGROUP));
        } else if (datasetName.length() > DATASET_NAME_MAX_LENGTH) {
            throw new CLIException(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_PARAMETER_LENGTH_INVALID,
                    OPTION_RESOURCEGROUP, DATASET_NAME_MAX_LENGTH));
        }
    }

    public static void validateIfPolicyIsAssociatedWithSchedules(
            List<String> policies, List<Map<String, String>> schedules)
            throws CLIException {
        if ((schedules != null && schedules.size() > 0)
                && (policies == null || !(policies.size() > 0))) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_HOST_RESOURCE_MAPPING_SCHEDULE_NEEDS_POLICY));
        }
    }

    public static List<SmPolicy> validateAndBuildPolicyAndSchedules(
            List<String> policies,
            List<ResourceGroupScheduleParameters> schedulesParamsList,
            Map<String, List<SmSchedulerType>> policyVsScheduleTypeMap)
            throws CLIException {

        List<SmPolicy> smPolicyList = new ArrayList<SmPolicy>();
        // This map holds the schedule types specified in the option -Schedules
        // for each of the policies.
        Map<String, List<SmSchedulerType>> policyVsScheduleTypeMapOfOptionSchedules = new HashMap<String, List<SmSchedulerType>>();

        for (String policy : policies) {
            SmPolicy smPolicy = new SmPolicy();
            smPolicy.setName(policy);
            smPolicyList.add(smPolicy);
            smPolicy.setAllowMultipleSchedules(true);
            boolean policyHasSchedule = false;
            List<SmSchedulerType> scheduleTypeList = policyVsScheduleTypeMap
                    .get(policy);
            if (schedulesParamsList != null && schedulesParamsList.size() > 0) {
                for (ResourceGroupScheduleParameters scheduleParams : schedulesParamsList) {

                    String policyName = scheduleParams.getPolicyName();
                    SmSchedulerType scheduleType = SmSchedulerType
                            .fromValue(scheduleParams.getScheduleType().value());

                    if (CLIUtils
                            .isEmpty(policyVsScheduleTypeMapOfOptionSchedules
                                    .get(policyName))) {

                        List<SmSchedulerType> list = new ArrayList<SmSchedulerType>();
                        list.add(SmSchedulerType
                                .fromValue(ScheduleType
                                        .fromValue(
                                                scheduleParams
                                                        .getScheduleType()
                                                        .name()).value()));
                        policyVsScheduleTypeMapOfOptionSchedules.put(
                                policyName, list);
                    } else {
                        policyVsScheduleTypeMapOfOptionSchedules
                                .get(policyName).add(
                                        SmSchedulerType
                                                .fromValue(scheduleParams
                                                        .getScheduleType()
                                                        .value()));
                    }

                    /*
                     * It validates the Policy name of -Schedules with the
                     * policies specified for -Policies
                     */
                    if (!policies.contains(policyName)) {
                        throw new CLIException(
                                MessageBundle
                                        .localizeMessage(CLIMsgKey.CLI_ADDSMRESOURCEGROUP_COMMAND_SCHEDULE_POLICY_NAME_NOT_MATCHING));
                    }

                    if (policy.equals(policyName)) {
                        CLIUtils.writeToLog(String.format(
                                "The policy %s is found to have a schedules.",
                                policy), logger, LogLevel.DEBUG);

                        if (scheduleTypeList == null) {
                            CLIUtils.writeToLog(
                                    String.format("The policy could not find schedules"),
                                    logger, LogLevel.DEBUG);

                        }

                        for (SmSchedulerType smscheduleType : scheduleTypeList) {

                            CLIUtils.writeToLog(
                                    String.format(
                                            "The policy '%s' has schedule '%s' attached",
                                            policy, smscheduleType.name()),
                                    logger, LogLevel.DEBUG);

                            if (scheduleType.equals(smscheduleType)) {
                                CLIUtils.writeToLog(
                                        String.format(
                                                "The policy %s is found to have the input schedule type of '%s'",
                                                policy, scheduleType.name()),
                                        logger, LogLevel.DEBUG);
                                policyHasSchedule = true;
                            }
                        }
                        if (!policyHasSchedule) {

                            throw new CLIException(
                                    MessageBundle
                                            .localizeMessage(
                                                    CLIMsgKey.CLI_ADDSMRESOURCEGROUP_COMMAND_SCHEDULE_NOT_PART_OF_POLICY,
                                                    scheduleType.value(),
                                                    policy));
                        }
                        // policy has schedule
                        createSchedulesInPolicy(scheduleParams, smPolicy);

                    }

                }

                // Verify whether option -Schedules have all the schedule
                // details for all the backup schedule types that are defined in
                // each of the policy specified under -Policies
                for (String policyName : policyVsScheduleTypeMap.keySet()) {
                    if (!CLIUtils.isEmpty(policyVsScheduleTypeMap
                            .get(policyName))) {
                        CLIUtils.removeDuplicateEntriesFromList(policyVsScheduleTypeMapOfOptionSchedules
                                .get(policyName));
                        if ((policyVsScheduleTypeMapOfOptionSchedules.get(
                                policyName).size() != policyVsScheduleTypeMap
                                .get(policyName).size())
                                || (!policyVsScheduleTypeMap.get(policyName)
                                        .containsAll(
                                                policyVsScheduleTypeMapOfOptionSchedules
                                                        .get(policyName)))) {
                            throw new CLIException(
                                    MessageBundle
                                            .localizeMessage(CLIMsgKey.CLI_SCHEDULES_MAPPING_SHOULD_BE_DEFINED_FOR_ALL_THE_POLICIES_WHICH_CONTAINS_SCHEDULE_TYPES));
                        }
                    }
                }
            } else {
                // If there are no schedules then send as empty list, otherwise
                // it will fail validation in server  
                ArrayOfSmSchedule arrayOfSmSchedule = new ArrayOfSmSchedule();
                smPolicy.setSchedules(arrayOfSmSchedule);
            }
            CLIUtils.writeToLog("The policy " + smPolicy.getName()
                    + " is recoreded", logger, LogLevel.DEBUG);
        }

        return smPolicyList;

    }


    public static void buildSmBackupPolicyVerificationSettings(
            List<SmBackupPolicyVerificationSettings> listSmBackupPolicyVerificationSettings,
            List<ResourceGroupVerificationParameters> listResourceGroupVerificationParameters)
            throws CLIException {

     for (ResourceGroupVerificationParameters resourceGroupVerificationParameters : listResourceGroupVerificationParameters) {
            SmBackupPolicyVerificationSettings smBackupPolicyVerificationSettings = new SmBackupPolicyVerificationSettings();
            smBackupPolicyVerificationSettings
                         .setBackupPolicyName(resourceGroupVerificationParameters
                                       .getPolicyName());
            smBackupPolicyVerificationSettings
                         .setScheduleType((resourceGroupVerificationParameters
                                       .getBackupscheduleType()));
            smBackupPolicyVerificationSettings
                         .setDeferedVerificationBackupCnt(resourceGroupVerificationParameters
                                       .getDefferedBackupCount());
            smBackupPolicyVerificationSettings
                         .setVerifyOnSecondaryStorage(resourceGroupVerificationParameters
                                       .isVerifyOnSecondary());
            smBackupPolicyVerificationSettings
                         .setEnumVerificationType(SmBackupVerificationType
                                       .fromValue(resourceGroupVerificationParameters
                                                     .getVerificationType().name()));

            SmSchedule smSchedule = new SmSchedule();

            if (resourceGroupVerificationParameters.getVerificationType()
                         .equals(BackupVerificationType.VERIFY_SCHEDULED)) {

                  switch (resourceGroupVerificationParameters
                                .getBackupscheduleType()) {
                  case ONE_TIME:
                         break;

                  case HOURLY:
                         smSchedule
                                       .setRepeatTaskEveryHour(resourceGroupVerificationParameters
                                                     .getRepeatTaskEveryHour());
                         smSchedule
                         .setRepeatTaskEveryMins(resourceGroupVerificationParameters
                                       .getRepeatTaskEveryMins());
                         break;

                  case DAILY:
                         if (resourceGroupVerificationParameters.getDaysInterval() != -1) {
                                smSchedule
                                              .setDaysInterval((short) resourceGroupVerificationParameters
                                                            .getDaysInterval());
                         }
                         if (resourceGroupVerificationParameters
                                       .getOccursEveryMinute() >= 0) {
                                smSchedule
                                              .setIntervalDuration(resourceGroupVerificationParameters
                                                            .getOccursEveryMinute());
                         }
                         break;

                  case WEEKLY:
                         if (resourceGroupVerificationParameters.getDaysOfTheWeek() != null) {
                                String daysOfTheWeekToString = StringUtils.join(
                                              resourceGroupVerificationParameters
                                                            .getDaysOfTheWeek(), ',');
                                smSchedule.setDaysOfTheWeek(daysOfTheWeekToString);
                         }
                         if (resourceGroupVerificationParameters
                                       .getOccursEveryMinute() >= 0) {
                                smSchedule
                                              .setIntervalDuration(resourceGroupVerificationParameters
                                                            .getOccursEveryMinute());
                         }
                         break;

                  case MONTHLY:
                         if (resourceGroupVerificationParameters.getDaysOfTheMonth() != null) {
                                String daysoftheMonthToString = StringUtils.join(
                                              resourceGroupVerificationParameters
                                                            .getDaysOfTheMonth(), ',');
                                smSchedule.setDaysoftheMonth(daysoftheMonthToString);

                                if (resourceGroupVerificationParameters
                                              .getMonthsOfTheYear() != null) {
                                       String monthofTheYearToString = StringUtils.join(
                                                     resourceGroupVerificationParameters
                                                                  .getMonthsOfTheYear(), ',');
                                       smSchedule
                                                     .setMonthsofTheYear(monthofTheYearToString);
                                } else {
                                       smSchedule
                                                     .setMonthlyFrequency(resourceGroupVerificationParameters
                                                                  .getOccursEveryMonth());
                                }
                         }
                         if (resourceGroupVerificationParameters
                                       .getOccursEveryMinute() >= 0) {
                                smSchedule
                                              .setIntervalDuration(resourceGroupVerificationParameters
                                                            .getOccursEveryMinute());
                         }
                         break;

                  case NONE:

                  default:
                         break;
                  }

                  if (resourceGroupVerificationParameters.getStartTime() != null) {
                      XMLGregorianCalendar startTime = CLIUtils
                              .getXMLGregorianCalendarFromDate(resourceGroupVerificationParameters
                                      .getStartTime());
                      smSchedule.setStartTime(startTime);
                  }

                  if (resourceGroupVerificationParameters.getEndTime() != null) {
                      XMLGregorianCalendar endTime = CLIUtils
                              .getXMLGregorianCalendarFromDate(resourceGroupVerificationParameters
                                      .getEndTime());
                      smSchedule.setEndTime(endTime);
                  }
            }

            smSchedule.setSchedulerType(SmSchedulerType
                         .fromValue(resourceGroupVerificationParameters
                                       .getScheduleType().value()));

            smBackupPolicyVerificationSettings
                         .setVerificationSchedule(smSchedule);

            smBackupPolicyVerificationSettings
                         .setBackupScheduleType((resourceGroupVerificationParameters
                                       .getBackupscheduleType()));

            listSmBackupPolicyVerificationSettings
                         .add(smBackupPolicyVerificationSettings);
     }

}

  
    private static void createSchedulesInPolicy(
            ResourceGroupScheduleParameters scheduleParams, SmPolicy smPolicy) {

        SmSchedule smSchedule = new SmSchedule();
        /*
         * Below two line are commented coz, server it is setting it to windows
         * scheduler
         */
        // smSchedule.setLocalScheduler(false);
        // smSchedule.setAppType(false);
        SmSchedulerType smSchedulerType = SmSchedulerType
                .valueOf(scheduleParams.getScheduleType().name());
        smSchedule.setSchedulerType(smSchedulerType);
        if (smPolicy.getSchedules() == null) {
            ArrayOfSmSchedule arrayOfSmSchedule = new ArrayOfSmSchedule();
            smPolicy.setSchedules(arrayOfSmSchedule);
        }
        smPolicy.getSchedules().getSmSchedule().add(smSchedule);
        CustomSMSPolicyObjectMapperHelper.addPolicyTriggers(
                scheduleParams.getDaysInterval(),
                scheduleParams.getDaysOfTheWeek(),
                scheduleParams.getDaysOfTheMonth(),
                scheduleParams.getMonthsOfTheYear(),
                scheduleParams.getRepeatTaskEveryHour(),
                scheduleParams.getRepeatTaskEveryMins(),
                scheduleParams.getOccursEveryMinute(),
                scheduleParams.getOccursEveryMonth(), smSchedule);

        if (scheduleParams.getStartTime() != null) {
            XMLGregorianCalendar startTime = CLIUtils
                    .getXMLGregorianCalendarFromDate(scheduleParams
                            .getStartTime());
            smSchedule.setStartTime(startTime);
        }

        if (scheduleParams.getEndTime() != null) {
            XMLGregorianCalendar endTime = CLIUtils
                    .getXMLGregorianCalendarFromDate(scheduleParams
                            .getEndTime());
            smSchedule.setEndTime(endTime);
        }
        CLIUtils.writeToLog(String.format(
                "Created a schedule %s attached to policy %s ",
                smSchedule.getSchedulerType(), smPolicy.getName()), logger,
                LogLevel.DEBUG);

    }

    // Also validate duplicate host resource
    public static void validateHostResources(
            List<Map<String, String>> resources, PluginCode pluginCode)
            throws CLIException {
        if (resources == null) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_HOST_RESOURCE_MAPPING_RESOURCES_NOT_SPECIFIED));
        }
        Set<String> resourceHosts = new TreeSet<>(String.CASE_INSENSITIVE_ORDER);
        for (Map<String, String> res : resources) {
            String hostName = res.get(RESOURCE_HOST);
            if (resourceHosts.contains(hostName)) {
                throw new CLIException(MessageBundle.localizeMessage(
                        CLIMsgKey.CLI_HOST_RESOURCE_MAPPING_DUPLICATE_HOST,
                        hostName));
            } else {
                resourceHosts.add(hostName);
            }

            String resType = res.get(RESOURCE_TYPE);
            if (pluginCode.equals(PluginCode.SCO)) {
                if (resType == null
                        || !resType.equalsIgnoreCase(ORACLE_DATABASE)) {
                    throw new IllegalArgumentException(
                            MessageBundle
                                    .localizeMessage(CLIMsgKey.CLI_HOST_RESOURCE_MAPPING_FOR_SMO_RESOURCE_TYPE_SHOULD_BE_ORACLE_DATABASE));

                }
            }

            String[] resourceNames = res.get(RESOURCE_NAMES).split(
                    RESOURCE_NAMES_DELIMETER);
            Set<String> uniqueResourcesNames = new HashSet<>(
                    Arrays.asList(resourceNames));
            if (uniqueResourcesNames.size() != resourceNames.length) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_HOST_RESOURCE_MAPPING_DUPLICATE_RESOURCES,
                                        res.get(RESOURCE_HOST)));
            }
        }
    }

    public static void processAndValidateSMOBackupDataSetParameters(
            SMOBackupDataSetParameters smoBackupDataSetParameters)
            throws CLIException {
        for (String excludeLogs : smoBackupDataSetParameters
                .getExcludeArchiveLogDestinationsFromBackup()) {
            if (StringUtils.isBlank(excludeLogs)) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_SMO_BACKUP_DATA_SET_PARAMETERS_EXCLUDE_ARCHIVE_LOGS_FROM_BACKUP_EMPTY));
            }
        }
    }

    public static void processAndValidateResourceGroupParameters(
            BackupResourceGroupParameters bdsParams) throws CLIException {

        CLIUtils.writeToLog(
                "Processing Resource Group Parameters " + bdsParams, logger,
                LogLevel.DEBUG);

        if (validateAndSetCustomSnapshotFormat(bdsParams)) {
            bdsParams.setCustomSnapshot(true);
        }

        if (bdsParams.getSecondaryLocators() != null
                && bdsParams.getSecondaryLocators().size() > 0) {
            SmsCommandArgumentsValidationUtil
                    .validateSecondaryLocators(bdsParams.getSecondaryLocators());
        }

        if (bdsParams.getNotificationParameters().isEnableEmail()
                && bdsParams.getNotificationParameters()
                        .getEmailNotificationPreference() == null) {
            bdsParams.getNotificationParameters()
                    .setEmailNotificationPreference(
                            EmailNotificationPreference.ALWAYS);
        } else if (bdsParams.getNotificationParameters()
                .getEmailNotificationPreference() != null
                && !bdsParams.getNotificationParameters()
                        .getEmailNotificationPreference()
                        .equals(EmailNotificationPreference.NEVER)) {
            bdsParams.getNotificationParameters().setEnableEmail(true);
        }
    }

    private static boolean validateAndSetCustomSnapshotFormat(
            BackupResourceGroupParameters bdsParams) throws CLIException {
        if (bdsParams.getCustomSnapshotFormat() == null) {
            if (bdsParams.getCustomText() != null) {
                throw new CLIException(MessageBundle.localizeMessage(
                        CLIMsgKey.CLI_PARAMETER_REQUIRES_ANOTHER_PARAMETER,
                        PARAMETER_CUSTOM_TEXT, PARAMETER_CUSTOM_SNAPSHOT));
            }
        } else if (StringUtils.isBlank(bdsParams.getCustomSnapshotFormat())) {
            throw new CLIException(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_PARAMETER_CANNOT_BE_BLANK,
                    PARAMETER_CUSTOM_SNAPSHOT));
        } else {
            String[] customSnapshotFormatFields = bdsParams
                    .getCustomSnapshotFormat().split("\\$");
            if (customSnapshotFormatFields.length > 0) {
                Set<String> customSnapshotFormatSet = new LinkedHashSet<>();
                if (StringUtils.isEmpty(customSnapshotFormatFields[0])) {
                    customSnapshotFormatFields = ArrayUtils.subarray(
                            customSnapshotFormatFields, 1,
                            customSnapshotFormatFields.length);
                } else {
                    throw new CLIException(
                            MessageBundle
                                    .localizeMessage(
                                            CLIMsgKey.CLI_PARAMETER_CUSTOM_SNAPSHOT_FORMAT_ENTRY_INVALID,
                                            customSnapshotFormatFields[0]));
                }
                for (String splitString : customSnapshotFormatFields) {
                    switch (splitString.toLowerCase()) {
                        case "resourcegroup":
                            customSnapshotFormatSet
                                    .add(CUSTOM_SNAPSHOT_DATASET_FORMAT);
                            break;
                        case "policy":
                            customSnapshotFormatSet
                                    .add(CUSTOM_SNAPSHOT_POLICY_FORMAT);
                            break;
                        case "hostname":
                            customSnapshotFormatSet
                                    .add(CUSTOM_SNAPSHOT_HOSTNAME_FORMAT);
                            break;
                        case "customtext":
                            customSnapshotFormatSet
                                    .add(CUSTOM_SNAPSHOT_CUSTOMTEXT_FORMAT);
                            break;
                        case "scheduletype":
                            customSnapshotFormatSet
                                    .add(CUSTOM_SNAPSHOT_SCHEDULETYPE_FORMAT);
                        default:
                            throw new CLIException(
                                    MessageBundle
                                            .localizeMessage(
                                                    CLIMsgKey.CLI_PARAMETER_CUSTOM_SNAPSHOT_FORMAT_ENTRY_INVALID,
                                                    splitString));
                    }
                }
                bdsParams.setCustomSnapshotFormat(StringUtils.join(
                        customSnapshotFormatSet, ""));

                String customText = bdsParams.getCustomText();
                if (bdsParams.getCustomSnapshotFormat().contains(
                        PARAMETER_CUSTOM_TEXT)) {
                    if (customText == null) {
                        throw new CLIException(
                                MessageBundle
                                        .localizeMessage(
                                                CLIMsgKey.CLI_PARAMETER_CUSTOM_TEXT_NOT_SPECIFIED,
                                                PARAMETER_CUSTOM_TEXT));
                    } else if (StringUtils.isBlank(customText)) {
                        throw new CLIException(MessageBundle.localizeMessage(
                                CLIMsgKey.CLI_PARAMETER_CANNOT_BE_BLANK,
                                PARAMETER_CUSTOM_TEXT));
                    } else if (customText.length() > CUSTOM_TEXT_MAX_LENGTH) {
                        throw new CLIException(MessageBundle.localizeMessage(
                                CLIMsgKey.CLI_PARAMETER_LENGTH_INVALID,
                                PARAMETER_CUSTOM_TEXT, CUSTOM_TEXT_MAX_LENGTH));
                    }
                } else if (customText != null) {
                    throw new CLIException(MessageBundle.localizeMessage(
                            CLIMsgKey.CLI_PARAMETER_REQUIRES_ANOTHER_PARAMETER,
                            PARAMETER_CUSTOM_TEXT, PARAMETER_CUSTOM_SNAPSHOT));
                }
                return true;
            } else {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_PARAMETER_CUSTOM_SNAPSHOT_FORMAT_INVALID));
            }
        }
        return false;
    }

    public static void validateBackupPolicies(
            SmvalidatePolicyResponse smvalidatePolicyResponse,
            PluginCode pluginCode) throws CLIException {
        SmPolicyType validPolicyType = SmPolicyType.BACKUP;
        SmPluginCode validAppPolicyType = SmPluginCode.fromValue(pluginCode
                .name());

        if (smvalidatePolicyResponse.getPolicies() != null
                && smvalidatePolicyResponse.getPolicies().getSmString().size() > 0) {
            StringBuilder nonExistentPolicies = new StringBuilder();
            for (SmString pol : smvalidatePolicyResponse.getPolicies()
                    .getSmString()) {
                nonExistentPolicies.append(pol.getValue());
                nonExistentPolicies.append(", ");
            }
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(
                                    CLIMsgKey.CLI_CREATION_OR_MODIFIY_DATASET_POLICY_DOESNOT_EXISTS,
                                    nonExistentPolicies.substring(0,
                                            nonExistentPolicies.length() - 2)));
        }

        ArrayOfSmPolicy arrayOfSmPolicy = smvalidatePolicyResponse
                .getAvailablePolicies();
        for (SmPolicy smPolicy : arrayOfSmPolicy.getSmPolicy()) {
            if (!smPolicy.getPolicyType().equals(validPolicyType)) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_CREATION_OR_MODIFIY_DATASET_NOT_A_BACKUP_POLICY,
                                        smPolicy.getName()));
            }
            if (smPolicy.getAppPolicies() == null
                    || smPolicy.getAppPolicies().getSmAppPolicy().size() == 0) {
                CLIUtils.writeToLog(
                        "No Application policies attached with policy "
                                + smPolicy.getName(), logger, LogLevel.DEBUG);
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_CREATION_OR_MODIFIY_DATASET_INVALID_BACKUP_POLICY,
                                        smPolicy.getName()));
            }
            SmAppPolicy smAppPolicy = smPolicy.getAppPolicies()
                    .getSmAppPolicy().iterator().next();
            if (pluginCode.equals(PluginCode.SCO)) {
                if (!smPolicy.getPluginPolicyType().equals(validAppPolicyType)
                        || !(smAppPolicy instanceof SmOraclePolicy)) {
                    throw new CLIException(
                            MessageBundle
                                    .localizeMessage(
                                            CLIMsgKey.CLI_CREATION_OR_MODIFIY_DATASET_INVALID_ORACLE_POLICY,
                                            smPolicy.getName()));
                }
            }
        }
    }

    public static void validateIfDuplicatePolicySchedulesProvided (List<String> policiesList, List<Map<String, String>> schedules) throws CLIException {
        List<String> policyScheduleProvided = new ArrayList<>();
        for(Map<String, String> schedule : schedules){
            String policyName = schedule.get(CustomStringToSchedulesMapConverter.POLICY_NAME);
            String scheduleName = schedule.get(CustomStringToSchedulesMapConverter.SCHEDULE_TYPE);
            if(StringUtils.isBlank(policyName) || StringUtils.isBlank(scheduleName)){
                throw new CLIException(MessageBundle.localizeMessage(CLIMsgKey.CLI_SCHEDULES_MAPPING_REQUIRED_ARGUMENTS));
            }
            if(!policiesList.contains(policyName)){
                throw new CLIException(MessageBundle.localizeMessage(CLIMsgKey.CLI_ADDSMRESOURCEGROUP_POLICYNOTPROVIDEDINPOLICIESPARAMETER,policyName));
            }
            if(policyScheduleProvided.contains(policyName)){
                throw new CLIException(MessageBundle.localizeMessage(CLIMsgKey.CLI_ADDSMRESOURCEGROUP_MULTIPLESCHEDULESFORPOLICY_ERROR,policyName));
            }
            policyScheduleProvided.add(policyName);
        }
    }

    public static void validateSchedulerRunAsName (BackupResourceGroupParameters backupResourceGroupParameters) throws CLIException {
        // if schedule exists then runAs should be specified
        if(backupResourceGroupParameters.getSchedulerRunAsName() != null) {
            if (StringUtils.isEmpty(backupResourceGroupParameters
                    .getSchedulerRunAsName()) ) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_ADDSMRESOURCEGROUPCOMMMAND_SCHEDULERRUNASNAME_EMPTY_ERROR));

            }
        }
    }

    public static String validateTag (String tag) throws CLIException {

        if(StringUtils.isBlank(tag)){
            throw new CLIException(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_PARAMETER_CANNOT_BE_BLANK,
                    "Tag"));
        }

        String[] tags = tag.split(",");
        for(int i=0;i<tags.length;i++){
            tags[i] = tags[i].trim();
        }
        String validatedTag = StringUtils.join(tags,",");

        return validatedTag;
    }


}
