/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/validation/ResourceGroupVerificationParametersValidationUtil.java#1 $
 * 
 * Copyright (c) 2016 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command.validation;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.builder.ToStringBuilder;

import com.beust.jcommander.ParameterException;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.ArrayOfSmPolicy;
import com.netapp.smcore.plugin.contracts.SmBackupPolicy;
import com.netapp.smcore.plugin.contracts.SmPolicy;
import com.netapp.smcore.plugin.contracts.SmRetention;
import com.netapp.smcore.plugin.contracts.SmSchedule;
import com.netapp.smcore.plugin.contracts.SmSchedulerType;
import com.netapp.snapcenter.cli.command.parameters.ResourceGroupVerificationParameters;
import com.netapp.snapcenter.cli.command.parser.CustomStringToDateConverter;
import com.netapp.snapcenter.cli.command.parser.CustomStringToSchedulesMapConverter;
import com.netapp.snapcenter.cli.command.parser.CustomStringToVerificationSchedulesMapConverter;
import com.netapp.snapcenter.cli.contracts.BackupVerificationType;
import com.netapp.snapcenter.cli.contracts.ScheduleType;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;

public class ResourceGroupVerificationParametersValidationUtil {

    private static ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(ResourceGroupVerificationParametersValidationUtil.class);

    public ResourceGroupVerificationParametersValidationUtil() {

    }

    public static List<ResourceGroupVerificationParameters> processVerificationScheduleParameters(
            List<Map<String, String>> verificationSchedules,
            ArrayOfSmPolicy policies, List<String> listOfPolicies)
            throws CLIException {

        CLIUtils.writeToLog(
                "Processing verification Schedules Parameters "
                        + com.netapp.common.util.StringUtils
                                .reflectiveToString(verificationSchedules),
                logger, LogLevel.DEBUG);
        Map<String, List<SmSchedulerType>> policyVsScheduleTypeMap = new HashMap<String, List<SmSchedulerType>>();

        List<ResourceGroupVerificationParameters> verificationSchedulesParamsList = new ArrayList<ResourceGroupVerificationParameters>();
        for (Map<String, String> verificationSchedule : verificationSchedules) {

            ResourceGroupVerificationParameters resourceGroupVerificationParameters = new ResourceGroupVerificationParameters();
            for (Map.Entry<String, String> scheduleEntry : verificationSchedule
                    .entrySet()) {

                if (scheduleEntry
                        .getKey()
                        .equalsIgnoreCase(
                                CustomStringToVerificationSchedulesMapConverter.VERIFICATION_TYPE)) {

                    resourceGroupVerificationParameters
                            .setVerificationType(BackupVerificationType
                                    .fromValue(scheduleEntry.getValue()));
                }

                if (scheduleEntry
                        .getKey()
                        .equalsIgnoreCase(
                                CustomStringToVerificationSchedulesMapConverter.BACKUP_POLICY_NAME)) {
                    String backupPolicyName = scheduleEntry.getValue();
                    // Check backupPolicyName specified in -VerificationSchedule
                    // should belong to one the -Policies
                    if (!listOfPolicies.contains(backupPolicyName)) {
                        throw new CLIException(
                                MessageBundle
                                        .localizeMessage(
                                                CLIMsgKey.CLI_VERIFICATION_BACKUP_POLICY_NOT_BELONGS_TO_SPECIFIED_POLICIES,
                                                backupPolicyName));
                    }
                    resourceGroupVerificationParameters
                            .setPolicyName(backupPolicyName);
                }

                if (scheduleEntry
                        .getKey()
                        .equalsIgnoreCase(
                                CustomStringToVerificationSchedulesMapConverter.VERIFICATION_ON_SECONDARY)) {
                    resourceGroupVerificationParameters
                            .setVerifyOnSecondary(Boolean
                                    .parseBoolean(scheduleEntry.getValue()));
                }

                if (scheduleEntry
                        .getKey()
                        .equalsIgnoreCase(
                                CustomStringToVerificationSchedulesMapConverter.DEFERRED_BACKUP_COUNT)) {
                    resourceGroupVerificationParameters
                            .setDefferedBackupCount(Integer
                                    .parseInt(scheduleEntry.getValue()));
                }

                if (scheduleEntry
                        .getKey()
                        .equalsIgnoreCase(
                                CustomStringToVerificationSchedulesMapConverter.BACKUP_SCHEDULE_TYPE)) {
                    if (ScheduleType.fromValue(scheduleEntry.getValue()).equals(ScheduleType.NONE)) {
                        throw new CLIException( 
                                MessageBundle
                                        .localizeMessage(CLIMsgKey.CLI_VERIFICATION_SCHEDULES_MAPPING_REQUIRED_SCHEDULE_TYPE));
                    }
                    resourceGroupVerificationParameters
                            .setBackupscheduleType(SmSchedulerType
                                    .fromValue(ScheduleType.fromValue(
                                            scheduleEntry.getValue()).value()));
                }

                // Fill common schedule parameters
                ResourceGroupScheduleParametersValidationUtil
                        .fillGenericScheduleParameters(scheduleEntry,
                                resourceGroupVerificationParameters);

            }

            // Check for generic required options
            if ((resourceGroupVerificationParameters.getBackupscheduleType() == null)
                    || (resourceGroupVerificationParameters
                            .getVerificationType() == null)
                    || (StringUtils.isEmpty(resourceGroupVerificationParameters
                            .getPolicyName()))) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_VERIFICATION_SCHEDULES_MAPPING_GENERIC_REQUIRED_ARGUMENTS));
            }

            // validate the parsed schedule
            if (resourceGroupVerificationParameters.getVerificationType()
                    .equals(BackupVerificationType.VERIFY_SCHEDULED)) {
                // Check for required options for scheduled verification
                if ((resourceGroupVerificationParameters.getScheduleType()
                        .equals(ScheduleType.NONE))
                        || (resourceGroupVerificationParameters.getStartTime() == null)) {
                    throw new CLIException(
                            MessageBundle
                                    .localizeMessage(CLIMsgKey.CLI_VERIFICATION_SCHEDULES_MAPPING_GENERIC_REQUIRED_ARGUMENTS_FOR_SCHEDULED_VERIFICATION));
                }

                if (resourceGroupVerificationParameters.getScheduleType()
                        .intValue() < ScheduleType.fromValue(
                        resourceGroupVerificationParameters
                                .getBackupscheduleType().name()).intValue()) {
                    throw new ParameterException(
                            MessageBundle
                                    .localizeMessage(CLIMsgKey.CLI_VERIFICATION_SCHEDULE_TYPES_CANNOT_BE_LOWER_THAN_BACKUP_SCHEDULE_TYPES));
                }
                PolicyCommandParametersValidationUtil
                        .processAndValidateCommonBackupCloneVerificationPolicyParameters(resourceGroupVerificationParameters);
            } else {
                // check if schedule details are provided for the
                Set<String> resourceKeys = new HashSet<>();

                resourceKeys
                        .add(CustomStringToSchedulesMapConverter.START_DATE_TIME);
                resourceKeys
                        .add(CustomStringToSchedulesMapConverter.END_DATE_TIME);
                resourceKeys
                        .add(CustomStringToSchedulesMapConverter.REPEATE_EVERY_HOUR);
                resourceKeys
                        .add(CustomStringToSchedulesMapConverter.OCCURS_EVERY_MINUTE);
                resourceKeys
                        .add(CustomStringToSchedulesMapConverter.OCCURS_EVERY_MONTH);
                resourceKeys
                        .add(CustomStringToSchedulesMapConverter.DAYS_INTERVAL);
                resourceKeys
                        .add(CustomStringToSchedulesMapConverter.MONTHS_OF_THE_YEAR);
                resourceKeys
                        .add(CustomStringToSchedulesMapConverter.DAYS_OF_THE_WEEK);
                resourceKeys
                        .add(CustomStringToSchedulesMapConverter.DAYS_OF_THE_MONTH);

                if (!Collections.disjoint(verificationSchedule.keySet(), resourceKeys)) {
                    throw new CLIException(
                            MessageBundle
                                    .localizeMessage(CLIMsgKey.CLI_VERIFICATION_SCHEDULES_MAPPING_SCHEDULE_DETAILS_ARE_NOT_REQUIRED_FOR_VERIFY_AFTER_BACKUP));
                }
            }

            String backupPolicyName = resourceGroupVerificationParameters
                    .getPolicyName();
            SmSchedulerType scheduleType = SmSchedulerType
                    .fromValue(resourceGroupVerificationParameters
                            .getBackupscheduleType().value());

            // Verify the Backup schedule type is part of the backup policy
            // and verify verification is enabled for that backup schedule type
            if (scheduleType != SmSchedulerType.NONE) {
                boolean isValidVerificationSchedule = false;
                boolean isVerificationEnabledForSchedule = false;

                for (SmPolicy smPolicy : policies.getSmPolicy()) {
                    if (smPolicy instanceof SmBackupPolicy) {
                        SmBackupPolicy smBackupPolicy = (SmBackupPolicy) smPolicy;

                        if (smBackupPolicy.getName().equalsIgnoreCase(
                                backupPolicyName)) {
                            for (SmSchedule sch : smBackupPolicy.getSchedules()
                                    .getSmSchedule()) {
                                if (sch.getSchedulerType() == scheduleType) {
                                    isValidVerificationSchedule = true;
                                }
                            }

                            for (SmRetention smRetention : smBackupPolicy
                                    .getRetentions().getSmRetention()) {
                                if ((smRetention.getSchedulerType()
                                        .equals(scheduleType))
                                        && (smRetention.isVerificationEnabled())) {
                                    isVerificationEnabledForSchedule = true;
                                }
                            }

                        }
                    }

                }
                if (!isValidVerificationSchedule) {
                    throw new CLIException(
                            MessageBundle
                                    .localizeMessage(
                                            CLIMsgKey.CLI_VERIFICATION_SCHEDULES_MAPPING_SHOULD_BELONG_TO_BACKUP_SCHEDULE,
                                            scheduleType));

                }

                if (!isVerificationEnabledForSchedule) {
                    throw new CLIException(
                            MessageBundle
                                    .localizeMessage(
                                            CLIMsgKey.CLI_BACKUP_SCHEDULE_TYPES_NOT_ENABLED_FOR_VERIFICATION,
                                            scheduleType));

                }

            }

            verificationSchedulesParamsList
                    .add(resourceGroupVerificationParameters);
        }

        return verificationSchedulesParamsList;

    }

    @Override
    public String toString() {
        return ToStringBuilder.reflectionToString(this);
    }

}
