/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/validation/SmsBackupValidationHelper.java#1 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.snapcenter.cli.command.validation;

import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.ORACLE_DATABASE;

import java.util.Collections;
import java.util.Comparator;

import javax.xml.datatype.XMLGregorianCalendar;

import org.apache.commons.lang3.StringUtils;

import com.netapp.smcore.plugin.contracts.ArrayOfSmBackup;
import com.netapp.smcore.plugin.contracts.ArrayOfSmObject;
import com.netapp.smcore.plugin.contracts.SmBackup;
import com.netapp.smcore.plugin.contracts.SmBackupFilter;
import com.netapp.smcore.plugin.contracts.SmGetArchivedDetailsRequest;
import com.netapp.smcore.plugin.contracts.SmGetArchivedDetailsResponse;
import com.netapp.smcore.plugin.contracts.SmGetBackupRequest;
import com.netapp.smcore.plugin.contracts.SmGetBackupResponse;
import com.netapp.smcore.plugin.contracts.SmObject;
import com.netapp.smcore.plugin.contracts.SmPluginCode;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.helper.SnapCenterServerServiceHelper;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

/**
 * This helper class contains snapcenter backup service calls. It provides the
 * API's for validating backups.
 *
 */
public class SmsBackupValidationHelper {

    private SnapCenterServerServiceHelper snapCenterServerServiceHelper;

    public static final String ORACLE_DATA_BACKUP_TYPE = "Oracle Database Data Backup";
    public static final String ORACLE_LOG_BACKUP_TYPE = "Oracle Database Log Backup";
    public static final String ORACLE = "Oracle";

    public SmsBackupValidationHelper(SnapCenterServerServiceHelper serviceHelper) {
        this.setSnapCenterServerServiceHelper(serviceHelper);
    }

    public SmGetArchivedDetailsResponse getSecondaryBackupDetails(
            String backupId, String appObjectId, PluginCode pluginCode)
            throws CLIException {
        return getSecondaryBackupDetails(backupId, appObjectId, pluginCode,
                false);
    }

    public SmGetArchivedDetailsResponse getSecondaryBackupDetails(
            String backupId, String appObjectId, PluginCode pluginCode,
            boolean getLogArchivedLocators) throws CLIException {
       SmGetArchivedDetailsResponse response = new SmGetArchivedDetailsResponse();

        SmGetArchivedDetailsRequest request = new SmGetArchivedDetailsRequest();

        ArrayOfSmObject arrayOfSmObject = new ArrayOfSmObject();
        SmObject smObject = new SmObject();
        smObject.setId(appObjectId);
        switch (pluginCode) {
            case SCO:
                smObject.setType(ORACLE_DATABASE);
                break;

            default:
                break;
        }
        arrayOfSmObject.getSmObject().add(smObject);
        request.setAppObjects(arrayOfSmObject);

        request.setPluginCode(SmPluginCode.valueOf(pluginCode.name()));

        if (StringUtils.isNotBlank(backupId)) {
            SmBackup smBackup = new SmBackup();
            smBackup.setBackupId(backupId);
            request.setBackup(smBackup);
        }

        request.setGetLogArchivedLocators(getLogArchivedLocators);

        SMSRestInvocationContext<?, ?> smsRestInvocationContext = getSnapCenterServerServiceHelper()
                .createSMSRestInvocationContext(
                        request,
                        SmGetArchivedDetailsResponse.class,
                        SMSRestConfiguration.SMS_GET_ARCHIVED_DETAILS_SERVICE_PATH);
        response = (SmGetArchivedDetailsResponse) getSnapCenterServerServiceHelper()
                .makeRestCallToSnapCenterServer(smsRestInvocationContext);

        if (!getSnapCenterServerServiceHelper()
                .isSnapCenterRestOperationResponseResultSuccess(
                        response.getResult())) {
            throw new CLIException(response.getResult().getMessage());
        }
        return response;
    }

    public SmGetBackupResponse getBackupInformation(
            GetBackupInformationRequest backupInfoReq) throws CLIException {
        SmGetBackupResponse response = new SmGetBackupResponse();

        SmGetBackupRequest request = new SmGetBackupRequest();

        SmBackup smBackup = new SmBackup();
        smBackup.setBackupId(backupInfoReq.getBackupId());
        smBackup.setBackupName(backupInfoReq.getBackupName());
        request.setBackup(smBackup);

        SmObject smObject = new SmObject();
        smObject.setId(backupInfoReq.getAppObjectId());
        request.setAppObject(smObject);

        SmBackupFilter smBackupFilter = new SmBackupFilter();
        smBackupFilter.setBackupType(backupInfoReq.getBackupType());
        smBackupFilter.setIsSecondary(backupInfoReq.isSecondary());
        smBackupFilter.setPluginCode(SmPluginCode.valueOf(backupInfoReq
                .getPluginCode().name()));
        request.setBackupFilter(smBackupFilter);

        SMSRestInvocationContext<?, ?> smsRestInvocationContext = getSnapCenterServerServiceHelper()
                .createSMSRestInvocationContext(request,
                        SmGetBackupResponse.class,
                        SMSRestConfiguration.SMS_GET_BACKUPS_SERVICE_PATH);
        response = (SmGetBackupResponse) getSnapCenterServerServiceHelper()
                .makeRestCallToSnapCenterServer(smsRestInvocationContext);

        if (!getSnapCenterServerServiceHelper()
                .isSnapCenterRestOperationResponseResultSuccess(
                        response.getResult())) {
            throw new CLIException(response.getResult().getMessage());
        }
        return response;
    }

    public GetRecentBackupInformationResponse getRecentOracleDataBackupInformation(
            GetRecentBackupInformationRequest request) throws CLIException {
        GetRecentBackupInformationResponse response = new GetRecentBackupInformationResponse();

        SmGetBackupResponse smGetBackupResponse = getBackupInformation(request);

        ArrayOfSmBackup arrayOfSmBackup = smGetBackupResponse.getBackups();
        if (arrayOfSmBackup != null
                && CLIUtils.isNotNullAndNotEmptyCollection(arrayOfSmBackup
                        .getSmBackup())) {
            response.setNumberOfBackups(arrayOfSmBackup.getSmBackup().size());
            /*
             * Sorting based on backup time so that latest backup will be at
             * top.
             */
            Collections.sort(arrayOfSmBackup.getSmBackup(),
                    new Comparator<SmBackup>() {

                        @Override
                        public int compare(SmBackup arg1, SmBackup arg2) {
                            XMLGregorianCalendar backupTime1 = arg1
                                    .getBackupTime();
                            XMLGregorianCalendar backupTime2 = arg2
                                    .getBackupTime();
                            if (backupTime1 == null && backupTime2 == null) {
                                return 0;
                            } else if (backupTime1 != null
                                    && backupTime2 == null) {
                                return 1;
                            } else if (backupTime1 == null
                                    && backupTime2 != null) {
                                return -1;
                            } else {
                                return backupTime2.compare(backupTime1);
                            }
                        }
                    });
            if (request.getRecentBackupNumber() < arrayOfSmBackup.getSmBackup()
                    .size()) {
                SmBackup recentBackup = arrayOfSmBackup.getSmBackup().get(
                        request.getRecentBackupNumber());
                response.setRecentBackup(recentBackup);
            }
        }

        return response;
    }

    public boolean checkIfValidOracleDataBackup(String backupId,
            String backupName, String appObjectId, boolean secondary)
            throws CLIException {

        GetBackupInformationRequest req = createGetBackupInformationRequest(
                backupId, backupName, appObjectId, secondary, PluginCode.SCO);

        return checkIfValidBackup(req, ORACLE_DATA_BACKUP_TYPE);
    }

    public boolean checkIfValidOracleBackup(String backupId, String backupName,
            String appObjectId, boolean secondary) throws CLIException {

        GetBackupInformationRequest req = createGetBackupInformationRequest(
                backupId, backupName, appObjectId, secondary, PluginCode.SCO);

        return checkIfValidBackup(req, ORACLE);
    }

    public String fetchBackupIdIfItIsValidOracleDataBackup(String backupName,
            String appObjectId, boolean secondary) throws CLIException {
        String backupId = null;

        GetBackupInformationRequest req = createGetBackupInformationRequest(
                backupId, backupName, appObjectId, secondary, PluginCode.SCO);

        SmGetBackupResponse smGetBackupResponse = getBackupInformation(req);

        SmBackup backup = getFirstBackupFromSmGetBackupResponse(smGetBackupResponse);
        if (backup != null && StringUtils.isNotBlank(backup.getBackupId())
                && StringUtils.isNotBlank(backup.getBackupName())
                && StringUtils.isNotBlank(backup.getBackupType())) {
            if (backup.getBackupType().toLowerCase()
                    .contains(ORACLE_DATA_BACKUP_TYPE.toLowerCase())
                    && backup.getBackupName().equalsIgnoreCase(
                            req.getBackupName())) {
                backupId = backup.getBackupId();
            }
        }

        return backupId;
    }

	public String fetchBackupIdIfItIsValidOracleBackup(String backupName,
			String appObjectId, boolean secondary) throws CLIException {
		String backupId = null;

		GetBackupInformationRequest req = createGetBackupInformationRequest(
				backupId, backupName, appObjectId, secondary, PluginCode.SCO);

		SmGetBackupResponse smGetBackupResponse = getBackupInformation(req);

		SmBackup backup = getFirstBackupFromSmGetBackupResponse(smGetBackupResponse);
		if (backup != null && StringUtils.isNotBlank(backup.getBackupId())) {
			backupId = backup.getBackupId();
		}

		return backupId;
	}

	private boolean checkIfValidBackup(GetBackupInformationRequest req,
			String backupType) throws CLIException {
		boolean validBackup = false;

		SmGetBackupResponse smGetBackupResponse = getBackupInformation(req);

		SmBackup backup = getFirstBackupFromSmGetBackupResponse(smGetBackupResponse);
		if (backup != null && StringUtils.isNotBlank(backup.getBackupId())
				&& StringUtils.isNotBlank(backup.getBackupName())
				&& StringUtils.isNotBlank(backup.getBackupType())) {
			if (backup.getBackupType().toLowerCase()
					.contains(backupType.toLowerCase())
					&& (backup.getBackupId()
							.equalsIgnoreCase(req.getBackupId()) || backup
							.getBackupName().equalsIgnoreCase(
									req.getBackupName()))) {
				validBackup = true;
			}
		}

		return validBackup;
	}

    public SmBackup getFirstBackupFromSmGetBackupResponse(
            SmGetBackupResponse smGetBackupResponse) {
        if (smGetBackupResponse != null
                && smGetBackupResponse.getBackups() != null
                && !smGetBackupResponse.getBackups().getSmBackup().isEmpty()) {
            return smGetBackupResponse.getBackups().getSmBackup().iterator()
                    .next();
        }
        return null;
    }

    private GetBackupInformationRequest createGetBackupInformationRequest(
            String backupId, String backupName, String appObjectId,
            boolean secondary, PluginCode pluginCode) {
        GetBackupInformationRequest req = new GetBackupInformationRequest();
        req.setBackupId(backupId);
        req.setBackupName(backupName);
        req.setAppObjectId(appObjectId);
        req.setPluginCode(pluginCode);
        req.setSecondary(secondary);
        return req;
    }

    public SnapCenterServerServiceHelper getSnapCenterServerServiceHelper() {
        return snapCenterServerServiceHelper;
    }

    public void setSnapCenterServerServiceHelper(
            SnapCenterServerServiceHelper snapCenterServerServiceHelper) {
        this.snapCenterServerServiceHelper = snapCenterServerServiceHelper;
    }

}
