/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/validation/SmsCommandArgumentsValidationUtil.java#1 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.snapcenter.cli.command.validation;

import static com.netapp.snapcenter.cli.command.parser.CustomStringToPrePostScriptMapConverter.POST_SCRIPT_PATH;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToPrePostScriptMapConverter.PRE_SCRIPT_PATH;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToPrePostScriptMapConverter.PRE_SCRIPT_ARGUMENTS;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToPrePostScriptMapConverter.POST_SCRIPT_ARGUMENTS;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToPrePostScriptMapConverter.SCRIPT_TIMEOUT;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.apache.commons.lang3.StringUtils;

import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.SmGetSMTPServerResponse;
import com.netapp.snapcenter.cli.command.parameters.NotificationParameters;
import com.netapp.snapcenter.cli.command.parameters.PrePostScriptParameters;
import com.netapp.snapcenter.cli.command.parameters.VerificationPrePostScriptParameters;
import com.netapp.snapcenter.cli.contracts.EmailNotificationPreference;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.startup.CLILauncher;
import com.netapp.snapcenter.cli.utils.CLIUtils;

/**
 * This class will perform validations on complex user input. Also includes
 * Common validations for user input arguments can be placed here.
 *
 */
public class SmsCommandArgumentsValidationUtil {

    private SmsCommandArgumentsValidationUtil() {

    }

    public static final String PRIMARY = "PRIMARY";
    public static final String SECONDARY = "SECONDARY";
    private static final String PRE_POST_SCRIPT_PATH_FOR_SMO = "/var/opt/snapcenter/spl/scripts/";
    private static ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(SmsCommandArgumentsValidationUtil.class);

    public static void validateSecondaryLocators(
            List<Map<String, String>> secondaryLocators) throws CLIException {
        Set<String> locatorsKeys = new TreeSet<>(String.CASE_INSENSITIVE_ORDER);
        locatorsKeys.add(PRIMARY);
        locatorsKeys.add(SECONDARY);

        Set<String> primaryLocators = new TreeSet<>(
                String.CASE_INSENSITIVE_ORDER);
        for (Map<String, String> sl : secondaryLocators) {
            Set<String> keySet = sl.keySet();
            if (keySet.size() != 2 || !locatorsKeys.containsAll(keySet)) {
                throw new IllegalArgumentException(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_OPTION_SECONDARY_LOCATOR_INVALID_FORMAT));

            }
            Collection<String> values = sl.values();
			for (String v : values) {
				CLIUtils.getIPandExportPath(v);
				if (CLIUtils.getIPandExportPath(v) == null
						|| CLIUtils.getIPandExportPath(v).length != 2
						|| StringUtils
								.isBlank(CLIUtils.getIPandExportPath(v)[0])
						|| StringUtils
								.isBlank(CLIUtils.getIPandExportPath(v)[1])) {
					throw new IllegalArgumentException(
							MessageBundle
									.localizeMessage(CLIMsgKey.CLI_OPTION_SECONDARY_LOCATOR_INVALID_FORMAT));
				}
			}
            String primarySVM = CLIUtils.getIPandExportPath(sl.get(PRIMARY))[0].trim();
            String primaryVol = CLIUtils.getIPandExportPath(sl.get(PRIMARY))[1].trim();
            String primaryLocator = String.format("%s:%s", primarySVM,
                    primaryVol);
            if (primaryLocators.contains(primaryLocator)) {
                throw new IllegalArgumentException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_OPTION_SECONDARY_LOCATOR_MULIPLE_PRIMARY_TO_SECONDARY_LOCATOR_ERROR,
                                        primaryLocator));
            } else {
                primaryLocators.add(primaryLocator);
            }
        }
    }

    public static boolean isValidAppObjectId(String appObjId) {
        String[] appObjIdSplit = StringUtils.split(appObjId, "\\");
        if (appObjIdSplit == null || appObjIdSplit.length != 2
                || StringUtils.isBlank(appObjIdSplit[0])
                || StringUtils.isBlank(appObjIdSplit[1])) {
            return false;
        }
        return true;
    }

    public static void processAndValidatePrePostScriptParameters(
            PrePostScriptParameters prePostScriptParams, PluginCode pluginCode)
            throws CLIException {

        // Specifying default timeout value of 60 seconds
        if (prePostScriptParams.getScriptTimeout() <= 0) {
            prePostScriptParams
                    .setScriptTimeout(PrePostScriptParameters.SCRIPT_DEFAULT_TIMEOUT);
        }

        if (pluginCode.equals(PluginCode.SCO)) {
            if (prePostScriptParams.getPreScriptPath() != null
                    && !prePostScriptParams.getPreScriptPath().startsWith(
                            PRE_POST_SCRIPT_PATH_FOR_SMO)) {
                throw new IllegalArgumentException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_SMO_PRE_SCRIPT_PATH_SHOULD_BE_INSIDE_PATH,
                                        prePostScriptParams.getPreScriptPath()));
            }
            if (prePostScriptParams.getPostScriptPath() != null
                    && !prePostScriptParams.getPostScriptPath().startsWith(
                            PRE_POST_SCRIPT_PATH_FOR_SMO)) {
                throw new IllegalArgumentException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_SMO_POST_SCRIPT_PATH_SHOULD_BE_INSIDE_PATH,
                                        prePostScriptParams.getPostScriptPath()));
            }
        }
    }

    public static void processAndValidateVerificationPrePostScriptParameters(
            VerificationPrePostScriptParameters verificationPrePostScriptParameters,
            PluginCode pluginCode) throws CLIException {

        Map<String, String> verificatoinPrePostScriptMap = verificationPrePostScriptParameters
                .getVerificationScriptOptions();
        // Specifying default timeout value of 60 seconds
        if (StringUtils.isEmpty(verificatoinPrePostScriptMap.get(SCRIPT_TIMEOUT
                .toLowerCase()))
                || (Integer.parseInt(verificatoinPrePostScriptMap
                        .get(SCRIPT_TIMEOUT.toLowerCase())) <= 0)) {
            CLIUtils.writeToLog(
                    MessageBundle
                            .localizeMessage(
                                    CLIMsgKey.CLI_PRE_POST_SCRIPT_PARAMETERS_FORCE_SET_VERIFICATION_SCRIPT_TIMEOUT,
                                    PrePostScriptParameters.SCRIPT_DEFAULT_TIMEOUT),
                    logger, CLIUtils.LogLevel.INFO);
            
            verificatoinPrePostScriptMap
                    .put(SCRIPT_TIMEOUT.toLowerCase(),
                            Integer.toString(PrePostScriptParameters.SCRIPT_DEFAULT_TIMEOUT));
        }

        if (pluginCode.equals(PluginCode.SCO)) {

            if (StringUtils.isNotEmpty(verificatoinPrePostScriptMap
                    .get(PRE_SCRIPT_PATH.toLowerCase()))
                    && !verificatoinPrePostScriptMap.get(
                            PRE_SCRIPT_PATH.toLowerCase()).startsWith(
                            PRE_POST_SCRIPT_PATH_FOR_SMO)) {
                throw new IllegalArgumentException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_SMO_PRE_SCRIPT_PATH_SHOULD_BE_INSIDE_PATH,
                                        verificatoinPrePostScriptMap
                                                .get(PRE_SCRIPT_PATH
                                                        .toLowerCase())));
            }

            if (StringUtils.isEmpty(verificatoinPrePostScriptMap
                    .get(PRE_SCRIPT_PATH.toLowerCase()))
                    && StringUtils.isNotEmpty(verificatoinPrePostScriptMap
                            .get(PRE_SCRIPT_ARGUMENTS.toLowerCase()))) {
                throw new IllegalArgumentException(
                        MessageBundle  
                                .localizeMessage(
                                        CLIMsgKey.CLI_PRE_POST_SCRIPT_PARAMETERS_PROVIDED_VERIFICATION_SCRIPT_ARGUMENTS_BUT_NOT_PATH,
                                        PRE_SCRIPT_PATH));
            }

            if (StringUtils.isNotEmpty(verificatoinPrePostScriptMap
                    .get(POST_SCRIPT_PATH.toLowerCase()))
                    && !verificatoinPrePostScriptMap.get(
                            POST_SCRIPT_PATH.toLowerCase()).startsWith(
                            PRE_POST_SCRIPT_PATH_FOR_SMO)) {
                throw new IllegalArgumentException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_SMO_POST_SCRIPT_PATH_SHOULD_BE_INSIDE_PATH,
                                        verificatoinPrePostScriptMap
                                                .get(POST_SCRIPT_PATH
                                                        .toLowerCase())));
            }

            if (StringUtils.isEmpty(verificatoinPrePostScriptMap
                    .get(POST_SCRIPT_PATH.toLowerCase()))
                    && StringUtils.isNotEmpty(verificatoinPrePostScriptMap
                            .get(POST_SCRIPT_ARGUMENTS.toLowerCase()))) {
                throw new IllegalArgumentException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_PRE_POST_SCRIPT_PARAMETERS_PROVIDED_VERIFICATION_SCRIPT_ARGUMENTS_BUT_NOT_PATH,
                                        POST_SCRIPT_PATH));
            }
        }
    }

	public static void validateAndSetNotificationParameters(
			NotificationParameters notificationParameters,
			SmGetSMTPServerResponse smGetSMTPServerResponse,String command )
			throws CLIException {

		if (notificationParameters.isEnableEmail()
				&& notificationParameters.getEmailNotificationPreference() == null) {
			notificationParameters
					.setEmailNotificationPreference(EmailNotificationPreference.ALWAYS);
		} else if (notificationParameters.getEmailNotificationPreference() != null
				&& !notificationParameters.getEmailNotificationPreference()
						.equals(EmailNotificationPreference.NEVER)) {
			notificationParameters.setEnableEmail(true);
		} else if (!notificationParameters.isEnableEmail()
				|| notificationParameters.getEmailNotificationPreference() == null) {
			notificationParameters
					.setEmailNotificationPreference(EmailNotificationPreference.NEVER);
		}

		if (notificationParameters.isEnableEmail()) {

			if (StringUtils.isBlank(notificationParameters.getEmailFrom())
					&& StringUtils.isBlank(smGetSMTPServerResponse
							.getSMTPEmailFrom())) {
				throw new IllegalArgumentException(
						MessageBundle
								.localizeMessage(CLIMsgKey.CLI_EMAIL_TO_AND_EMAIL_FROM_IS_REQUIRED));
			} else if (StringUtils.isBlank(notificationParameters
					.getEmailFrom())
					&& StringUtils.isNotBlank(smGetSMTPServerResponse
							.getSMTPEmailFrom())) {
				notificationParameters.setEmailFrom(smGetSMTPServerResponse
						.getSMTPEmailFrom());
			}

			if (StringUtils.isBlank(notificationParameters.getEmailTo())
					&& StringUtils.isBlank(smGetSMTPServerResponse
							.getSMTPEmailTo())) {
				throw new IllegalArgumentException(
						MessageBundle
								.localizeMessage(CLIMsgKey.CLI_EMAIL_TO_AND_EMAIL_FROM_IS_REQUIRED));
			} else if (StringUtils.isBlank(notificationParameters.getEmailTo())
					&& StringUtils.isNotBlank(smGetSMTPServerResponse
							.getSMTPEmailTo())) {
				notificationParameters.setEmailTo(smGetSMTPServerResponse
						.getSMTPEmailTo());
			}

			if (StringUtils.isBlank(notificationParameters.getEmailSubject())) {
				String workflowType = CLIUtils.getWorkflowType(command);
				String emailSubject = CLIUtils.getDefaultEmailSubject(
						notificationParameters.getEmailNotificationPreference(),
						workflowType);
				notificationParameters.setEmailSubject(emailSubject);
			}

			validateEmailToandEmailFrom(notificationParameters.getEmailFrom(),
					notificationParameters.getEmailTo());
		}

	}

    public static void validateEmailToandEmailFrom(String emailFrom,
            String emailTo) throws CLIException {

        if (!CLIUtils.validateEmailString(emailFrom)) {
            throw new CLIException(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_EMAIL_FROM_IS_INVALID, emailFrom));
        }

        if (!CLIUtils.validateEmailString(emailTo)) {
            throw new CLIException(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_EMAIL_TO_IS_INVALID, emailTo));
        }
    }
}
