/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/formatter/CustomToStringStyle.java#1 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.snapcenter.cli.formatter;

import java.util.Collection;

import org.apache.commons.lang3.ClassUtils;
import org.apache.commons.lang3.SystemUtils;
import org.apache.commons.lang3.builder.ReflectionToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;

/***
 * 
 * Extending Apache Commons Lang ToStringStyle to Customize the toString style.
 * It is extended to achieve Recursive, Multiline and other style settings.
 * 
 * TODO: Should Look into excluding field names not to be displayed in recursive
 * manner.
 */
public class CustomMultilineRecursiveToStringStyle extends ToStringStyle {

    private static final long serialVersionUID = 1L;

    public CustomMultilineRecursiveToStringStyle() {
        super();
        this.setUseShortClassName(true);
        this.setUseIdentityHashCode(false);
        this.setContentStart("{");
        this.setFieldSeparator(SystemUtils.LINE_SEPARATOR + "\t");
        this.setFieldSeparatorAtStart(true);
        this.setContentEnd("}");
        this.setFieldSeparatorAtEnd(true);
        this.setArrayContentDetail(true);
        this.setArraySeparator(",");
        this.setArrayStart("[");
        this.setArrayEnd("]");
        this.setFieldNameValueSeparator(" : ");
    }

    @Override
    public void appendDetail(final StringBuffer buffer, final String fieldName,
            final Object value) {
        if (!ClassUtils.isPrimitiveWrapper(value.getClass())
                && !String.class.equals(value.getClass())
                && acceptClass(value.getClass())) {
            buffer.append(ReflectionToStringBuilder.toString(value, this));
        } else {
            super.appendDetail(buffer, fieldName, value);
        }
    }

    @Override
    protected void appendDetail(final StringBuffer buffer,
            final String fieldName, final Collection<?> coll) {
        appendClassName(buffer, coll);
        appendIdentityHashCode(buffer, coll);
        appendDetail(buffer, fieldName, coll.toArray());
    }

    /**
     * Returns whether or not to recursively format the given <code>Class</code>
     * . By default, this method always returns {@code true}, but may be
     * overwritten by sub-classes to filter specific classes.
     *
     * @param clazz
     *            The class to test.
     * @return Whether or not to recursively format the given <code>Class</code>
     *         .
     */
    protected boolean acceptClass(final Class<?> clazz) {
        return true;
    }

}
