/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/formatter/JSONFormatter.java#1 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.snapcenter.cli.formatter;

import java.io.IOException;

import org.codehaus.jackson.JsonGenerationException;
import org.codehaus.jackson.map.JsonMappingException;
import org.codehaus.jackson.map.ObjectMapper;

import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.utils.CLIUtils;

public class JSONFormatter implements Formatter {

    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(JSONFormatter.class);

    @Override
    public <T> void writeFormattedObject(T t) throws CLIException {
        String formattedToStringOfObject = getFormattedObject(t);
        CLIUtils.writeToConsole(formattedToStringOfObject);
    }

    @Override
    public <T> String getFormattedObject(T t) throws CLIException {

        String formattedToStringOfObject = null;

        ObjectMapper mapper = new ObjectMapper();
        try {
            formattedToStringOfObject = mapper.writerWithDefaultPrettyPrinter()
                    .writeValueAsString(t);
        } catch (JsonGenerationException e) {
            logger.error("Json Generation Error " + e.getLocalizedMessage());
            throw new CLIException(e.getLocalizedMessage());
        } catch (JsonMappingException e) {
            logger.error("Json Mapping Error " + e.getLocalizedMessage());
            throw new CLIException(e.getLocalizedMessage());
        } catch (IOException e) {
            logger.error("IO Error " + e.getLocalizedMessage());
            throw new CLIException(e.getLocalizedMessage());
        }

        return formattedToStringOfObject;
    }

}
