/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/formatter/TextTable.java#1 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.snapcenter.cli.formatter;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.SystemUtils;
import org.apache.commons.lang3.text.WordUtils;

import com.netapp.snapcenter.cli.utils.CLIUtils;

/**
 * TextTable class is a printing utility which allows to create table and print
 * to the console.
 * 
 * @author kmadan
 */

//TODO: Enhancement required. refer to Python Texttable.
public class TextTable implements TextTableInterface {

    private int columns;
    private List<String[]> table = new LinkedList<>();
    private List<String> tableHeaders = new ArrayList<>();
    private static final String NEW_LINE = SystemUtils.LINE_SEPARATOR;

    // Default Column Delimiter
    private String columnSeperator = "|";

    public TextTable(String... headers) {
        setColumns(headers.length);
        for (int i = 0; i < headers.length; i++) {
            tableHeaders.add(headers[i]);
        }
    }

    public TextTable(int columns) {
        setColumns(columns);
    }

    @Override
    public void addRow(String... values) {
        if (values.length > columns) {
            throw new IllegalArgumentException("Expected " + getColumns()
                    + " Cloumns values but got " + values.length);
        }
        table.add(values);
    }

    @Override
    public void addRow(List<String> values) {
        addRow((String[]) values.toArray(new String[values.size()]));
    }

    @Override
    public void addTableHeaders(List<String> tableHeaders) {
        addTableHeaders((String[]) tableHeaders.toArray(new String[tableHeaders
                .size()]));
    }

    @Override
    public void addTableHeaders(String... tableHeaders) {
        if (tableHeaders.length != getColumns()) {
            throw new IllegalArgumentException("Expected " + getColumns()
                    + " header Cloumns but got " + tableHeaders.length);
        }
        for (int i = 0; i < tableHeaders.length; i++) {
            this.tableHeaders.add(tableHeaders[i]);
        }
    }

    @Override
    public List<String> getTableHeaders() {
        return tableHeaders;
    }

    @Override
    public int getColumns() {
        return columns;
    }

    @Override
    public void setColumns(int columns) {
        this.columns = columns;
    }

    // Prints table with all columns
    @Override
    public void displayFormattedTable() {
        displayFormattedTable(getColumns());
    }

    // Prints table with first n columns
    @Override
    public void displayFormattedTable(int n) {
        if (n > getColumns()) {
            throw new IllegalArgumentException(
                    "Numbers of Columns("
                            + n
                            + ") to print is more than the exsisting number of Columns("
                            + getColumns() + ")");
        }

        Map<Integer, Boolean> colsMap = createAndInitializeColsMap(false);
        for (int i = 0; i < n; i++) {
            colsMap.put(i, true);
        }
        String tableAsString = renderTable(colsMap);
        CLIUtils.writeToConsole(tableAsString);
    }

    // Print table with selective headers columns

    @Override
    public void displayFormattedTable(List<String> tableHeaders) {
        displayFormattedTable((String[]) tableHeaders
                .toArray(new String[tableHeaders.size()]));
    }

    // Print table with selective headers columns

    @Override
    public void displayFormattedTable(String... tableHeaders) {
        if (tableHeaders.length > getColumns()) {
            throw new IllegalArgumentException(
                    "Numbers of Header Columns("
                            + tableHeaders.length
                            + ") to print is more than the exsisting number of Columns("
                            + getColumns() + ")");
        }
        Map<Integer, Boolean> colsMap = createAndInitializeColsMap(false);
        String[] actualtableHeaders = (String[]) getTableHeaders().toArray(
                new String[getTableHeaders().size()]);

        /*
         * Search for the existence of header and mark its respective column for
         * printing
         */
        for (int i = 0; i < tableHeaders.length; i++) {
            boolean found = false;
            int loc = -1;
            for (int j = 0; j < actualtableHeaders.length; j++) {
                if (tableHeaders[i].equalsIgnoreCase(actualtableHeaders[j])) {
                    found = true;
                    loc = j;
                    break;
                }
            }
            if (found && loc != -1) {
                colsMap.put(loc, true);
            } else {
                throw new IllegalArgumentException("No such Header called '"
                        + tableHeaders[i] + "' exsist in table");
            }
        }
        String tableAsString = renderTable(colsMap);
        CLIUtils.writeToConsole(tableAsString);

    }

    /**
     * Creates a map specifying column inclusion or exclusion for printing with
     * a specified a default value.
     * 
     * @param value
     * @return
     */
    private Map<Integer, Boolean> createAndInitializeColsMap(boolean value) {
        Map<Integer, Boolean> colsMap = new HashMap<>();
        for (int i = 0; i < getColumns(); i++) {
            colsMap.put(i, value);
        }
        return colsMap;
    }

    /**
     * Return a String which is formatted as a table. Every line is wrapped to
     * the terminal console size.
     * 
     * @param colsMap
     * @return
     */
    private String renderTable(Map<Integer, Boolean> colsMap) {

        /* Col width for padding so that each entry is correctly spaced */
        int[] colWidth = getMaxWidthOfColumns(colsMap);
        
        // Add extra 4 to each column width for better spacing
        for (int i = 0; i < columns; i++) {
            if (colsMap.get(i)) {
                colWidth[i] += 4;
            }
        }
        
        // Initialize to include end colSeperator
        int tableWidth = 1;
        for (int i = 0; i < columns; i++) {
            if (colsMap.get(i)) {
                tableWidth += colWidth[i];
                // incrementing to include size for each column separator
                tableWidth++;
            }
        }

        // Get the terminal console width
        int consoleWidth = CLIUtils.getConsoleWidth();

        // Wrap Header line based on console size
        String headerSeparatorLine = WordUtils.wrap(
                generateLineSeparatorString(tableWidth, '='), consoleWidth, NEW_LINE, true);

        
        /***
         * 
         * 
         * Table format with default column separator:
         * 
         * ======header Separator Line============
         * |header1 | header 2 | ...| header N   |
         * ======header Separator Line============
         * |row1Val1 | row1Val2 | ...| row1valN  |
         * |row2Val1 | row2Val2 | ...| row2valN  |
         * |.        | .        | ...| .         |
         * |.        | .        | ...| .         |
         * |rowMVal1 | rowMVal2 | ...| rowMvalN  |
         * ======header Separator Line============
         * 
         */
        
        
        StringBuilder tableBuilder = new StringBuilder();

        tableBuilder.append(headerSeparatorLine);
        tableBuilder.append(NEW_LINE);

        // Wrap Headers entry based on console size
        String headersEntry = WordUtils.wrap(
                getRowString((String[]) tableHeaders
                        .toArray(new String[tableHeaders.size()]), colsMap,
                        colWidth), consoleWidth, NEW_LINE, true);
        tableBuilder.append(headersEntry);
        tableBuilder.append(NEW_LINE);

        tableBuilder.append(headerSeparatorLine);
        tableBuilder.append(NEW_LINE);

        for (String[] row : table) {
            // Wrap row entry based on console size
            String rowEntry = WordUtils.wrap(
                    getRowString(row, colsMap, colWidth), consoleWidth, NEW_LINE,
                    true);
            tableBuilder.append(rowEntry);
            tableBuilder.append(NEW_LINE);
        }

        tableBuilder.append(headerSeparatorLine);
        tableBuilder.append(NEW_LINE);

        return tableBuilder.toString();

    }

    private String getRowString(String[] row, Map<Integer, Boolean> colsMap,
            int[] colWidth) {
        StringBuilder rowBuilder = new StringBuilder();
        rowBuilder.append(getColumnSeperator());
        for (int i = 0; i < getColumns(); i++) {
            if (colsMap.get(i)) {
                // Pad each string to fit to column size uniformly
                rowBuilder.append(String.format("%" + colWidth[i] + "s"
                        + getColumnSeperator(),
                        StringUtils.rightPad(row[i], colWidth[i] - 2)));
            }
        }
        return rowBuilder.toString();
    }

    /**
     * Generates a line by repeating the specified character for specified
     * length.
     * 
     * @param length
     * @param character
     * @return
     */
    private String generateLineSeparatorString(int length, char character) {
        StringBuilder lineBuilder = new StringBuilder();
        for (int i = 0; i < length; i++) {
            lineBuilder.append(character);
        }
        return lineBuilder.toString();
    }

    /***
     * 
     * Calculates the maximum length of string that exists in each column.
     * Columns can be included or excluded from calculating using colMap
     * 
     * @param colsMap
     *            - A Map to specify the column inclusion and exclusion for
     *            calculating max width.
     * @return int[] - an array containing the maximum length of String in each
     *         column
     */
    private int[] getMaxWidthOfColumns(Map<Integer, Boolean> colsMap) {

        int[] colWidth = new int[getColumns()];

        // Determine header columns max width
        for (int i = 0; i < getColumns(); i++) {
            if (colsMap.get(i)) {
                colWidth[i] = Math.max(colWidth[i], tableHeaders.get(i)
                        .length());
            }
        }

        // Determine columns max width for rest of the table entries
        for (String[] row : table) {
            for (int i = 0; i < getColumns(); i++) {
                if (colsMap.get(i)) {
                    colWidth[i] = Math.max(colWidth[i], row[i].length());
                }
            }
        }
        
        return colWidth;
    }

    @Override
    public String getColumnSeperator() {
        return columnSeperator;
    }

    @Override
    public void setColumnSeperator(String columnSeperator) {
        this.columnSeperator = columnSeperator;
    }

    @Override
    public String getFormattedTable() {
        Map<Integer, Boolean> colsMap = createAndInitializeColsMap(true);
        return renderTable(colsMap);
    }

}
