/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/utils/ResourceProtectionGroupUtil.java#4 $
 *
 * Copyright (c) 2016 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.snapcenter.cli.helper;

import org.apache.commons.lang3.StringUtils;

import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.plugin.contracts.ArrayOfSmObjectOperationResult;
import com.netapp.smcore.plugin.contracts.ArrayOfSmResourceResult;
import com.netapp.smcore.plugin.contracts.SmDiscoverResourcesResponse;
import com.netapp.smcore.plugin.contracts.SmGetProtectionGroupsResponse;
import com.netapp.smcore.plugin.contracts.SmObject;
import com.netapp.smcore.plugin.contracts.SmObjectOperationResult;
import com.netapp.smcore.plugin.contracts.SmResourceResult;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;

public class ResourceProtectionGroupHelper {

    /**
     * 
     * @param smDiscoverResourcesResponse
     * @param resourceName
     * @return SmObjectGuid
     * @throws CLIException
     */
    public static String getSmObjectGuidFromResourceName(
            SmDiscoverResourcesResponse smDiscoverResourcesResponse,
            String resourceName) throws CLIException {

        String smObjectGuid = new String();
        ArrayOfSmResourceResult arrayOfSmResourceResult = smDiscoverResourcesResponse
                .getResources();
        for (SmResourceResult smResourceResult : arrayOfSmResourceResult
                .getSmResourceResult()) {

            ArrayOfSmObjectOperationResult arrayOfSmObjectOperationResult = smResourceResult
                    .getOperationResults();
            if (arrayOfSmObjectOperationResult == null
                    || CLIUtils
                            .isNullOrEmptyCollection(arrayOfSmObjectOperationResult
                                    .getSmObjectOperationResult())) {
                continue;
            }
            for (SmObjectOperationResult smObjectOperationResult : arrayOfSmObjectOperationResult
                    .getSmObjectOperationResult()) {
                SmObject smOracleDb = smObjectOperationResult.getTarget();
                if (smOracleDb != null) {
                    String dbName = smOracleDb.getName();
                    if (resourceName.equals(dbName)) {
                        smObjectGuid = smOracleDb.getId();
                    }
                }

            }

        }

        // Through error if resource is not found
        if (StringUtils.isEmpty(smObjectGuid)) {
            throw new CLIException(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_SMPROTECTRESOURCE_RESOURCE_NOT_FOUND,
                    resourceName));
        }

        return smObjectGuid;
    }

    /**
     * @param smGetProtectionGroupsResponse
     * @param smObjectGuid
     * @return Internal resource group name
     */
    public static String getInternalResourceGroupNameFromProtectionGroup(
            SmGetProtectionGroupsResponse smGetProtectionGroupsResponse,
            String smObjectGuid) throws CLIException {

        if (smGetProtectionGroupsResponse.getProtectionGroups()
                .getSmProtectionGroup().size() == 0) {

            String[] resSplit = StringUtils.split(smObjectGuid, "\\");
            String hostOrClusterName = StringUtils.trim(resSplit[0]);
            String resourceName = StringUtils.trim(resSplit[1]);

            throw new CLIException(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_SMPROTECTRESOURCE_RESOURCE_NOT_PROTECTED,
                    resourceName));
        }

        return smGetProtectionGroupsResponse.getProtectionGroups()
                .getSmProtectionGroup().get(0).getName();

    }

    /**
     * Verifies whether given SmObject Id is part of Protection group.
     * 
     * @param smGetProtectionGroupsResponse
     * @return true, if SmObject Id is part of Protection group.
     */
    public static boolean isResourceAlreadyProtected(
            SmGetProtectionGroupsResponse smGetProtectionGroupsResponse) {
        boolean isProtected = false;
        // If size is not zero then it is already part of protection group
        if (smGetProtectionGroupsResponse.getProtectionGroups()
                .getSmProtectionGroup().size() != 0) {
            isProtected = true;
        }
        return isProtected;

    }

}
