/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/startup/ManPageInvocator.java#12 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.snapcenter.cli.startup;

import java.io.File;
import java.io.IOException;
import java.lang.ProcessBuilder.Redirect;
import java.util.Arrays;
import java.util.List;

import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.spl.common.utils.ApplicationHome;

public class ManPageInvocator implements Runnable {

    public static final File CLI_MANPAGE_PATH = new File(
            ApplicationHome.getPath(), "man");
    public static final String MANPAGE_FILE_NAME_FORMAT = "%s.man";

    public ManPageInvocator(String commandName) {
        this.commandName = commandName;
    }

    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(ManPageInvocator.class);

    private String commandName;

    @Override
    public void run() {
        final Thread manThread = Thread.currentThread();
        Runtime.getRuntime().addShutdownHook(new Thread() {
            @Override
            public void run() {
                try {
                    manThread.join();
                } catch (InterruptedException e) {
                    logger.debug(e.getLocalizedMessage(), e);
                }
            }
        });
        File commandManFile = new File(CLI_MANPAGE_PATH, String.format(
                MANPAGE_FILE_NAME_FORMAT, commandName));
        List<String> manCommand = Arrays.asList("sh", "-c", "man "
                + commandManFile.getPath());
        logger.debug("Executing man command: " + manCommand.toString());

        Process process = null;
        ProcessBuilder processBuilder = new ProcessBuilder(manCommand);

        processBuilder.redirectOutput(Redirect.INHERIT);
        processBuilder.redirectError(Redirect.INHERIT);

        try {
            process = processBuilder.start();
            process.waitFor();
        } catch (InterruptedException | IOException e) {
            logger.debug(e.getLocalizedMessage(), e);
        }
    }

}
