/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/utils/CLIUtils.java#1 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.utils;

import java.io.BufferedReader;
import java.io.Console;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.EnumSet;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TimeZone;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.Duration;
import javax.xml.datatype.XMLGregorianCalendar;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.SystemUtils;
import org.apache.commons.lang3.text.WordUtils;

import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.SmCatalogingStatus;
import com.netapp.smcore.plugin.contracts.SmVerificatinResultEnum;
import com.netapp.snapcenter.cli.command.HelpCommand;
import com.netapp.snapcenter.cli.command.parameters.PrePostScriptParameters;
import com.netapp.snapcenter.cli.command.parser.CommandParserInterface;
import com.netapp.snapcenter.cli.contracts.EmailNotificationPreference;
import com.netapp.snapcenter.cli.contracts.GenericConstants;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.contracts.ResourceType;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.startup.CLILauncher;

public class CLIUtils {

    // Window/Terminal Console width
    private static Integer consoleWidth = -1;

    public static final String SEMICOLON = ";";

    public static final String NEW_LINE = SystemUtils.LINE_SEPARATOR;
    public static final String CARRIAGE_RETURN = "\r";

    public static final String DATE_FORMAT_WITH_MILLISECONDS = "yyyy-MM-dd_HH.mm.ss.SSS";
    public static final String DATE_FORMAT = "yyyy-MM-dd HH:mm:ss";
    public static final String UTC_TIME_ZONE = "UTC";
    public static final String DURATION_FORMAT = "HH:mm:ss";
    private static final Pattern TNS_NAME_PATTERN = Pattern.compile("[A-Za-z][A-Za-z0-9]{0,}");
    public static final Pattern VALID_EMAIL_ADDRESS_REGEX = Pattern.compile(
            "^[_A-Za-z0-9-\\+]+(\\.[_A-Za-z0-9-]+)*@"
                    + "[A-Za-z0-9-]+(\\.[A-Za-z0-9]+)*(\\.[A-Za-z]{2,})$",
            Pattern.CASE_INSENSITIVE);
    private static final String IPV4_REGEX_PATTERN = "((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])";
    private static final String IPV6_REGEX_PATTERN = "::(ffff(:0{1,4}){0,1}:){0,1}((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])|([0-9a-fA-F]{1,4}:){1,4}:((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])|([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA-F]{1,4}){1,4}|([0-9a-fA-F]{1,4}:){1,6}:[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,5}(:[0-9a-fA-F]{1,4}){1,2}|([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA-F]{1,4}){1,3}|([0-9a-fA-F]{1,4}:){1,3}(:[0-9a-fA-F]{1,4}){1,4}|([0-9a-fA-F]{1,4}:){1,2}(:[0-9a-fA-F]{1,4}){1,5}|[0-9a-fA-F]{1,4}:((:[0-9a-fA-F]{1,4}){1,6})|:((:[0-9a-fA-F]{1,4}){1,7}|:)|fe08:(:[0-9a-fA-F]{1,4}){2,2}%[0-9a-zA-Z]{1,}|([0-9a-fA-F]{1,4}:){7,7}[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,7}:";
    private static final String IP_AND_VOLUME_SEPERATOR = "(:/|:)";
    private static int CONSOLE_OUTPUT_WIDTH = 80;

    private static BufferedReader bufferedReader = new BufferedReader(
            new InputStreamReader(System.in));

    private static ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(CLILauncher.class);

    private CLIUtils() {

    }

    public static enum LogLevel {
        INFO, ERROR, DEBUG, WARN, FATAL, TRACE;
    }

    // TODO: try getting console by reflection to make sure that exists
    public static void writeToConsole(String displayMessage) {
        System.out.println(displayMessage);
    }

    public static void writeToLog(String logMessage,
            ExtendedGenericLogger logger, LogLevel logLevel) {
        switch (logLevel) {
            case INFO:
                logger.info(logMessage);
                break;
            case ERROR:
                logger.error(logMessage);
                break;
            case DEBUG:
                logger.debug(logMessage);
                break;
            case WARN:
                logger.warn(logMessage);
                break;
            case FATAL:
                logger.fatal(logMessage);
                break;
            case TRACE:
                logger.trace(logMessage);
                break;
        }
    }

    public static void writeToLog(Throwable throwable,
            ExtendedGenericLogger logger, LogLevel logLevel) {
        String localizedMessage = throwable.getLocalizedMessage();
        switch (logLevel) {
            case INFO:
                logger.info(localizedMessage, throwable);
                break;
            case ERROR:
                logger.error(localizedMessage, throwable);
                break;
            case DEBUG:
                logger.debug(localizedMessage, throwable);
                break;
            case WARN:
                logger.warn(localizedMessage, throwable);
                break;
            case FATAL:
                logger.fatal(localizedMessage, throwable);
                break;
            case TRACE:
                logger.trace(localizedMessage, throwable);
                break;
        }
    }

    public static String getCanonicalFilePath(File file) {
        try {
            return file.getCanonicalPath();
        } catch (IOException e) {
            CLIUtils.writeToLog(e, logger, LogLevel.DEBUG);
        }
        return null;
    }

    public static void writeToLogAndConsole(String logAndDisplayMessage,
            ExtendedGenericLogger logger, LogLevel logLevel) {

        writeToLog(logAndDisplayMessage, logger, logLevel);

        String logAndDisplayMessageWithLogLevelPrefix = String.format("%s: %s",
                logLevel.name(), logAndDisplayMessage);
        writeToConsole(logAndDisplayMessageWithLogLevelPrefix);
    }

    public static String readLineFromStdin(String format, Object... args) {
        System.out.printf(format, args);
        String line;
        try {
            if ((line = bufferedReader.readLine()) != null) {
                return line;
            }
        } catch (IOException ioException) {
            writeToLog(ioException, logger, LogLevel.ERROR);
        }
        return null;
    }

    public static void closeResources() {
        try {
            if (bufferedReader != null) {
                bufferedReader.close();
            }
        } catch (IOException ioException) {
            writeToLog(ioException, logger, LogLevel.ERROR);
        }
    }

    public static String readLineFromConsole(String format, Object... args) {
        Console console = System.console();
        if (console == null) {
            return (readLineFromStdin(format, args));
        } else {
            return console.readLine(format, args);
        }
    }

    public static String readPasswordFromConsole(String format, Object... args) {
        Console console = System.console();
        if (console == null) {
            return (readLineFromStdin(format, args));
        } else {
            return new String(console.readPassword(format, args));
        }
    }

    public static void promptUserToContinue(String userMessage)
            throws CLIException {
        writeToConsole(String.format("%s", userMessage));
        String promptUser = String.format("%s: ", MessageBundle
                .localizeMessage(CLIMsgKey.CLI_PROMPT_USER_YER_OR_NO));
        String userInput = readLineFromConsole(promptUser);
        while (true) {
            String userInputTrimmed = userInput.trim();
            if (userInputTrimmed.equalsIgnoreCase("Y")
                    || userInputTrimmed.equalsIgnoreCase("Yes")) {
                writeToLog(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_USER_CONTINUE_OPERATION),
                        logger, LogLevel.DEBUG);
                return;
            } else if (userInputTrimmed.equalsIgnoreCase("N")
                    || userInputTrimmed.equalsIgnoreCase("No")
                    || userInputTrimmed.isEmpty()) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_USER_ABORT_OPERATION));
            } else {
                writeToConsole("Invalid value '" + userInput + "'");
                userInput = readLineFromConsole(promptUser);
                continue;
            }
        }
    }

    public static String localizeDateAndGetFormattedString(
            XMLGregorianCalendar time) {
        String localizedDateString = StringUtils.EMPTY;
        try {
            SimpleDateFormat df = new SimpleDateFormat(DATE_FORMAT);
            localizedDateString = df.format(time.toGregorianCalendar().getTime());
        } catch (Exception e) {
            CLIUtils.writeToLog(e.getLocalizedMessage(), logger, LogLevel.DEBUG);
        }
        return localizedDateString;
    }

    public static String getDuration(XMLGregorianCalendar startTime,
            XMLGregorianCalendar endTime) {
        String durationString = StringUtils.EMPTY;
        try {
            Duration duration = DatatypeFactory.newInstance()
                    .newDuration(
                            endTime.toGregorianCalendar().getTimeInMillis()
                                    - startTime.toGregorianCalendar()
                                            .getTimeInMillis());
            TimeZone tz = TimeZone.getTimeZone(UTC_TIME_ZONE);
            SimpleDateFormat df = new SimpleDateFormat(DURATION_FORMAT);
            df.setTimeZone(tz);
            durationString = df.format(new Date(duration.getSeconds() * 1000L));
        } catch (DatatypeConfigurationException e) {
            CLIUtils.writeToLog(e.getLocalizedMessage(), logger, LogLevel.DEBUG);
        }
        return durationString;
    }

    public static XMLGregorianCalendar getXMLGregorianCalendarFromDate(Date date) {
        GregorianCalendar gregorianCalendar = new GregorianCalendar();
        gregorianCalendar.setTime(date);
        XMLGregorianCalendar xmlGregorianCalendar = null;
        try {
            xmlGregorianCalendar = DatatypeFactory.newInstance()
                    .newXMLGregorianCalendar(gregorianCalendar);
        } catch (DatatypeConfigurationException e) {
            CLIUtils.writeToLog(e.getLocalizedMessage(), logger, LogLevel.DEBUG);
        }
        return xmlGregorianCalendar;
    }

    public static <T extends Collection<?>> boolean isNullOrEmptyCollection(T t) {
        return (t == null || t.isEmpty());
    }

    public static <T extends Collection<?>> boolean isNotNullAndNotEmptyCollection(
            T t) {
        return (t != null && !t.isEmpty());
    }

    /**
     * Test if the supplied map is null or empty.
     *
     * @param map
     *            Map to test
     * @return True if the map is null or empty
     */
    public static <T extends Map<?, ?>> boolean isEmpty(T t) {
        return t == null || t.isEmpty();
    }

    /**
     * Test if the supplied list is null or empty.
     *
     * @param list
     *            List to test
     * @return True if the list is null or empty
     */
    public static <T extends List<?>> boolean isEmpty(T t) {
        return t == null || t.isEmpty();
    }

    public static String getStringValue(String s, String nullReturnValue) {
        return ((s == null) ? nullReturnValue : s);
    }

    public static String getStringValue(String s) {
        return getStringValue(s, StringUtils.EMPTY);
    }

    public static <E extends Enum<E>> String getEnumName(E e,
            String nullReturnValue) {
        return ((e == null) ? nullReturnValue : e.name());
    }

    public static <E extends Enum<E>> String getEnumName(E e) {
        return getEnumName(e, "");
    }

    public static int getConsoleWidth() {
        /*
         * Since Cli is invoked once and not be loaded it is fine having static
         * variable comparision once and fetching the console width. But in case
         * of smart or always loaded cli static should be avoided.
         */
        
        if (consoleWidth != -1) {
            return consoleWidth;
        }

        // Initialize to default
        int windowTerminalConsoleWidth = CONSOLE_OUTPUT_WIDTH;

        String os = System.getProperty("os.name").toLowerCase();

        if (os.contains("win")) {
            // Windows
            String[] commands = new String[] { "cmd.exe", "/c", "mode", "con" };
            try {
                List<String> processOutput = runCommandsAndGetOutput(commands);
                if (!CLIUtils.isNullOrEmptyCollection(processOutput)) {
                    for (String outputLine : processOutput) {
                        if (outputLine.contains("Columns:")) {
                            String colwidth = StringUtils.remove(outputLine,
                                    "Columns:");
                            windowTerminalConsoleWidth = Integer
                                    .parseInt(colwidth.trim());
                            break;
                        }
                    }
                }
            } catch (Exception e) {
                logger.debug(e.getLocalizedMessage());
            }
        } else {

            // Should work for Linux, AIX and solaris
            String[] commands = new String[] { "sh", "-c",
                    "tput cols 2> /dev/tty" };
            try {
                List<String> processOutput = runCommandsAndGetOutput(commands);
                if (!CLIUtils.isNullOrEmptyCollection(processOutput)) {
                    windowTerminalConsoleWidth = Integer.parseInt(processOutput
                            .get(0).trim());
                }
            } catch (Exception e) {
                logger.debug(e.getLocalizedMessage());
            }
        }

        consoleWidth = windowTerminalConsoleWidth;
        return consoleWidth;
    }
    
    public static void setConsoleWidth(Integer width) {
        consoleWidth = width;
    }

    private static List<String> runCommandsAndGetOutput(String[] commands)
            throws Exception {
        List<String> processOutput = new ArrayList<>();

        Process process = null;
        ProcessBuilder processBuilder = new ProcessBuilder(commands);
        processBuilder.redirectErrorStream(true);
        process = processBuilder.start();

        try (InputStream inputStream = process.getInputStream();
                InputStreamReader inputStreamReader = new InputStreamReader(
                        inputStream, "UTF-8");
                BufferedReader bufferedReader = new BufferedReader(
                        inputStreamReader)) {
            String line;
            while ((line = bufferedReader.readLine()) != null) {
                processOutput.add(line);
            }
        }
        return processOutput;
    }

    public static void printCliUsage(CommandParserInterface commandParser)
            throws CLIException {
        StringBuilder cliUsageBuilder = new StringBuilder();
        cliUsageBuilder.append(MessageBundle.localizeMessage(
                CLIMsgKey.CLI_USAGE, CLILauncher.CLI_EXECUTABLE_NAME));
        cliUsageBuilder.append(NEW_LINE);
        cliUsageBuilder.append(NEW_LINE);

        cliUsageBuilder.append(MessageBundle.localizeMessage(
                CLIMsgKey.CLI_MOST_COMMON_USED_COMMAND_TEXT,
                CLILauncher.CLI_EXECUTABLE_NAME));
        cliUsageBuilder.append(NEW_LINE);

        String commandsAndItsDescription = getRegisteredCommandsWithDescription(commandParser);
        cliUsageBuilder.append(commandsAndItsDescription);
        cliUsageBuilder.append(NEW_LINE);

        cliUsageBuilder.append(MessageBundle.localizeMessage(
                CLIMsgKey.CLI_COMMAND_SPECIFIC_HELP,
                CLILauncher.CLI_EXECUTABLE_NAME));

        CLIUtils.writeToConsole(cliUsageBuilder.toString());
    }

    public static String getVerificationStatus(
            SmVerificatinResultEnum verifyResultSummary) {
        if (verifyResultSummary == null) {
            return StringUtils.EMPTY;
        }
        switch (verifyResultSummary) {
            case NOT_APPLICABLE:
                return "Not Applicable";
            case NOT_VERIFIED:
                return "Unverified";
            case VERIFIED_FAILED:
                return "Failed";
            case VERIFIED_OK:
                return "Verified";
            case VERIFIED_PARTIAL:
                return "Partial";
            case VERIFY_RUNNING:
                return "Running";
            default:
                return StringUtils.EMPTY;
        }

    }

    public static String getCatalogingStatus(
            SmCatalogingStatus catalogingStatus) {
        if (catalogingStatus == null) {
            return StringUtils.EMPTY;
        }
        switch (catalogingStatus) {
            case NOT_CATALOGED:
                return "Not Cataloged";
            case FAILED:
                return "Failed";
            case CATALOGED:
                return "Cataloged";
            case PARTIAL:
                return "Partial";
            case RUNNING:
                return "Running";
            case NOT_APPLICABLE:
                return "Not Applicable";
            default:
                return StringUtils.EMPTY;
        }

    }

    private static String getRegisteredCommandsWithDescription(
            CommandParserInterface commandParser) throws CLIException {

        // Wrap lines based on Console Width for pretty printing
        int displayConsoleWidth = CLIUtils.getConsoleWidth();
        Map<String, String> registeredCommandsWithDescription = new HashMap<>();
        List<String> registeredCommands = commandParser
                .getAllRegisteredCommands();
        registeredCommands.remove(new HelpCommand().getCommandName());
        Collections.sort(registeredCommands);

        int commandCollMax = 0;
        int commandDescriptionCollMax = 0;

        /**
         * Insert command description and also calculate the max length of
         * String in columns
         */
        for (String command : registeredCommands) {
            String commandDescription = commandParser
                    .getCommandDescription(command);
            commandCollMax = Math.max(commandCollMax, command.length());
            commandDescriptionCollMax = Math.max(commandDescriptionCollMax,
                    commandDescription.length());
            registeredCommandsWithDescription.put(command, commandDescription);
        }

        // For better spacing
        commandCollMax += 3;

        StringBuilder allCommandsWithDescBuilder = new StringBuilder();

        for (String command : registeredCommands) {
            StringBuilder cmdDescBuilder = new StringBuilder();

            cmdDescBuilder.append(String.format("%" + commandCollMax + "s",
                    StringUtils.rightPad(command, commandCollMax)));
            cmdDescBuilder.append(String.format("%s",
                    registeredCommandsWithDescription.get(command)));

            // For indentation
            allCommandsWithDescBuilder.append(StringUtils.rightPad("", 3));
            /*
             * Wrap based on console size excluding indent. It is aligned to
             * respective column during wrapping by padding whitespace.
             */
            allCommandsWithDescBuilder.append(WordUtils.wrap(
                    cmdDescBuilder.toString(), displayConsoleWidth - 3,
                    NEW_LINE + StringUtils.rightPad("", commandCollMax + 3),
                    true));
            allCommandsWithDescBuilder.append(NEW_LINE);
        }
        return allCommandsWithDescBuilder.toString();
    }

    public static <T extends Enum<T>> List<T> convertStringListToEnumList(
            Class<T> enumClass, List<String> stringList) throws CLIException {
        Set<T> enumSet = new LinkedHashSet<>();

        for (String string : stringList) {
            T t = getEnumFromValue(enumClass, string);
            enumSet.add(t);
        }

        List<T> enumList = new ArrayList<>(enumSet);
        return enumList;
    }

    /**
     * Converts string to corresponding enum value.
     * 
     * First we try to convert the string as it is to enum. If failed in first
     * attempt then will try by converting string value to upper case and
     * convert to enum. Even if that didn't work then will try by converting
     * string value to capitalized case and convert to enum. Even if that fails
     * then will throw error.
     * 
     * For example: sunday is string value. first we try to convert sunday if it
     * wont work then try SUNDAY even if this don't then will try Sunday. if
     * nothing works then error out.
     * 
     * @param enumClass
     * @param value
     * @return
     * @throws CLIException
     */
    @SuppressWarnings("unchecked")
    public static <T extends Enum<T>> T getEnumFromValue(Class<T> enumClass,
            String value) throws CLIException {
        T t = null;
        try {
            t = (T) Enum.valueOf((Class<? extends Enum>) enumClass, value);
        } catch (IllegalArgumentException e) {
            try {
                t = (T) Enum.valueOf((Class<? extends Enum>) enumClass,
                        value.toUpperCase());
            } catch (IllegalArgumentException e1) {
                try {
                    t = (T) Enum.valueOf((Class<? extends Enum>) enumClass,
                            StringUtils.capitalize(value));
                } catch (IllegalArgumentException e2) {
                    String enumConversionExceptionMessage = MessageBundle
                            .localizeMessage(
                                    CLIMsgKey.CLI_ENUM_CONVERSION_INVALID_VALUE,
                                    value,
                                    enumClass.getSimpleName(),
                                    EnumSet.allOf((Class<? extends Enum>) enumClass));
                    throw new CLIException(enumConversionExceptionMessage);
                }
            }
        } catch (Exception e) {
            String enumConversionExceptionMessage = MessageBundle
                    .localizeMessage(
                            CLIMsgKey.CLI_ENUM_CONVERSION_INVALID_VALUE, value,
                            enumClass.getSimpleName(),
                            EnumSet.allOf((Class<? extends Enum>) enumClass));
            throw new CLIException(enumConversionExceptionMessage);
        }
        return t;
    }

    public static boolean isValidPortNumber(int portNumber) {
        return ((portNumber < 0 || portNumber > 65536) ? false : true);
    }

    public static boolean isValidRmanTnsName(String tnsName) {
         return TNS_NAME_PATTERN.matcher(tnsName).matches();
    }

    public static PluginCode getPluginTypeFromResourceType(ResourceType resourceType) throws CLIException {

        PluginCode pluginCode;

        switch (resourceType) {
            case ORACLE_DATABASE:
                pluginCode = PluginCode.SCO;
                break;
            case FILESYSTEM:
                pluginCode = PluginCode.SCU;
                break;
            default:
                throw new CLIException(MessageBundle.localizeMessage(
                        CLIMsgKey.CLI_COMMAND_NOT_SUPPORTED_FOR_PLUGIN,
                        resourceType.name()));
        }

        return pluginCode;
    }

    /*
     * It removes the duplicate entries in the list
     */
    public static <T> void removeDuplicateEntriesFromList(List<T> list) {
        Set<T> linkedHashSet = new LinkedHashSet<>();
        for (T val : list) {
            linkedHashSet.add(val);
        }

        if (list.size() != linkedHashSet.size()) {
            CLIUtils.writeToLog(MessageBundle
                            .localizeMessage(
                                    CLIMsgKey.CLI_SMPOLICY_REMOVING_DUPLICATE_ENTRIES),
                    logger, CLIUtils.LogLevel.INFO);
        }

        list.clear();
        list.addAll(linkedHashSet);
    }

    public static boolean validateEmailString(String emailStr) {
        String[] emails = emailStr.split(",");
        for( String email : emails )
        {
            Matcher matcher = VALID_EMAIL_ADDRESS_REGEX.matcher(email);
            if(!matcher.find())
               return false;
        }
        return true;
    }

	public static String getWorkflowType(String command) {
		String workflowType = "";
		switch (command) {
			case "Add-SmProtectResource":
			case "Set-SmProtectResource":
			case "Add-SmResourceGroup":
			case "Set-SmResourceGroup":
				workflowType = GenericConstants.BACKUP.value();
				break;
			case "Restore-SmBackup":
				workflowType = GenericConstants.RESTORE.value();
				break;
			case "New-SmClone":
				workflowType = GenericConstants.CLONE.value();
				break;
			case "Invoke-SmResourceSplit":
				workflowType = GenericConstants.CLONESPLIT.value();
				break;
		}
		return workflowType;
	}

	public static String getDefaultEmailSubject(
			EmailNotificationPreference emailNotificationPreference,
			String workflowType) {
		String subject = "";
		switch (emailNotificationPreference) {
			case ALWAYS:
				subject = MessageBundle.localizeMessage(
						CLIMsgKey.CLI_NOTIFICATION_PARAMETERS_ALWAYS_EMAIL_SUBJECT,
						workflowType);
				break;
			case ON_ERROR:
				subject = MessageBundle.localizeMessage(
						CLIMsgKey.CLI_NOTIFICATION_PARAMETERS_ERROR_EMAIL_SUBJECT,
						workflowType);
				break;
			case ON_ERROR_OR_WARNING:
				subject = MessageBundle
						.localizeMessage(
								CLIMsgKey.CLI_NOTIFICATION_PARAMETERS_ERROR_OR_WARNING_EMAIL_SUBJECT,
								workflowType);
				break;
			case NEVER:
				break;
		}
		return subject;
	}

	public static String[] getIPandExportPath(String mountPath)
			throws IllegalArgumentException {
		String[] ipAndExportPath = new String[2];
		String[] arrayOfIpAndExportPath = new String[2];
		try {
			final Pattern IPV4 = Pattern.compile(IPV4_REGEX_PATTERN);
			final Pattern IPV6 = Pattern.compile(IPV6_REGEX_PATTERN);
			final Pattern SEPERATOR = Pattern.compile(IP_AND_VOLUME_SEPERATOR);
			Matcher matcherIPV6 = IPV6.matcher(mountPath);
			Matcher matcherIPV4 = IPV4.matcher(mountPath);

			if (matcherIPV6.find()) {
				//Split the the mountpath at last ':' for IPv6
				int index = mountPath.lastIndexOf(":");
				ipAndExportPath[0] = mountPath.substring(0, index);
				ipAndExportPath[1] = mountPath.substring(index);
				arrayOfIpAndExportPath[0] = ipAndExportPath[0];
			} else if (matcherIPV4.find()) {
				ipAndExportPath = mountPath.split(matcherIPV4.group());
				arrayOfIpAndExportPath[0] = matcherIPV4.group();
			} else {
				// mounted with FQDN

				arrayOfIpAndExportPath = mountPath
						.split(SEPERATOR.pattern(), 2);
				if (arrayOfIpAndExportPath[0] == null
						|| arrayOfIpAndExportPath[1] == null) {
					CLIUtils.writeToLog("FQDN is configured, failed to split the export path and data lif for "+ mountPath,
		                    logger, CLIUtils.LogLevel.ERROR);
					throw new IllegalArgumentException(MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_OPTION_SECONDARY_LOCATOR_INVALID_FORMAT));
		
				}
				return arrayOfIpAndExportPath;

			}

			String[] removeSeperator = ipAndExportPath[1].split(
					SEPERATOR.pattern(), 2);
			arrayOfIpAndExportPath[1] = removeSeperator[1];
			if (arrayOfIpAndExportPath[0] == null
					|| arrayOfIpAndExportPath[1] == null) {
				CLIUtils.writeToLog("Failed to split the export path and data lif for "+ mountPath,
	                    logger, CLIUtils.LogLevel.ERROR);
				throw new IllegalArgumentException(MessageBundle
                        .localizeMessage(CLIMsgKey.CLI_OPTION_SECONDARY_LOCATOR_INVALID_FORMAT));
			}

		} catch (IllegalArgumentException ex) {
			throw new IllegalArgumentException(ex.getMessage());

		}
		return arrayOfIpAndExportPath;
	}
}
